@@ -1,29 +1,8 @@
-/* app_getvcp.c
- *
- * <copyright>
- * Copyright (C) 2014-2017 Sanford Rockowitz <rockowitz@minsoft.com>
- *
- * Licensed under the GNU General Public License Version 2
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- * </endcopyright>
+/** @file app_getvcp.c
  */
 
-/** \file
- *
- */
+// Copyright (C) 2014-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// SPDX-License-Identifier: GPL-2.0-or-later
 
 /** \cond */
 #include <config.h>
@@ -51,6 +30,8 @@
 
 #include "vcp/vcp_feature_codes.h"
 
+#include "dynvcp/dyn_feature_codes.h"
+
 #include "i2c/i2c_bus_core.h"
 
 #include "ddc/ddc_output.h"
@@ -61,6 +42,10 @@
 #include "app_ddcutil/app_getvcp.h"
 
 
+// Default trace class for this file
+static DDCA_Trace_Group TRACE_GROUP = DDCA_TRC_TOP;
+
+#ifdef OLD
 /* Shows a single VCP value specified by its feature table entry.
  *
  * Arguments:
@@ -69,7 +54,7 @@
  *
  * Returns:
  *    status code 0 = normal
- *                DDCL_INVALID_OPERATION - feature is deprecated or write-only
+ *                DDCRC_INVALID_OPERATION - feature is deprecated or write-only
  *                from get_formatted_value_for_feature_table_entry()
  */
 Public_Status_Code
@@ -78,12 +63,12 @@ app_show_single_vcp_value_by_feature_table_entry(
       VCP_Feature_Table_Entry *  entry)
 {
    bool debug = false;
-   DBGMSF(debug, "Starting. Getting feature 0x%02x for %s",
-                 entry->code, dh_repr(dh) );
+   DBGTRC(debug, TRACE_GROUP,
+         "Starting. Getting feature 0x%02x for %s", entry->code, dh_repr(dh) );
 
    DDCA_MCCS_Version_Spec vspec      = get_vcp_version_by_display_handle(dh);
    Public_Status_Code     psc        = 0;
-   Byte                   feature_id = entry->code;
+   DDCA_Vcp_Feature_Code  feature_id = entry->code;
 
    if (!is_feature_readable_by_vcp_version(entry, vspec)) {
       char * feature_name =  get_version_sensitive_feature_name(entry, vspec);
@@ -93,7 +78,7 @@ app_show_single_vcp_value_by_feature_table_entry(
                 feature_id, feature_name, vspec.major, vspec.minor);
       else
          printf("Feature %02x (%s) is not readable\n", feature_id, feature_name);
-      psc = DDCL_INVALID_OPERATION;
+      psc = DDCRC_INVALID_OPERATION;
    }
 
    if (psc == 0) {
@@ -111,50 +96,103 @@ app_show_single_vcp_value_by_feature_table_entry(
       }
    }
 
-   DBGMSF(debug, "Done.  Returning: %s", psc_desc(psc));
+   DBGTRC(debug, TRACE_GROUP, "Done.  Returning: %s", psc_desc(psc));
    return psc;
 }
+#endif
 
 
-/* Shows a single VCP value specified by its feature id.
+/* Shows a single VCP value specified by its #Display_Feature_Metadata
  *
  * Arguments:
  *    dh          handle of open display
- *    feature_id  hex feature id
- *    force       attempt to show value even if feature_id not in feature table
+ *    meta        feature metadata
  *
  * Returns:
- *    status code 0 = success
- *                DDCL_UNKNOWN_FEATURE  feature_id not in feature table and !force
- *                from app_show_single_vcp_value_by_feature_table_entry()
+ *    status code 0 = normal
+ *                DDCRC_INVALID_OPERATION - feature is deprecated or write-only
+ *                from get_formatted_value_for_feature_table_entry()
  */
-Public_Status_Code
-app_show_single_vcp_value_by_feature_id(
-      Display_Handle * dh,
-      Byte feature_id,
-      bool force)
+DDCA_Status
+app_show_single_vcp_value_by_dfm(
+      Display_Handle *             dh,
+      Display_Feature_Metadata *  dfm)
 {
    bool debug = false;
-   DBGMSF(debug, "Starting. Getting feature 0x%02x for %s, force=%s",
-                 feature_id, dh_repr(dh), bool_repr(force) );
+   DBGTRC(debug, TRACE_GROUP, "Starting. Getting feature 0x%02x for %s",
+                               dfm->feature_code, dh_repr(dh) );
+
+   // DDCA_Feature_Metadata * extmeta = dfm_to_ddca_feature_metadata(meta);
+
+   DDCA_MCCS_Version_Spec vspec      = get_vcp_version_by_display_handle(dh);
+   DDCA_Status            ddcrc      = 0;
+   DDCA_Vcp_Feature_Code  feature_id = dfm->feature_code;
+
+   if (!(dfm->feature_flags & DDCA_READABLE)) {
+      char * feature_name =  dfm->feature_name;
+
+      DDCA_Feature_Flags vflags = dfm->feature_flags;
+      // should get vcp version from metadata
+      if (vflags & DDCA_DEPRECATED)
+         printf("Feature %02x (%s) is deprecated in MCCS %d.%d\n",
+                feature_id, feature_name, vspec.major, vspec.minor);
+      else
+         printf("Feature %02x (%s) is not readable\n", feature_id, feature_name);
+      ddcrc = DDCRC_INVALID_OPERATION;
+   }
+
+   if (ddcrc == 0) {
+      char * formatted_value = NULL;
+      ddcrc = ddc_get_formatted_value_for_display_feature_metadata(
+               dh,
+               dfm,
+               false,      /* suppress_unsupported */
+               true,       /* prefix_value_with_feature_code */
+               &formatted_value,
+               stdout);    /* msg_fh */
+      if (formatted_value) {
+         printf("%s\n", formatted_value);
+         free(formatted_value);
+      }
+   }
+
+   DBGTRC(debug, TRACE_GROUP, "Done.  Returning: %s", psc_desc(ddcrc));
+   return ddcrc;
+}
+
+
+Public_Status_Code
+app_show_single_vcp_value_by_feature_id_new_dfm(
+      Display_Handle *      dh,
+      DDCA_Vcp_Feature_Code feature_id,
+      bool                  force)
+{
+   bool debug = false;
+   DBGTRC(debug, TRACE_GROUP, "Starting. Getting feature 0x%02x for %s, force=%s",
+                              feature_id, dh_repr(dh), bool_repr(force) );
 
    Public_Status_Code         psc = 0;
-   VCP_Feature_Table_Entry *  entry = NULL;
 
-   entry = vcp_find_feature_by_hexid(feature_id);
-   if (!entry && (force || feature_id >= 0xe0)) {    // don't require force if manufacturer specific code
-      entry = vcp_create_dummy_feature_for_hexid(feature_id);
-   }
-   if (!entry) {
+   Display_Feature_Metadata * dfm =
+   dyn_get_feature_metadata_by_dh_dfm(
+         feature_id,
+         dh,
+         force || feature_id >= 0xe0    // with_default
+         );
+
+   // VCP_Feature_Table_Entry *  entry = NULL;
+
+
+   if (!dfm) {
       printf("Unrecognized VCP feature code: 0x%02x\n", feature_id);
-      // gsc = modulate_rc(-EINVAL, RR_ERRNO);
-      psc = DDCL_UNKNOWN_FEATURE;
+      psc = DDCRC_UNKNOWN_FEATURE;
    }
    else {
-      psc = app_show_single_vcp_value_by_feature_table_entry(dh, entry);
+      psc = app_show_single_vcp_value_by_dfm(dh, dfm);
+      dfm_free(dfm);
    }
 
-   DBGMSF(debug, "Done.  Returning: %s", psc_desc(psc));
+   DBGTRC(debug, TRACE_GROUP, "Done.  Returning: %s", psc_desc(psc));
    return psc;
 }
 
@@ -164,7 +202,7 @@ app_show_single_vcp_value_by_feature_id(
  * Arguments:
  *    dh                display handle
  *    subset_id         feature subset
- *    show_unsupported  report unsupported values
+ *    flags             option flags
  *    features_seen     if non-null, collect list of features found
  *
  * Returns:
@@ -174,14 +212,27 @@ Public_Status_Code
 app_show_vcp_subset_values_by_display_handle(
         Display_Handle *    dh,
         VCP_Feature_Subset  subset_id,
- //        bool                show_unsupported,  // deprecated
         Feature_Set_Flags   flags,
         Byte_Bit_Flags      features_seen)
 {
-   // DBGMSG("Starting.  subset=%d   ", subset );
+   bool debug = false;
+   DBGTRC(debug, TRACE_GROUP,
+          "Starting. dh=%s, subset_id=%s, flags=%s, features_seen=%p",
+          dh_repr(dh), feature_subset_name(subset_id), feature_set_flag_names_t(flags), features_seen );
 
    GPtrArray * collector = NULL;
-   Public_Status_Code psc = show_vcp_values(dh, subset_id, collector, flags, features_seen);
+   Public_Status_Code psc = ddc_show_vcp_values(dh, subset_id, collector, flags, features_seen);
+
+   if (debug || IS_TRACING()) {
+      if (features_seen) {
+        char * s = bbf_to_string(features_seen, NULL, 0);
+        DBGMSG("Returning: %s. features_seen=%s",  psc_desc(psc), s);
+        free(s);
+      }
+      else {
+         DBGMSG("Returning: %s", psc_desc(psc));
+      }
+   }
    return psc;
 }
 
@@ -209,7 +260,7 @@ void app_show_vcp_subset_values_by_display_ref(
    if (dref->ddc_io_mode == DDC_IO_DEVI2C) {
       // Is this needed?  or checked by openDisplay?
       Bus_Info * bus_info = i2c_get_bus_info(dref->busno);
-      if (!bus_info ||  !(bus_info->flags & I2C_BUS_ADDR_0X37) ) {
+      if (!bus_info ||  !(bus_info->feature_flags & I2C_BUS_ADDR_0X37) ) {
          printf("Address 0x37 not detected on bus %d. I2C communication not available.\n", dref->busno );
          validDisp = false;
       }
@@ -221,7 +272,7 @@ void app_show_vcp_subset_values_by_display_ref(
    if (validDisp) {
       GPtrArray * collector = NULL;
       Display_Handle * pDispHandle = ddc_open_display(dref, EXIT_IF_FAILURE);
-      show_vcp_values(pDispHandle, subset_id, collector, show_unsupported);
+      ddc_show_vcp_values(pDispHandle, subset_id, collector, show_unsupported);
       ddc_close_display(pDispHandle);
    }
 }
@@ -233,8 +284,7 @@ void app_show_vcp_subset_values_by_display_ref(
  * Arguments:
  *    dh                display handle
  *    fsref             feature set reference
- *    show_unsupported  report unsupported values (applies if not a single feature feature set)
- *    force             applies if is a single feature feature set
+ *    flags             option flags
  *
  * Returns:
  *    status code       from app_show_single_vcp_value_by_feature_id() or
@@ -244,37 +294,29 @@ Public_Status_Code
 app_show_feature_set_values_by_display_handle(
       Display_Handle *     dh,
       Feature_Set_Ref *    fsref,
-//       bool                 show_unsupported,
-//       bool                 force,
-      Feature_Set_Flags    flags
-     )
+      Feature_Set_Flags    flags)
 {
    bool debug = false;
-   // bool show_unsupported = flags & FSF_SHOW_UNSUPPORTED;
-   if (debug) {
-      char * s0 = feature_set_flag_names(flags);
-      DBGMSG("Starting. flags: %s, dh: %s", s0, dh_repr(dh));
-      dbgrpt_feature_set_ref(fsref,1);
-      free(s0);
+   if (debug || IS_TRACING()) {
+      char * s0 = feature_set_flag_names_t(flags);
+      DBGMSG("Starting. dh: %s. fsref: %s, flags: %s", dh_repr(dh), fsref_repr_t(fsref), s0);
+      // dbgrpt_feature_set_ref(fsref,1);
    }
 
    Public_Status_Code psc = 0;
    if (fsref->subset == VCP_SUBSET_SINGLE_FEATURE) {
-#ifdef OLD
-      psc = app_show_single_vcp_value_by_feature_id(
-            dh, fsref->specific_feature, force);
-#endif
-      psc = app_show_single_vcp_value_by_feature_id(
+      psc = app_show_single_vcp_value_by_feature_id_new_dfm(
             dh, fsref->specific_feature, flags&FSF_FORCE);
    }
    else {
       psc = app_show_vcp_subset_values_by_display_handle(
             dh,
             fsref->subset,
-   //      show_unsupported,
             flags,
             NULL);
    }
+
+   DBGTRC(debug, TRACE_GROUP, "Returning: %s", psc_desc(psc));
    return psc;
 }
 
@@ -284,7 +326,7 @@ app_show_feature_set_values_by_display_handle(
 //
 
 void reset_vcp_x02(Display_Handle * dh) {
-   Error_Info * ddc_excp = set_nontable_vcp_value(dh, 0x02, 0x01);
+   Error_Info * ddc_excp = ddc_set_nontable_vcp_value(dh, 0x02, 0x01);
    if (ddc_excp) {
       DBGMSG("set_nontable_vcp_value_by_display_handle() returned %s", errinfo_summary(ddc_excp) );
       errinfo_free(ddc_excp);
@@ -294,11 +336,11 @@ void reset_vcp_x02(Display_Handle * dh) {
 }
 
 bool new_control_values_exist(Display_Handle * dh) {
-   bool debug = true;
+   bool debug = false;
    Parsed_Nontable_Vcp_Response * p_nontable_response = NULL;
    // DBGMSF(debug, "VCP version: %d.%d", vspec.major, vspec.minor);
    bool result = false;
-    Error_Info * ddc_excp = get_nontable_vcp_value(
+    Error_Info * ddc_excp = ddc_get_nontable_vcp_value(
              dh,
              0x02,
              &p_nontable_response);
@@ -331,7 +373,7 @@ bool new_control_values_exist(Display_Handle * dh) {
 Byte show_changed_feature(Display_Handle * dh) {
    Parsed_Nontable_Vcp_Response * p_nontable_response = NULL;
    Byte changed_feature = 0x00;
-   Error_Info * ddc_excp = get_nontable_vcp_value(
+   Error_Info * ddc_excp = ddc_get_nontable_vcp_value(
               dh,
               0x52,
               &p_nontable_response);
@@ -344,7 +386,7 @@ Byte show_changed_feature(Display_Handle * dh) {
      changed_feature = p_nontable_response->sl;
      free(p_nontable_response);
      if (changed_feature)
-        app_show_single_vcp_value_by_feature_id(dh, changed_feature, false);
+        app_show_single_vcp_value_by_feature_id_new_dfm(dh, changed_feature, false);
   }
   return changed_feature;
 }
@@ -384,7 +426,7 @@ app_read_changes(Display_Handle * dh) {
    DDCA_MCCS_Version_Spec vspec = get_vcp_version_by_display_handle(dh);
 
    if (new_control_values_exist(dh)) {
-      if ( vcp_version_le(vspec, VCP_SPEC_V21) ) {
+      if ( vcp_version_le(vspec, DDCA_VSPEC_V21) ) {
          show_changed_feature(dh);
       }
       else {  // x52 is a FIFO
@@ -402,7 +444,6 @@ app_read_changes(Display_Handle * dh) {
       }
       reset_vcp_x02(dh);
    }
-
 }
 
 
@@ -457,7 +498,7 @@ app_read_changes_usb(Display_Handle * dh) {
  */
 void
 app_read_changes_forever(Display_Handle * dh) {
-   bool debug = true;
+   bool debug = false;
 
    printf("Watching for VCP feature changes on display %s\n", dh_repr(dh));
    printf("Type ^C to exit...\n");
