@@ -1,37 +1,18 @@
-/* main.c
+/** @file main.c
  *
- * Program mainline
- *
- * <copyright>
- * Copyright (C) 2014-2017 Sanford Rockowitz <rockowitz@minsoft.com>
- *
- * Licensed under the GNU General Public License Version 2
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- * </endcopyright>
+ *  ddcutil standalone application mainline
  */
 
-/** \file
- * ddcutil standalone application mainline
- */
+// Copyright (C) 2014-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// SPDX-License-Identifier: GPL-2.0-or-later
 
 /** \cond */
 #include <config.h>
 
 #include <assert.h>
 #include <ctype.h>
+#include <dynvcp/dyn_dynamic_features.h>
+#include <dynvcp/dyn_parsed_capabilities.h>
 #include <errno.h>
 #include <setjmp.h>
 #include <stdbool.h>
@@ -43,9 +24,12 @@
 #include "util/data_structures.h"
 #include "util/error_info.h"
 #include "util/failsim.h"
+#include "util/report_util.h"
 #include "util/sysfs_util.h"
 /** \endcond */
 
+#include "public/ddcutil_types.h"
+
 #include "base/adl_errors.h"
 #include "base/base_init.h"
 #include "base/core.h"
@@ -60,6 +44,8 @@
 #include "vcp/parse_capabilities.h"
 #include "vcp/vcp_feature_codes.h"
 
+#include "dynvcp/dyn_dynamic_features.h"
+
 #include "i2c/i2c_bus_core.h"
 #include "i2c/i2c_do_io.h"
 
@@ -68,7 +54,6 @@
 #include "usb/usb_displays.h"
 
 #include "ddc/ddc_displays.h"
-#include "ddc/ddc_dynamic_features.h"
 #include "ddc/ddc_multi_part_io.h"
 #include "ddc/ddc_output.h"
 #include "ddc/ddc_packet_io.h"
@@ -82,8 +67,7 @@
 #include "cmdline/cmd_parser.h"
 #include "cmdline/parsed_cmd.h"
 
-#include "app_common/app_dynamic_features.h"    // future
-
+#include "app_ddcutil/app_dynamic_features.h"
 #include "app_ddcutil/app_dumpload.h"
 #include "app_ddcutil/app_getvcp.h"
 #include "app_ddcutil/app_setvcp.h"
@@ -102,6 +86,10 @@
 #endif
 
 
+// Default race class for this file
+static DDCA_Trace_Group TRACE_GROUP = DDCA_TRC_TOP;
+
+
 //
 // Initialization and Statistics
 //
@@ -181,9 +169,10 @@ perform_get_capabilities_by_display_handle(Display_Handle * dh) {
          if (dh->dref->io_path.io_mode == DDCA_IO_USB)
             pcap->raw_value_synthesized = true;
          // report_parsed_capabilities(pcap, dh->dref->io_path.io_mode);    // io_mode no longer needed
-         report_parsed_capabilities(
+         dyn_report_parsed_capabilities(
                pcap,
-               dh->dref->mmid,
+               dh,
+               NULL,
                0);
          // free_parsed_capabilities(pcap);
       }
@@ -212,7 +201,10 @@ void probe_display_by_dh(Display_Handle * dh)
    // }
 
    // reports capabilities, and if successful returns Parsed_Capabilities
+   DDCA_Output_Level saved_ol = get_output_level();
+   set_output_level(DDCA_OL_VERBOSE);
    Parsed_Capabilities * pcaps = perform_get_capabilities_by_display_handle(dh);
+   set_output_level(saved_ol);
 
    // how to pass this information down into app_show_vcp_subset_values_by_display_handle()?
    bool table_reads_possible = parsed_capabilities_may_support_table_commands(pcaps);
@@ -248,11 +240,23 @@ void probe_display_by_dh(Display_Handle * dh)
          for (int code = 0; code < 256; code++) {
             if (bbf_is_set(caps_not_seen, code)) {
                VCP_Feature_Table_Entry * vfte = vcp_find_feature_by_hexid_w_default(code);
+               Display_Feature_Metadata * dfm =
+                     dyn_get_feature_metadata_by_dh_dfm(
+                        code,
+                         dh,
+                         true);   //  with_default
                char * feature_name = get_version_sensitive_feature_name(vfte, pcaps->parsed_mccs_version);
+               if (!streq(feature_name, dfm->feature_name)) {
+                  rpt_vstring(1, "VCP_Feature_Table_Entry feature name: %s", feature_name);
+                  rpt_vstring(1, "Display_Feature_Metadata feature name: %s",
+                                 dfm->feature_name);
+               }
+               // assert( streq(feature_name, ifm->external_metadata->feature_name));
                f0printf(fout, "   Feature x%02x - %s\n", code, feature_name);
-               if (vfte->vcp_global_flags & DDCA_SYNTHETIC) {
+               if (vfte->vcp_global_flags & DDCA_SYNTHETIC_VCP_FEATURE_TABLE_ENTRY) {
                   free_synthetic_vcp_entry(vfte);
                }
+               // need to free ifm?
             }
          }
       }
@@ -264,11 +268,24 @@ void probe_display_by_dh(Display_Handle * dh)
          for (int code = 0; code < 256; code++) {
             if (bbf_is_set(seen_not_caps, code)) {
                VCP_Feature_Table_Entry * vfte = vcp_find_feature_by_hexid_w_default(code);
+
+               Display_Feature_Metadata * dfm =
+                     dyn_get_feature_metadata_by_dh_dfm(
+                        code,
+                         dh,
+                         true);   //  with_default
                char * feature_name = get_version_sensitive_feature_name(vfte, vspec);
                f0printf(fout, "   Feature x%02x - %s\n", code, feature_name);
-               if (vfte->vcp_global_flags & DDCA_SYNTHETIC) {
+               if (!streq(feature_name, dfm->feature_name)) {
+                  rpt_vstring(1, "VCP_Feature_Table_Entry feature name: %s", feature_name);
+                  rpt_vstring(1, "Internal_Feature_Metadata feature name: %s",
+                                 dfm->feature_name);
+               }
+               // assert( streq(feature_name, ifm->external_metadata->feature_name));
+               if (vfte->vcp_global_flags & DDCA_SYNTHETIC_VCP_FEATURE_TABLE_ENTRY) {
                   free_synthetic_vcp_entry(vfte);
                }
+               // free ifm
             }
          }
       }
@@ -289,7 +306,7 @@ void probe_display_by_dh(Display_Handle * dh)
 
    puts("");
    // get VCP 0B
-   Single_Vcp_Value * valrec;
+   DDCA_Any_Vcp_Value * valrec;
    int color_temp_increment = 0;
    int color_temp_units = 0;
    ddc_excp = ddc_get_vcp_value(
@@ -301,7 +318,7 @@ void probe_display_by_dh(Display_Handle * dh)
    if (psc == 0) {
       if (debug)
          f0printf(fout, "Value returned for feature x0b: %s\n", summarize_single_vcp_value(valrec) );
-      color_temp_increment = valrec->val.c.cur_val;
+      color_temp_increment = valrec->val.c_nc.sl;
       free_single_vcp_value(valrec);
 
       ddc_excp = ddc_get_vcp_value(
@@ -313,8 +330,7 @@ void probe_display_by_dh(Display_Handle * dh)
       if (psc == 0) {
          if (debug)
             f0printf(fout, "Value returned for feature x0c: %s\n", summarize_single_vcp_value(valrec) );
-         color_temp_units = valrec->val.c.cur_val;
-         free_single_vcp_value(valrec);
+         color_temp_units = valrec->val.c_nc.sl;
          int color_temp = 3000 + color_temp_units * color_temp_increment;
          f0printf(fout, "Color temperature increment (x0b) = %d degrees Kelvin\n", color_temp_increment);
          f0printf(fout, "Color temperature request   (x0c) = %d\n", color_temp_units);
@@ -369,28 +385,18 @@ int main(int argc, char *argv[]) {
    bool main_debug = false;
    int main_rc = EXIT_FAILURE;
 
-#ifdef OBSOLETE
-   // For aborting out of shared library
-   jmp_buf abort_buf;
-   int jmprc = setjmp(abort_buf);
-   if (jmprc) {
-      fprintf(stderr, "Aborting. Internal status code = %d\n", jmprc);
-      goto bye;       // main_rc == EXIT_FAILURE
-   }
-
-   register_jmp_buf(&abort_buf);
-#endif
-
    // set_trace_levels(TRC_ADL);   // uncomment to enable tracing during initialization
    init_base_services();  // so tracing related modules are initialized
    Parsed_Cmd * parsed_cmd = parse_command(argc, argv);
    if (!parsed_cmd) {
       goto bye;      // main_rc == EXIT_FAILURE
    }
+
+   // configure tracing
    if (parsed_cmd->flags & CMD_FLAG_TIMESTAMP_TRACE)         // timestamps on debug and trace messages?
       dbgtrc_show_time = true;              // extern in core.h
    report_freed_exceptions = parsed_cmd->flags & CMD_FLAG_REPORT_FREED_EXCP;   // extern in core.h
-   set_trace_levels(parsed_cmd->trace);
+   set_trace_levels(parsed_cmd->traced_groups);
    if (parsed_cmd->traced_functions) {
       for (int ndx = 0; ndx < ntsa_length(parsed_cmd->traced_functions); ndx++)
          add_traced_function(parsed_cmd->traced_functions[ndx]);
@@ -399,6 +405,7 @@ int main(int argc, char *argv[]) {
       for (int ndx = 0; ndx < ntsa_length(parsed_cmd->traced_files); ndx++)
          add_traced_file(parsed_cmd->traced_files[ndx]);
    }
+
 #ifdef ENABLE_FAILSIM
    fsim_set_name_to_number_funcs(
          status_name_to_modulated_number,
@@ -414,6 +421,9 @@ int main(int argc, char *argv[]) {
    }
 #endif
 
+   // global variable in dyn_dynamic_features:
+   enable_dynamic_features = parsed_cmd->flags & CMD_FLAG_ENABLE_UDF;
+
    init_ddc_services();  // n. initializes start timestamp
    // overrides setting in init_ddc_services():
    i2c_set_io_strategy(DEFAULT_I2C_IO_STRATEGY);
@@ -444,6 +454,11 @@ int main(int argc, char *argv[]) {
                 0,"",
                 28, "Force I2C slave address:",
                 bool_repr(i2c_force_slave_addr_flag));
+      f0printf( fout, "%.*s%-*s%s\n",
+                0,"",
+                28, "User defined features:",
+                (enable_dynamic_features) ? "enabled" : "disabled" );  // "Enable user defined features" is too long a title
+                // bool_repr(enable_dynamic_features));
       f0puts("\n", fout);
    }
 
@@ -476,12 +491,20 @@ int main(int argc, char *argv[]) {
    if (parsed_cmd->sleep_strategy >= 0)
       set_sleep_strategy(parsed_cmd->sleep_strategy);
 
+#ifdef USE_USB
+   // if ( !(parsed_cmd->flags & CMD_FLAG_ENABLE_USB)) {
+      DDCA_Status rc = ddc_enable_usb_display_detection( parsed_cmd->flags & CMD_FLAG_ENABLE_USB );
+      assert (rc == DDCRC_OK);
+   // }
+#endif
+
    int threshold = DISPLAY_CHECK_ASYNC_NEVER;
    if (parsed_cmd->flags & CMD_FLAG_ASYNC)
       threshold = DISPLAY_CHECK_ASYNC_THRESHOLD;
    ddc_set_async_threshold(threshold);
 
    main_rc = EXIT_SUCCESS;     // from now on assume success;
+   DBGTRC(main_debug, TRACE_GROUP, "Initialization complete, process commands");
 
    if (parsed_cmd->cmd_id == CMDID_LISTVCP) {
       vcp_list_feature_codes(stdout);
@@ -501,7 +524,7 @@ int main(int argc, char *argv[]) {
       if (parsed_cmd->flags & CMD_FLAG_WO_ONLY)
          flags |= FSF_WO_ONLY;
 
-      VCP_Feature_Set fset = create_feature_set_from_feature_set_ref(
+      VCP_Feature_Set * fset = create_feature_set_from_feature_set_ref(
                                 parsed_cmd->fref,
                                 parsed_cmd->mccs_vspec,
                                 flags);
@@ -535,8 +558,10 @@ int main(int argc, char *argv[]) {
    // start of commands that actually access monitors
 
    else if (parsed_cmd->cmd_id == CMDID_DETECT) {
+      DBGTRC(main_debug, TRACE_GROUP, "Detecting displays...");
       ddc_ensure_displays_detected();
       ddc_report_displays(/*include_invalid_displays=*/ true, 0);
+      DBGTRC(main_debug, TRACE_GROUP, "Display detection complete");
       main_rc = EXIT_SUCCESS;
    }
 
@@ -589,8 +614,10 @@ int main(int argc, char *argv[]) {
    }
 
    else if (parsed_cmd->cmd_id == CMDID_ENVIRONMENT) {
+      DBGTRC(main_debug, TRACE_GROUP, "Processing command ENVIRONMENT...");
       dup2(1,2);   // redirect stderr to stdout
       ddc_ensure_displays_detected();   // *** NEEDED HERE ??? ***
+      DBGTRC(main_debug, TRACE_GROUP, "display detection complete");
 
       f0printf(fout, "The following tests probe the runtime environment using multiple overlapping methods.\n");
       query_sysenv();
@@ -599,8 +626,10 @@ int main(int argc, char *argv[]) {
 
    else if (parsed_cmd->cmd_id == CMDID_USBENV) {
 #ifdef USE_USB
+      DBGTRC(main_debug, TRACE_GROUP, "Processing command USBENV...");
       dup2(1,2);   // redirect stderr to stdout
       ddc_ensure_displays_detected();   // *** NEEDED HERE ??? ***
+      DBGTRC(main_debug, TRACE_GROUP, "display detection complete");
       f0printf(fout, "The following tests probe for USB connected monitors.\n");
       // DBGMSG("Exploring USB runtime environment...\n");
       query_usbenv();
@@ -614,6 +643,7 @@ int main(int argc, char *argv[]) {
    else if (parsed_cmd->cmd_id == CMDID_CHKUSBMON) {
 #ifdef USE_USB
       // DBGMSG("Processing command chkusbmon...\n");
+      DBGTRC(main_debug, TRACE_GROUP, "Processing command CHKUSBMON...");
       bool is_monitor = check_usb_monitor( parsed_cmd->args[0] );
       main_rc = (is_monitor) ? EXIT_SUCCESS : EXIT_FAILURE;
 #else
@@ -623,6 +653,7 @@ int main(int argc, char *argv[]) {
    }
 
    else if (parsed_cmd->cmd_id == CMDID_INTERROGATE) {
+      DBGTRC(main_debug, TRACE_GROUP, "Processing command INTERROGATE...");
       dup2(1,2);   // redirect stderr to stdout
       // set_ferr(fout);    // ensure that all messages are collected - made unnecessary by dup2()
       f0printf(fout, "Setting output level verbose...\n");
@@ -637,6 +668,7 @@ int main(int argc, char *argv[]) {
       ddc_set_max_multi_part_read_tries(MAX_MAX_TRIES);
 
       ddc_ensure_displays_detected();    // *** ???
+      DBGTRC(main_debug, TRACE_GROUP, "display detection complete");
 
       query_sysenv();
 #ifdef USE_USB
@@ -684,6 +716,7 @@ int main(int argc, char *argv[]) {
 
    // *** Commands that require Display Identifier ***
    else {
+      DBGTRC(main_debug, TRACE_GROUP, "display identifier supplied");
       if (!parsed_cmd->pdid)
          parsed_cmd->pdid = create_dispno_display_identifier(1);   // default monitor
       // assert(parsed_cmd->pdid);
@@ -724,7 +757,9 @@ int main(int argc, char *argv[]) {
          }
       }
       else {
+         DBGTRC(main_debug, TRACE_GROUP, "Detecting displays...");
          ddc_ensure_displays_detected();
+         DBGTRC(main_debug, TRACE_GROUP, "display detection complete");
          dref = get_display_ref_for_display_identifier(parsed_cmd->pdid, callopts);
       }
 
@@ -750,11 +785,12 @@ int main(int argc, char *argv[]) {
 
             case CMDID_CAPABILITIES:
                {
-                  // check_dynamic_features(dref);  // needed for this command?
+                  check_dynamic_features(dref);
 
                   Parsed_Capabilities * pcaps = perform_get_capabilities_by_display_handle(dh);
                   main_rc = (pcaps) ? EXIT_SUCCESS : EXIT_FAILURE;
-                  free_parsed_capabilities(pcaps);
+                  if (pcaps)
+                     free_parsed_capabilities(pcaps);
                   break;
                }
 
@@ -887,6 +923,6 @@ int main(int argc, char *argv[]) {
    free_parsed_cmd(parsed_cmd);
 
 bye:
-   DBGMSF(main_debug, "Done.  main_rc=%d", main_rc);
+   DBGTRC(main_debug, TRACE_GROUP, "Done.  main_rc=%d", main_rc);
    return main_rc;
 }
