@@ -13,35 +13,14 @@
  * - debug and trace messages
  */
 
-/* core.c
- *
- * <copyright>
- * Copyright (C) 2014-2017 Sanford Rockowitz <rockowitz@minsoft.com>
- *
- * Licensed under the GNU General Public License Version 2
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- * </endcopyright>
- */
+// Copyright (C) 2014-2018 Sanford Rockowitz <rockowitz@minsoft.com>
+// SPDX-License-Identifier: GPL-2.0-or-later
 
 
 //* \cond */
 #include <assert.h>
 #include <glib-2.0/glib.h>
 #include <errno.h>
-#include <setjmp.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -237,84 +216,6 @@ FILE * cur_fout() {
 #endif
 
 
-//
-// Global error return
-//
-
-#ifdef OBSOLETE
-/** @defgroup abnormal_termination Abnormal Termination
- *
- */
-
-static jmp_buf* global_abort_jmp_buf_ptr = NULL;
-
-// It's an error handler.  Do not dynamically allocate
-DDCA_Global_Failure_Information global_failure_information = {0};
-
-/** Registers the longjmp() target with the **ddcutil** library.
- *
- * @param jb  pointer to jmp_buf
- *
- * @ingroup abnormal_termination
- */
-void register_jmp_buf(jmp_buf* jb) {
-   DBGMSG("setting global_abort_jmp_buf_ptr = %p", jb);
-   global_abort_jmp_buf_ptr = jb;
-}
-#endif
-
-
-#ifdef UNUSED
-// currently unused, iftest out to avoid rpmlint complaint about exit from library
-
-/** Primary function for terminating **ddcutil** execution
- * due to an internal error.
- *
- * If a longjump jump buffer has been registered, basic error information
- * is stored in buffer global_failure_information, and longjmp() is called.
- * Otherwise, exit() is called.
- *
- *  @param funcname  function name of error
- *  @param lineno    line number of error
- *  @param fn        file name of error
- *  @param status    status code
- *
- *  @ingroup abnormal_termination
- */
-/* coverity [+kill] avoid coverity memory leak warnings */
-void ddc_abort(
-      const char * funcname,
-      const int    lineno,
-      const char * fn,
-      int          status)
-{
-   show_backtrace(2);
-
-#ifdef OBSOLETE
-   DBGMSG("global_abort_jmp_buf_ptr = %p", global_abort_jmp_buf_ptr);
-   if (global_abort_jmp_buf_ptr) {
-
-      // save failure information in case it's of use at longjmp() return
-      global_failure_information.info_set_fg = true;
-      global_failure_information.status = status;
-      g_strlcpy(global_failure_information.funcname, funcname, sizeof(global_failure_information.funcname));
-      global_failure_information.lineno = lineno;
-      g_strlcpy(global_failure_information.fn, fn, sizeof(global_failure_information.fn));
-
-      longjmp(*global_abort_jmp_buf_ptr, status);
-   }
-   else {
-#endif
-      // no point setting global_failure_information, we're outta here
-      f0puts("Terminating execution.\n", FERR);
-      exit(EXIT_FAILURE);     // or return status?
-#ifdef OBSOLETE
-   }
-#endif
-}
-#endif
-
-
 //
 // Standard call options
 //
@@ -330,29 +231,7 @@ Value_Name_Table callopt_bitname_table2 = {
 };
 
 
-#ifdef OLD
 /** Interprets a **Call_Options** byte as a printable string.
- *  The returned value is valid until the next call of this function.
- *
- *  @param calloptions  **Call_Options** byte
- *
- *  @return interpreted value
- */
-char * interpret_call_options(Call_Options calloptions) {
-   static char * buffer = NULL;
-   if (buffer) {
-      free(buffer);
-      buffer = NULL;
-   }
-   buffer = vnt_interpret_flags(calloptions, callopt_bitname_table2, false, "|");
-   return buffer;
-}
-#endif
-
-
-/** Thread safe version of **interpret_call_options()**.
- *
- *  Interprets a **Call_Options** byte as a printable string.
  *  The returned value is valid until the next call of this function in
  *  the current thread.
  *
@@ -361,18 +240,8 @@ char * interpret_call_options(Call_Options calloptions) {
  *  @return interpreted value
  */
 char * interpret_call_options_t(Call_Options calloptions) {
-   static GPrivate  callopts_buf_key = G_PRIVATE_INIT(g_free);
-
-   char * buf = g_private_get(&callopts_buf_key);
-
-   // GThread * this_thread = g_thread_self();
-   // printf("(%s) this_thread=%p, callopts_buf_key=%p, buf=%p\n",
-   //        __func__, this_thread, &callopts_buf_key, buf);
-
-   if (!buf) {
-      buf = g_new(char, 200);
-      g_private_set(&callopts_buf_key, buf);
-   }
+   static GPrivate  buf_key = G_PRIVATE_INIT(g_free);
+   char * buf = get_thread_fixed_buffer(&buf_key, 100);
 
    char * buftemp = vnt_interpret_flags(calloptions, callopt_bitname_table2, false, "|");
    g_strlcpy(buf, buftemp, 200);
@@ -382,14 +251,6 @@ char * interpret_call_options_t(Call_Options calloptions) {
 }
 
 
-#ifdef UNUSED
-char * interpret_call_options_a(Call_Options calloptions) {
-   char * buffer = vnt_interpret_flags(calloptions, callopt_bitname_table2, false, "|");
-   return buffer;
-}
-#endif
-
-
 // Local definitions and functions shared by all message control categories
 
 #define SHOW_REPORTING_TITLE_START 0
@@ -406,6 +267,7 @@ print_simple_title_value(int    offset_start_to_title,
             offset_start_to_title,"",
             offset_title_start_to_value, title,
             value);
+   fflush(fout());
 }
 
 
@@ -433,6 +295,7 @@ DDCA_Output_Level get_output_level() {
    return settings->output_level;
 }
 
+
 /** Sets the output level.
  *
  * @param newval output level to set
@@ -449,6 +312,7 @@ DDCA_Output_Level set_output_level(DDCA_Output_Level newval) {
    return old_level;
 }
 
+
 /** Gets the printable name of an output level.
  *
  * @param val  output level
@@ -502,15 +366,19 @@ bool dbgtrc_show_time = false;    ///< include elapsed time in debug/trace outpu
 
 static
 Value_Name_Title_Table trace_group_table = {
-      VNT(TRC_BASE, "BASE"),
-      VNT(TRC_I2C, "I2C"),
+      VNT(DDCA_TRC_BASE, "BASE"),
+      VNT(DDCA_TRC_I2C, "I2C"),
 #ifdef HAVE_ADL
       VNT(TRC_ADL, "ADL"),
 #endif
-      VNT(TRC_DDC, "DDC"),
-      VNT(TRC_USB, "USB"),
-      VNT(TRC_TOP, "TOP"),
-      VNT(TRC_ENV, "ENV"),
+      VNT(DDCA_TRC_DDC, "DDC"),
+      VNT(DDCA_TRC_USB, "USB"),
+      VNT(DDCA_TRC_TOP, "TOP"),
+      VNT(DDCA_TRC_ENV, "ENV"),
+      VNT(DDCA_TRC_API, "API"),
+      VNT(DDCA_TRC_UDF, "UDF"),
+      VNT(DDCA_TRC_VCP, "VCP"),
+      VNT(DDCA_TRC_DDCIO, "DDCIO"),
       VNT_END
 };
 const int trace_group_ct = ARRAY_SIZE(trace_group_table)-1;
@@ -525,16 +393,16 @@ const int trace_group_ct = ARRAY_SIZE(trace_group_table)-1;
  *
  *  /ingroup dbgtrace
  */
-Trace_Group trace_class_name_to_value(char * name) {
-   return (Trace_Group) vnt_find_id(
+DDCA_Trace_Group trace_class_name_to_value(char * name) {
+   return (DDCA_Trace_Group) vnt_find_id(
                            trace_group_table,
                            name,
                            true,      // search title field
                            true,      // ignore-case
-                           TRC_NEVER);
+                           DDCA_TRC_NONE);
 }
 
-static Byte trace_levels = TRC_NEVER;   // 0x00
+static DDCA_Trace_Group trace_levels = DDCA_TRC_NONE;   // 0x00
 
 
 /** Specifies the trace groups to be traced.
@@ -543,9 +411,9 @@ static Byte trace_levels = TRC_NEVER;   // 0x00
  *
  * @ingroup dbgtrace
  */
-void set_trace_levels(Trace_Group trace_flags) {
+void set_trace_levels(DDCA_Trace_Group trace_flags) {
    bool debug = false;
-   DBGMSF(debug, "trace_flags=0x%02x\n", trace_flags);
+   DBGMSF(debug, "trace_flags=0x%04x\n", trace_flags);
 
    trace_levels = trace_flags;
 }
@@ -703,7 +571,7 @@ void show_traced_files() {
  * @ingroup dbgtrace
  *
  */
-bool is_tracing(Trace_Group trace_group, const char * filename, const char * funcname) {
+bool is_tracing(DDCA_Trace_Group trace_group, const char * filename, const char * funcname) {
    bool result =  (trace_group == 0xff) || (trace_levels & trace_group); // is trace_group being traced?
 
    result = result || is_traced_function(funcname) || is_traced_file(filename);
@@ -718,6 +586,7 @@ bool is_tracing(Trace_Group trace_group, const char * filename, const char * fun
  *  Output is written to the current **FOUT** device.
  */
 void show_trace_groups() {
+   // DBGMSG("trace_levels: 0x%04x", trace_levels);
    char * buf = vnt_interpret_flags(trace_levels, trace_group_table, true /* use title */, ", ");
    print_simple_title_value(SHOW_REPORTING_TITLE_START,
                               "Trace groups active: ",
@@ -739,8 +608,6 @@ bool report_freed_exceptions = false;
 // Report DDC data errors
 //
 
-
-
 #ifdef PER_THREAD
 bool enable_report_ddc_errors(bool onoff) {
    Thread_Output_Settings * dests = get_thread_settings();
@@ -789,7 +656,7 @@ bool is_report_ddc_errors_enabled() {
  *  @remark
  *  This function is normally wrapped in function **IS_REPORTING_DDC()**
  */
-bool is_reporting_ddc(Trace_Group trace_group, const char * filename, const char * funcname) {
+bool is_reporting_ddc(DDCA_Trace_Group trace_group, const char * filename, const char * funcname) {
   bool result = (is_tracing(trace_group,filename, funcname) ||
 #ifdef PER_THREAD
         is_report_ddc_errors_enabled()
@@ -814,7 +681,7 @@ bool is_reporting_ddc(Trace_Group trace_group, const char * filename, const char
  *
  * Normally, invocation of this function is wrapped in macro DDCMSG.
  */
-bool ddcmsg(Trace_Group  trace_group,
+bool ddcmsg(DDCA_Trace_Group  trace_group,
             const char * funcname,
             const int    lineno,
             const char * filename,
@@ -833,6 +700,7 @@ bool ddcmsg(Trace_Group  trace_group,
          f0printf(fout(), "(%s) DDC: %s\n", funcname, buffer);
       else
          f0printf(fout(), "DDC: %s\n", buffer);
+      fflush(fout());
       va_end(args);
    }
    return result;
@@ -901,6 +769,7 @@ void severemsg(
       vsnprintf(buffer, 200, format, args);
       snprintf(buf2, 250, "(%s) %s\n", funcname, buffer);
       f0puts(buf2, ferr());
+      fflush(ferr());
       va_end(args);
 }
 
@@ -924,7 +793,7 @@ void severemsg(
  *  @return **true** if message was output, **false** if not
  */
 bool dbgtrc(
-        Trace_Group  trace_group,
+        DDCA_Trace_Group  trace_group,
         const char * funcname,
         const int    lineno,
         const char * filename,
@@ -965,6 +834,7 @@ bool dbgtrc(
       else
          snprintf(buf2, bufsz+60, "(%s) %s\n", funcname, buffer);
       f0puts(buf2, fout());    // no automatic terminating null
+      fflush(fout());
       msg_emitted = true;
    }
 
@@ -977,37 +847,6 @@ bool dbgtrc(
 // error messages and possible program termination.
 //
 
-#ifdef OLD
-/* Report an IOCTL error and possibly terminate execution.
- *
- * Arguments:
- *    errnum         errno value
- *    funcname       function name of error
- *    lineno         line number of error
- *    filename       file name of error
- *    fatal          if true, terminate execution
- *
- *  Returns:         nothing
- */
-void report_ioctl_error_old(
-      int   errnum,
-      const char* funcname,   // const to avoid warning msg on references at compile time
-      int   lineno,
-      char* filename,
-      bool  fatal) {
-   int errsv = errno;
-   f0printf(FERR, "ioctl error in function %s at line %d in file %s: errno=%s\n",
-           funcname, lineno, filename, linux_errno_desc(errnum) );
-   // not worth the linkage issues:
-   // fprintf(stderr, "  %s\n", explain_errno_ioctl(errnum, filedes, request, data));
-   if (fatal) {
-      ddc_abort(funcname, lineno, filename, DDCL_INTERNAL_ERROR);
-   }
-   errno = errsv;
-}
-#endif
-
-
 /** Reports an IOCTL error.
  *  The message is written to the current **FERR** device.
  *
@@ -1027,6 +866,7 @@ void report_ioctl_error(
    int errsv = errno;
    f0printf(ferr(), "(%s) Error in ioctl(%s), errno=%s\n",
            funcname, ioctl_name, linux_errno_desc(errnum) );
+   fflush(ferr());
    errno = errsv;
 }
 
@@ -1068,96 +908,18 @@ void program_logic_error(
   f0puts(buf2,   f);
   f0puts(buffer, f);
   f0puts("\n",   f);
+  fflush(f);
 }
 
 
-#ifdef UNUSED
-/** Called when a condition that should be impossible has been detected.
- * Issues messages to **stderr** and terminates execution.
+//
+// DDCA_Error_Detail related functions
+//
+
+/** Frees a #DDCA_Error_Detail instance
  *
- * This function is normally invoked using macro PROGRAM_LOGIC_ERROR()
- *
- *  @param  funcname    function name
- *  @param  lineno      line number in source file
- *  @param  fn          source file name
- *  @param  format      format string, as in printf()
- *  @param  ...         one or more substitution values for the format string
- *
- * @ingroup output_redirection
+ *  @param  instance to free
  */
-void program_logic_error_fatal(
-      const char * funcname,
-      const int    lineno,
-      const char * fn,
-      char *       format,
-      ...)
-{
-  // assemble the error message
-  char buffer[200];
-  va_list(args);
-  va_start(args, format);
-  vsnprintf(buffer, 200, format, args);
-
-  // assemble the location message:
-  char buf2[250];
-  snprintf(buf2, 250, "Program logic error in function %s at line %d in file %s:\n",
-                      funcname, lineno, fn);
-
-  // don't combine into 1 line, might be very long.  just output 2 lines:
-  f0puts(buf2,   FERR);
-  f0puts(buffer, FERR);
-  f0puts("\n",   FERR);
-
-  // fputs("Terminating execution.\n", stderr);
-  ddc_abort(funcname, lineno, fn, DDCL_INTERNAL_ERROR);
-}
-#endif
-
-
-
-#ifdef UNUSED
-/** This function is called to terminate execution on a fatal error.
- *
- *  It is normally wrapped in macro TERMINATE_EXECUTION_ON_ERROR(format,...)
- *
- *  @param  trace_group trace group for function where error occurred
- *  @param  funcname    function name
- *  @param  lineno      line number
- *  @param  filename    file name
- *  @param  format      printf() style format string
- *  @param  ...         arguments for format string
- *
- *  @ingroup output_redirection
- */
-void terminate_execution_on_error(
-        Trace_Group   trace_group,
-        const char * funcname,
-        const int    lineno,
-        const char * filename,
-        char *       format,
-        ...)
-{
-   char buffer[200];
-   char buf2[250];
-   char * finalBuffer = buffer;
-   va_list(args);
-   va_start(args, format);
-   vsnprintf(buffer, 200, format, args);
-
-   if ( is_tracing(trace_group, filename, funcname) ) {
-      snprintf(buf2, 250, "(%s) %s", funcname, buffer);
-      finalBuffer = buf2;
-   }
-
-   f0puts(finalBuffer, FERR);
-   f0puts("\n", FERR);
-
-   ddc_abort(funcname, lineno, filename, DDCL_INTERNAL_ERROR);
-}
-#endif
-
-
-
 void
 free_error_detail(DDCA_Error_Detail * ddca_erec)
 {
@@ -1172,8 +934,103 @@ free_error_detail(DDCA_Error_Detail * ddca_erec)
    }
 }
 
-// static Thread_Output_Settings *  get_thread_settings()
 
+/** Converts an internal #Error_Info instance to a publicly visible #DDCA_Error_Detail
+ *
+ *  @param  erec  instance to convert
+ *  @return new #DDCA_Error_Detail instance
+ */
+DDCA_Error_Detail *
+error_info_to_ddca_detail(Error_Info * erec)
+{
+   bool debug = false;
+   DBGMSF(debug, "Starting. erec=%p", erec);
+   if (debug)
+      errinfo_report(erec, 2);
+
+   DDCA_Error_Detail * result = NULL;
+   if (erec) {
+      // ???
+      int reqd_size = sizeof(DDCA_Error_Detail) + erec->cause_ct * sizeof(DDCA_Error_Detail*);
+      result = calloc(1, reqd_size);
+      memcpy(result->marker, DDCA_ERROR_DETAIL_MARKER, 4);
+      result->status_code = erec->status_code;
+      if (erec->detail)
+         result->detail = strdup(erec->detail);
+      for (int ndx = 0; ndx < erec->cause_ct; ndx++) {
+         DDCA_Error_Detail * cause = error_info_to_ddca_detail(erec->causes[ndx]);
+         result->causes[ndx] = cause;
+      }
+      result->cause_ct = erec->cause_ct;
+   }
+
+   DBGMSF(debug, "Done. Returning: %p", result);
+   if (debug)
+      report_error_detail(result, 2);
+   return result;
+}
+
+
+/** Makes a deep copy of a #DDC_Error_Detail instance.
+ *
+ *  @param  old  instance to copy
+ *  @return new copy
+ */
+DDCA_Error_Detail *
+dup_error_detail(DDCA_Error_Detail * old) {
+   bool debug = false;
+   DBGMSF(debug, "Starting. old=%p", old);
+   if (debug)
+      report_error_detail(old, 2);
+
+   DDCA_Error_Detail * result = NULL;
+   if (old) {
+      // ???
+      int reqd_size = sizeof(DDCA_Error_Detail) + old->cause_ct * sizeof(DDCA_Error_Detail*);
+      result = calloc(1, reqd_size);
+      memcpy(result->marker, DDCA_ERROR_DETAIL_MARKER, 4);
+      result->status_code = old->status_code;
+      if (old->detail)
+         result->detail = strdup(old->detail);
+      for (int ndx = 0; ndx < old->cause_ct; ndx++) {
+         DDCA_Error_Detail * cause = dup_error_detail(old->causes[ndx]);
+         result->causes[ndx] = cause;
+      }
+      result->cause_ct = old->cause_ct;
+   }
+
+   DBGMSF(debug, "Done. Returning: %p", result);
+   if (debug)
+      report_error_detail(result, 2);
+   return result;
+}
+
+
+/** Emits a detailed report of a #DDCA_Error_Detail struct.
+ *  Output is written to the current report output destination.
+ *
+ *  @param ddca_erec  instance to report
+ *  @param depth      logical indentation depth
+ */
+void report_error_detail(DDCA_Error_Detail * ddca_erec, int depth)
+{
+   if (ddca_erec) {
+      rpt_vstring(depth, "status_code=%s, detail=%s", ddcrc_desc_t(ddca_erec->status_code), ddca_erec->detail);
+      if (ddca_erec->cause_ct > 0) {
+         rpt_label(depth,"Caused by: ");
+         for (int ndx = 0; ndx < ddca_erec->cause_ct; ndx++) {
+            struct ddca_error_detail * cause = ddca_erec->causes[ndx];
+            report_error_detail(cause, depth+1);
+         }
+      }
+   }
+}
+
+
+// Thread-specific functions
+
+/** Frees the #DDCA_Error_Detail (if any) for the current thread.
+ */
 void free_thread_error_detail() {
    Thread_Output_Settings * settings = get_thread_settings();
    if (settings->error_detail) {
@@ -1183,56 +1040,31 @@ void free_thread_error_detail() {
 }
 
 
-DDCA_Error_Detail *
-error_info_to_ddca_detail(Error_Info * erec)
-{
-   DDCA_Error_Detail * result = NULL;
-   if (erec) {
-      // ???
-      int reqd_size = sizeof(DDCA_Error_Detail) + erec->cause_ct * sizeof(DDCA_Error_Detail*);
-      result = calloc(1, reqd_size);
-      memcpy(result->marker, DDCA_ERROR_DETAIL_MARKER, 4);
-      if (erec->detail)
-         result->detail = strdup(erec->detail);
-      for (int ndx = 0; ndx < erec->cause_ct; ndx++) {
-         DDCA_Error_Detail * cause = error_info_to_ddca_detail(erec->causes[ndx]);
-         result->causes[ndx] = cause;
-      }
-   }
-
-   return result;
-}
-
+/** Gets the #DDCA_Error_Detail record for the current thread
+ *
+ *  @return #DDCA_Error_Detail instance, NULL if none
+ */
 DDCA_Error_Detail * get_thread_error_detail() {
    Thread_Output_Settings * settings = get_thread_settings();
    return settings->error_detail;
 }
 
+
+/** Set the #DDCA_Error_Detail record for the current thread.
+ *
+ *  @param error_detail  #DDCA_Error_Detail record to set
+ */
 void save_thread_error_detail(DDCA_Error_Detail * error_detail) {
+   bool debug = false;
+   DBGMSF(debug, "Starting. error_detail=%p", error_detail);
+   if (debug)
+      report_error_detail(error_detail, 2);
+
    Thread_Output_Settings * settings = get_thread_settings();
    if (settings->error_detail)
       free_error_detail(settings->error_detail);
    settings->error_detail = error_detail;
+
+   DBGMSF(debug, "Done");
 }
 
-DDCA_Error_Detail *
-dup_error_detail(DDCA_Error_Detail * old) {
-   DDCA_Error_Detail * result = NULL;
-   if (old) {
-      // ???
-      int reqd_size = sizeof(DDCA_Error_Detail) + old->cause_ct * sizeof(DDCA_Error_Detail*);
-      result = calloc(1, reqd_size);
-      memcpy(result->marker, DDCA_ERROR_DETAIL_MARKER, 4);
-      if (old->detail)
-         result->detail = strdup(old->detail);
-      for (int ndx = 0; ndx < old->cause_ct; ndx++) {
-         DDCA_Error_Detail * cause = dup_error_detail(old->causes[ndx]);
-         result->causes[ndx] = cause;
-      }
-   }
-   return result;
-}
-
-
-
-
