@@ -13,35 +13,14 @@
  * - debug and trace messages
  */
 
-/* core.h
- *
- * <copyright>
- * Copyright (C) 2014-2017 Sanford Rockowitz <rockowitz@minsoft.com>
- *
- * Licensed under the GNU General Public License Version 2
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- * </endcopyright>
- */
+// Copyright (C) 2014-2018 Sanford Rockowitz <rockowitz@minsoft.com>
+// SPDX-License-Identifier: GPL-2.0-or-later
 
 #ifndef BASE_CORE_H_
 #define BASE_CORE_H_
 
 /** \cond */
 #include <linux/limits.h>
-#include <setjmp.h>
 #include <stdarg.h>
 #include <stdbool.h>
 #include <stdio.h>
@@ -51,10 +30,16 @@
 #include "public/ddcutil_types.h"
 
 #include "util/coredefs.h"
+#include "util/error_info.h"
 
 
-// /** @addtogroup abnormal_termination
-//  */
+//
+// Common macros
+//
+
+#define ASSERT_MARKER(_struct_ptr, _marker_value) \
+   assert(_struct_ptr && memcmp(_struct_ptr->marker, _marker_value, 4) == 0)
+
 
 //
 // Initialization
@@ -62,29 +47,6 @@
 void init_msg_control();
 
 
-//
-// For aborting out of shared library
-//
-#ifdef OBSOLETE
-void register_jmp_buf(jmp_buf* jb);
-#endif
-
-#ifdef UNUSED
-void ddc_abort(
-      const char * funcname,
-      const int    lineno,
-      const char * fn,
-      int          status);
-
-#define DDC_ABORT(status) \
-   ddc_abort(__func__, __LINE__, __FILE__, status)
-#endif
-
-#ifdef OBSOLETE
-extern DDCA_Global_Failure_Information global_failure_information;
-#endif
-
-
 //
 // Standard function call arguments and return values
 //
@@ -97,6 +59,7 @@ typedef Byte Call_Options;
 #define CALLOPT_RDONLY       0x20    ///< open read-only
 #define CALLOPT_WARN_FINDEX  0x10    ///< issue warning msg re hiddev_field_info.field_index change
 #define CALLOPT_FORCE        0x08    ///< ignore various validity checks
+#define CALLOPT_WAIT         0x04    ///< wait on locked resources, if false then fail
 
 char * interpret_call_options_t(Call_Options calloptions);
 
@@ -117,21 +80,24 @@ typedef struct {
 // Global redirection for messages that normally go to stdout and stderr,
 // used within functions that are part of the shared library.
 //
-extern FILE * FOUT;
-extern FILE * FERR;
+// extern FILE * FOUT;
+// extern FILE * FERR;
 
 void set_fout(FILE * fout);
 void set_ferr(FILE * ferr);
 void set_fout_to_default();
 void set_ferr_to_default();
 
+FILE * fout();
+FILE * ferr();
+
 
 //
 // Message level control
 //
 
 DDCA_Output_Level get_output_level();
-void              set_output_level(DDCA_Output_Level newval);
+DDCA_Output_Level set_output_level(DDCA_Output_Level newval);
 char *            output_level_name(DDCA_Output_Level val);
 
 
@@ -149,27 +115,12 @@ void add_traced_file(const char * filename);
 bool is_traced_file( const char * filename);
 void show_traced_files();
 
-typedef enum {
- TRC_BASE = 0x80,
- TRC_I2C  = 0x40,
- TRC_ADL  = 0x20,
- TRC_DDC  = 0x10,
- TRC_USB  = 0x08,
- TRC_TOP  = 0x04,
- TRC_ENV  = 0x02,
-
- TRC_NEVER  = 0x00,
- TRC_ALWAYS = 0xff
-} Trace_Group;
-
-#define TRC_NONE TRC_NEVER
-
-Trace_Group trace_class_name_to_value(char * name);
-void set_trace_levels(Trace_Group trace_flags);
-char * get_active_trace_group_names();
+DDCA_Trace_Group trace_class_name_to_value(char * name);
+void set_trace_levels(DDCA_Trace_Group trace_flags);
+// char * get_active_trace_group_names();  // unimplemented
 void show_trace_groups();
 
-bool is_tracing(Trace_Group trace_group, const char * filename, const char * funcname);
+bool is_tracing(DDCA_Trace_Group trace_group, const char * filename, const char * funcname);
 
 /** Checks if tracking is currently active for the globally defined TRACE_GROUP value,
  *  current file and function.
@@ -179,9 +130,9 @@ bool is_tracing(Trace_Group trace_group, const char * filename, const char * fun
  */
 #define IS_TRACING() is_tracing(TRACE_GROUP, __FILE__, __func__)
 
-#define IS_TRACING_GROUP(grp) is_tracing(grp, __FILE__, __func__)
+#define IS_TRACING_GROUP(grp) is_tracing((grp), __FILE__, __func__)
 
-#define IS_TRACING_BY_FUNC_OR_FILE() is_tracing(TRC_NEVER, __FILE__, __func__)
+#define IS_TRACING_BY_FUNC_OR_FILE() is_tracing(DDCA_TRC_NONE, __FILE__, __func__)
 
 
 //
@@ -196,12 +147,17 @@ extern bool report_freed_exceptions;
 //
 
 // Controls display of messages regarding I2C error conditions that can be retried.
-extern bool report_ddc_errors;
+// extern bool report_ddc_errors;
 
-bool is_reporting_ddc(Trace_Group trace_group, const char * filename, const char * funcname);
+// thread specific
+bool enable_report_ddc_errors(bool onoff);
+bool is_report_ddc_errors_enabled();
+
+
+bool is_reporting_ddc(DDCA_Trace_Group trace_group, const char * filename, const char * funcname);
 #define IS_REPORTING_DDC() is_reporting_ddc(TRACE_GROUP, __FILE__, __func__)
 
-bool ddcmsg(Trace_Group trace_group, const char* funcname, const int lineno, const char* fn, char* format, ...);
+bool ddcmsg(DDCA_Trace_Group trace_group, const char* funcname, const int lineno, const char* fn, char* format, ...);
 #define DDCMSG0(format, ...) ddcmsg(TRACE_GROUP, __func__, __LINE__, __FILE__, format, ##__VA_ARGS__)
 
 /** Variant of **DDCMSG** that takes an explicit trace group as an argument.
@@ -226,7 +182,6 @@ bool ddcmsg(Trace_Group trace_group, const char* funcname, const int lineno, con
    ddcmsg(( (debug_flag) ) ? 0xff : (TRACE_GROUP), __func__, __LINE__, __FILE__, format, ##__VA_ARGS__)
 
 
-
 // Show report levels for all types
 void show_reporting();
 
@@ -243,7 +198,7 @@ void severemsg(
         ...);
 
 bool dbgtrc(
-        Trace_Group  trace_group,
+        DDCA_Trace_Group  trace_group,
         const char * funcname,
         const int    lineno,
         const char * fn,
@@ -290,19 +245,20 @@ bool dbgtrc(
     dbgtrc( ( (debug_flag) ) ? 0xff : (trace_group), __func__, __LINE__, __FILE__, format)
 
 
+// typedef (*dbg_struct_func)(void * structptr, int depth);
+#define DBG_RET_STRUCT(_flag, _structname, _dbgfunc, _structptr) \
+if (_flag) { \
+   dbgtrc(0xff, __func__, __LINE__, __FILE__, "Returning %s at %p", #_structname, _structptr); \
+   if (_structptr) { \
+      _dbgfunc(_structptr, 1); \
+   } \
+}
+
+
 //
 // Error handling
 //
 
-#ifdef OLD
-void report_ioctl_error_old(
-      int         errnum,
-      const char* funcname,
-      int         lineno,
-      char*       filename,
-      bool        fatal);
-#endif
-
 void report_ioctl_error(
       const char * ioctl_name,
       int          errnum,
@@ -330,22 +286,13 @@ void program_logic_error(
 #define PROGRAM_LOGIC_ERROR(format, ...) \
    program_logic_error(__func__, __LINE__, __FILE__, format, ##__VA_ARGS__)
 
+DDCA_Error_Detail * error_info_to_ddca_detail(Error_Info * erec);
+DDCA_Error_Detail * dup_error_detail(DDCA_Error_Detail * old);
+void free_error_detail(DDCA_Error_Detail * ddca_erec);
+void report_error_detail(DDCA_Error_Detail * ddca_erec, int depth);
 
-#ifdef OLD
-void terminate_execution_on_error(
-        Trace_Group  trace_group,
-        const char * funcname,
-        const int    lineno,
-        const char * fn,
-        char *       format,
-        ...);
-
-/** @def TERMINATE_EXECUTION_ON_ERROR(format,...)
- *  Wraps call to terminate_execution_on_error()
- * @ingroup abnormal_termination
- */
-#define TERMINATE_EXECUTION_ON_ERROR(format, ...) \
-   terminate_execution_on_error(TRACE_GROUP, __func__, __LINE__, __FILE__, format, ##__VA_ARGS__)
-#endif
+void free_thread_error_detail();
+DDCA_Error_Detail * get_thread_error_detail();
+void save_thread_error_detail(DDCA_Error_Detail * error_detail);
 
 #endif /* BASE_CORE_H_ */
