@@ -1,31 +1,9 @@
-/* ddc_packets.c
- *
+/** \file ddc_packets.c
  * Functions for creating DDC packets and interpreting DDC response packets.
- *
- * <copyright>
- * Copyright (C) 2014-2017 Sanford Rockowitz <rockowitz@minsoft.com>
- *
- * Licensed under the GNU General Public License Version 2
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- * </endcopyright>
  */
 
-/** \file
- * Functions for creating DDC packets and interpreting DDC response packets.
- */
+// Copyright (C) 2014-2018 Sanford Rockowitz <rockowitz@minsoft.com>
+// SPDX-License-Identifier: GPL-2.0-or-later
 
 /** \cond */
 #include <config.h>
@@ -50,7 +28,7 @@
 // Trace control
 //
 
-Trace_Group TRACE_GROUP = TRC_DDC;
+DDCA_Trace_Group TRACE_GROUP = DDCA_TRC_DDCIO;
 
 
 //
@@ -582,7 +560,7 @@ create_ddc_base_response_packet(
       *packet_ptr_addr = NULL;
 
    DBGTRC(debug, TRACE_GROUP,
-          "Returning %s, *packet_ptr_addr=%p", ddcrc_desc(result), *packet_ptr_addr);
+          "Returning %s, *packet_ptr_addr=%p", ddcrc_desc_t(result), *packet_ptr_addr);
    assert( (result==DDCRC_OK && *packet_ptr_addr) || (result != DDCRC_OK && !*packet_ptr_addr));
    return result;
 }
@@ -645,7 +623,7 @@ create_ddc_response_packet(
    }
 
    DBGTRC(debug, TRACE_GROUP,
-          "Returning %s, *packet_ptr_addr=%p", ddcrc_desc(result), *packet_ptr_addr);
+          "Returning %s, *packet_ptr_addr=%p", ddcrc_desc_t(result), *packet_ptr_addr);
    assert( (result==DDCRC_OK && *packet_ptr_addr) || (result != DDCRC_OK && !*packet_ptr_addr));
    return result;
 }
@@ -655,23 +633,7 @@ create_ddc_response_packet(
 // Packet data parsers
 //
 
-// Capabilities response data 
-
-#ifdef OLD
-// deprecated, use more general dbgrpt_interpreted_multi_read_fragment()
-void
-report_interpreted_capabilities(
-      Interpreted_Multi_Part_Read_Fragment * interpreted,
-      int depth)
-{
-   int d1 = depth+1;
-   rpt_vstring(depth, "Capabilities response contents:");
-   rpt_vstring(d1,    "offset:          %d", interpreted->fragment_offset);
-   rpt_vstring(d1,    "fragment length: %d", interpreted->fragment_length);
-   rpt_vstring(d1,    "text:            |%.*s|", interpreted->fragment_length, interpreted->bytes);
-}
-#endif
-
+// Capabilities and table response data
 
 /** Interprets the bytes of a multi part read response.
  *
@@ -716,7 +678,7 @@ interpret_multi_part_read_response(
       // aux_data->text[text_length] = '\0';     // CHANGED
    }
    if (debug)
-      DBGMSG("returning %s", ddcrc_desc(result));
+      DBGMSG("returning %s", ddcrc_desc_t(result));
    return result;
 }
 
@@ -980,7 +942,7 @@ create_ddc_typed_response_packet(
       *packet_ptr_addr = NULL;
    }
 
-   DBGTRC(debug, TRACE_GROUP, "Returning %s, *packet_ptr=%p", ddcrc_desc(rc), *packet_ptr_addr);
+   DBGTRC(debug, TRACE_GROUP, "Returning %s, *packet_ptr=%p", ddcrc_desc_t(rc), *packet_ptr_addr);
    if ( (debug || IS_TRACING()) && rc >= 0)
       dbgrpt_packet(*packet_ptr_addr, 1);
 
@@ -1007,7 +969,7 @@ create_ddc_multi_part_read_response_packet(
                                               &packet);
    if (rc != 0) {
       // DBGMSG("create_ddc_response_packet() returned %s, packet=%p", ddcrc_description(rc), packet);
-      TRCMSG("create_ddc_response_packet() returned %s, packet=%p", ddcrc_desc(rc), packet);
+      TRCMSG("create_ddc_response_packet() returned %s, packet=%p", ddcrc_desc_t(rc), packet);
    }
    if (rc == 0) {
       // dump_packet(packet);
@@ -1062,7 +1024,7 @@ create_ddc_getvcp_response_packet(
                &packet);
    if (rc != 0) {
       // DBGMSG("create_ddc_response_packet() returned %s, packet=%p", ddcrc_description(rc), packet);
-      TRCMSG("create_ddc_response_packet() returned %s, packet=%p", ddcrc_desc(rc), packet);
+      TRCMSG("create_ddc_response_packet() returned %s, packet=%p", ddcrc_desc_t(rc), packet);
    }
    if (rc == 0) {
       // dump_packet(packet);
