@@ -1,38 +1,15 @@
-/* displays.c
- *
- * Maintains list of all detected monitors.
- *
- * <copyright>
- * Copyright (C) 2014-2018 Sanford Rockowitz <rockowitz@minsoft.com>
- *
- * Licensed under the GNU General Public License Version 2
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- * </endcopyright>
- */
-
-/** \file
+/** @file displays.c
  * Monitor identifier, reference, handle
  */
 
+// Copyright (C) 2014-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// SPDX-License-Identifier: GPL-2.0-or-later
+
 #include <config.h>
 
 /** \cond */
 #include <assert.h>
-#include <glib.h>
-// #include <glib-2.0/glib.h>
+#include <glib-2.0/glib.h>
 #include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -45,14 +22,40 @@
 #include "util/udev_usb_util.h"
 /** \endcond */
 
-#include "public/ddcutil_c_api.h"
+#include "public/ddcutil_types.h"
+#include "public/ddcutil_status_codes.h"
 
 #include "core.h"
+#include "monitor_model_key.h"
 #include "vcp_version.h"
 
 #include "displays.h"
 
 
+#ifdef NOT_NEEDED
+typedef struct {
+   char * s_did;
+} Thread_Displays_Data;
+
+static Thread_Displays_Data *  get_thread_displays_data() {
+   static GPrivate per_thread_data_key = G_PRIVATE_INIT(g_free);
+
+   Thread_Displays_Data *thread_data = g_private_get(&per_thread_data_key);
+
+   // GThread * this_thread = g_thread_self();
+   // printf("(%s) this_thread=%p, settings=%p\n", __func__, this_thread, settings);
+
+   if (!thread_data) {
+      thread_data = g_new0(Thread_Displays_Data, 1);
+      g_private_set(&per_thread_data_key, thread_data);
+   }
+
+   // printf("(%s) Returning: %p\n", __func__, thread_data);
+   return thread_data;
+}
+#endif
+
+
 // *** Miscellaneous ***
 
 /** Reports whether a #DDCA_Adlno value is set or is currently undefined.
@@ -78,7 +81,7 @@ bool dpath_eq(DDCA_IO_Path p1, DDCA_IO_Path p2) {
    bool result = false;
    if (p1.io_mode == p2.io_mode) {
       switch(p1.io_mode) {
-      case DDCA_IO_DEVI2C:
+      case DDCA_IO_I2C:
          result = (p1.path.i2c_busno == p2.path.i2c_busno);
          break;
       case DDCA_IO_ADL:
@@ -461,36 +464,36 @@ char * did_repr(Display_Identifier * pdid) {
       char * did_type_name = display_id_type_name(pdid->id_type);
       switch (pdid->id_type) {
       case(DISP_ID_BUSNO):
-            pdid->repr = gaux_asprintf(
+            pdid->repr = g_strdup_printf(
                      "Display Id[type=%s, bus=/dev/i2c-%d]", did_type_name, pdid->busno);
             break;
       case(DISP_ID_ADL):
-            pdid->repr = gaux_asprintf(
+            pdid->repr = g_strdup_printf(
                      "Display Id[type=%s, adlno=%d.%d]", did_type_name, pdid->iAdapterIndex, pdid->iDisplayIndex);
             break;
       case(DISP_ID_MONSER):
-            pdid->repr = gaux_asprintf(
+            pdid->repr = g_strdup_printf(
                      "Display Id[type=%s, mfg=%s, model=%s, sn=%s]",
                      did_type_name, pdid->mfg_id, pdid->model_name, pdid->serial_ascii);
             break;
       case(DISP_ID_EDID):
       {
             char * hs = hexstring(pdid->edidbytes, 128);
-            pdid->repr = gaux_asprintf(
+            pdid->repr = g_strdup_printf(
                      "Display Id[type=%s, edid=%8s...%8s]", did_type_name, hs, hs+248);
             free(hs);
             break;
       }
       case(DISP_ID_DISPNO):
-            pdid->repr = gaux_asprintf(
+            pdid->repr = g_strdup_printf(
                      "Display Id[type=%s, dispno=%d]", did_type_name, pdid->dispno);
             break;
       case DISP_ID_USB:
-            pdid->repr = gaux_asprintf(
+            pdid->repr = g_strdup_printf(
                      "Display Id[type=%s, usb bus:device=%d.%d]", did_type_name, pdid->usb_bus, pdid->usb_device);;
             break;
       case DISP_ID_HIDDEV:
-            pdid->repr = gaux_asprintf(
+            pdid->repr = g_strdup_printf(
                      "Display Id[type=%s, hiddev_devno=%d]", did_type_name, pdid->hiddev_devno);
             break;
 
@@ -575,8 +578,8 @@ char * dpath_short_name_t(DDCA_IO_Path * dpath) {
 
    char * buf = get_thread_fixed_buffer(&dpath_short_name_key, 100);
    switch(dpath->io_mode) {
-   case DDCA_IO_DEVI2C:
-      snprintf(buf, 100, "bus dev/i2c-%d", dpath->path.i2c_busno);
+   case DDCA_IO_I2C:
+      snprintf(buf, 100, "bus /dev/i2c-%d", dpath->path.i2c_busno);
       break;
    case DDCA_IO_ADL:
       snprintf(buf, 100, "adlno (%d.%d)", dpath->path.adlno.iAdapterIndex, dpath->path.adlno.iDisplayIndex);
@@ -600,7 +603,7 @@ char * dpath_repr_t(DDCA_IO_Path * dpath) {
 
    char * buf = get_thread_fixed_buffer(&dpath_repr_key, 100);
    switch(dpath->io_mode) {
-   case DDCA_IO_DEVI2C:
+   case DDCA_IO_I2C:
       snprintf(buf, 100, "Display_Path[/dev/i2c-%d]", dpath->path.i2c_busno);
       break;
    case DDCA_IO_ADL:
@@ -620,7 +623,7 @@ static Display_Ref * create_base_display_ref(DDCA_IO_Path io_path) {
    Display_Ref * dref = calloc(1, sizeof(Display_Ref));
    memcpy(dref->marker, DISPLAY_REF_MARKER, 4);
    dref->io_path = io_path;
-   dref->vcp_version = VCP_SPEC_UNQUERIED;
+   dref->vcp_version = DDCA_VSPEC_UNQUERIED;
 
    dref->async_rec  = get_display_async_rec(io_path);    // keep?
 
@@ -630,7 +633,7 @@ static Display_Ref * create_base_display_ref(DDCA_IO_Path io_path) {
 
 
 // PROBLEM: bus display ref getting created some other way
-/** Creates a #Display_Ref for IO mode #DDCA_IO_DEVI2C
+/** Creates a #Display_Ref for IO mode #DDCA_IO_I2C
  *
  * @param busno /dev/i2c bus number
  * \return pointer to newly allocated #Display_Ref
@@ -638,7 +641,7 @@ static Display_Ref * create_base_display_ref(DDCA_IO_Path io_path) {
 Display_Ref * create_bus_display_ref(int busno) {
    bool debug = false;
    DDCA_IO_Path io_path;
-   io_path.io_mode   = DDCA_IO_DEVI2C;
+   io_path.io_mode   = DDCA_IO_I2C;
    io_path.path.i2c_busno = busno;
    Display_Ref * dref = create_base_display_ref(io_path);
    if (debug) {
@@ -720,18 +723,25 @@ Display_Ref * clone_display_ref(Display_Ref * old) {
 
 // Is it still meaningful to free a display ref?
 
-void free_display_ref(Display_Ref * dref) {
+DDCA_Status free_display_ref(Display_Ref * dref) {
+   DDCA_Status ddcrc = 0;
    if (dref && (dref->flags & DREF_TRANSIENT) ) {
-      assert(memcmp(dref->marker, DISPLAY_REF_MARKER,4) == 0);
-      dref->marker[3] = 'x';
-      if (dref->usb_hiddev_name)       // always set by strdup()
-         free(dref->usb_hiddev_name);
-      if (dref->capabilities_string)   // always a private copy
-         free(dref->capabilities_string);
-      // 9/2017: what about pedid, detail2?
-      // what to do with gdl, request_queue?
-      free(dref);
+      if (dref->flags & DREF_OPEN) {
+         ddcrc = DDCRC_LOCKED;
+      }
+      else {
+         assert(memcmp(dref->marker, DISPLAY_REF_MARKER,4) == 0);
+         dref->marker[3] = 'x';
+         if (dref->usb_hiddev_name)       // always set by strdup()
+            free(dref->usb_hiddev_name);
+         if (dref->capabilities_string)   // always a private copy
+            free(dref->capabilities_string);
+         // 9/2017: what about pedid, detail2?
+         // what to do with gdl, request_queue?
+         free(dref);
+      }
    }
+   return ddcrc;
 }
 
 /** Tests if 2 #Display_Ref instances specify the same path to the
@@ -746,41 +756,13 @@ void free_display_ref(Display_Ref * dref) {
  *  \retval false different displays
  */
 bool dref_eq(Display_Ref* this, Display_Ref* that) {
-#ifdef OLD
-   bool result = false;
-   if (!this && !that)
-      result = true;
-   else if (this && that) {
-      if (this->io_mode == that->io_mode) {
-         switch (this->io_mode) {
-
-         case DDCA_IO_DEVI2C:
-            result = (this->busno == that->busno);
-            break;
-
-         case DDCA_IO_ADL:
-            result = (this->iAdapterIndex == that->iAdapterIndex &&
-                      this->iDisplayIndex == that->iDisplayIndex);
-            break;
-
-         case DDCA_IO_USB:
-            result = (this->usb_bus    == that->usb_bus  &&
-                      this->usb_device == that->usb_device);
-            break;
-
-         }
-      }
-   }
-   return result;
-#endif
-   // return dpath_eq(dpath_from_dref(this), dpath_from_dref(that));
    return dpath_eq(this->io_path, that->io_path);
 }
 
 
 /** Reports the contents of a #Display_Ref in a format appropriate for debugging.
  *
- *  \param  pref  pointer to #Display_Ref instance
+ *  \param  dref  pointer to #Display_Ref instance
  *  \param  depth logical indentation depth
  */
 void dbgrpt_display_ref(Display_Ref * dref, int depth) {
@@ -794,7 +776,7 @@ void dbgrpt_display_ref(Display_Ref * dref, int depth) {
 
    switch (dref->io_mode) {
 
-   case DDCA_IO_DEVI2C:
+   case DDCA_IO_I2C:
       rpt_int("busno", NULL, dref->busno, d1);
       break;
 
@@ -815,22 +797,26 @@ void dbgrpt_display_ref(Display_Ref * dref, int depth) {
 
    // alt:
    rpt_vstring(d1, "io_path:      %s", dpath_repr_t(&(dref->io_path)));
+
    if (dref->io_path.io_mode == DDCA_IO_USB) {
-      rpt_int("usb_bus",    NULL, dref->usb_bus,    d1);
-      rpt_int("usb_device", NULL, dref->usb_device, d1);
+      rpt_int("usb_bus",         NULL, dref->usb_bus,         d1);
+      rpt_int("usb_device",      NULL, dref->usb_device,      d1);
       rpt_str("usb_hiddev_name", NULL, dref->usb_hiddev_name, d1);
    }
 
    // rpt_vstring(d1, "vcp_version:  %d.%d\n", dref->vcp_version.major, dref->vcp_version.minor );
    rpt_vstring(d1, "vcp_version:  %s", format_vspec(dref->vcp_version) );
    rpt_vstring(d1, "flags:        0x%02x", dref->flags);
-   rpt_vstring(d2, "DDC communication checked:                  %s", (dref->flags & DREF_DDC_COMMUNICATION_CHECKED) ? "true" : "false");
+   rpt_vstring(d2, "DDC communication checked:                  %s", sbool(dref->flags & DREF_DDC_COMMUNICATION_CHECKED) );
    if (dref->flags & DREF_DDC_COMMUNICATION_CHECKED)
-   rpt_vstring(d2, "DDC communication working:                  %s", (dref->flags & DREF_DDC_COMMUNICATION_WORKING) ? "true" : "false");
-   rpt_vstring(d2, "DDC NULL response usage checked:            %s", bool_repr(dref->flags & DREF_DDC_NULL_RESPONSE_CHECKED));
+   rpt_vstring(d2, "DDC communication working:                  %s", sbool(dref->flags & DREF_DDC_COMMUNICATION_WORKING) );
+   rpt_vstring(d2, "DDC NULL response usage checked:            %s", sbool(dref->flags & DREF_DDC_NULL_RESPONSE_CHECKED) );
    if (dref->flags & DREF_DDC_NULL_RESPONSE_CHECKED)
-   rpt_vstring(d2, "DDC NULL response may indicate unsupported: %s", bool_repr(dref->flags & DREF_DDC_USES_NULL_RESPONSE_FOR_UNSUPPORTED));
-}
+   rpt_vstring(d2, "DDC NULL response may indicate unsupported: %s", sbool(dref->flags & DREF_DDC_USES_NULL_RESPONSE_FOR_UNSUPPORTED));
+   rpt_vstring(d2, "DDC normal all byte 0 response may indicate unsupported: %s", sbool(dref->flags & DREF_DDC_USES_MH_ML_SH_SL_ZERO_FOR_UNSUPPORTED));
+   rpt_vstring(d2, "DDC does not indicate unsupported:          %s", sbool(dref->flags & DREF_DDC_DOES_NOT_INDICATE_UNSUPPORTED));
+   rpt_vstring(d2, "Display Ref is open:                        %s", sbool(dref->flags & DREF_OPEN));
+   rpt_vstring(d2, "mmid:                                       %s", (dref->mmid) ? mmk_repr(*dref->mmid) : "NULL");}
 
 
 #ifdef OLD
@@ -848,7 +834,7 @@ char * dref_short_name_r(Display_Ref * dref, char * buf, int bufsz) {
 
    switch (dref->io_mode) {
 
-   case DDCA_IO_DEVI2C:
+   case DDCA_IO_I2C:
       SAFE_SNPRINTF(buf, bufsz, "bus /dev/i2c-%d", dref->busno);
 
       // snprintf(buf, bufsz, "bus /dev/i2c-%d", dref->busno);
@@ -902,8 +888,10 @@ char * dref_repr_t(Display_Ref * dref) {
    static GPrivate  dref_repr_key = G_PRIVATE_INIT(g_free);
 
    char * buf = get_thread_fixed_buffer(&dref_repr_key, 100);
-   // char buf2[80];
-   SAFE_SNPRINTF(buf, 100, "Display_Ref[%s]", dpath_short_name_t(&dref->io_path));
+   if (dref)
+      g_snprintf(buf, 100, "Display_Ref[%s]", dpath_short_name_t(&dref->io_path));
+   else
+      strcpy(buf, "Display_Ref[NULL]");
    return buf;
 }
 
@@ -921,14 +909,15 @@ char * dref_repr_t(Display_Ref * dref) {
  */
 Display_Handle * create_bus_display_handle_from_display_ref(int fh, Display_Ref * dref) {
    // assert(dref->io_mode == DDCA_IO_DEVI2C);
-   assert(dref->io_path.io_mode == DDCA_IO_DEVI2C);
+   assert(dref->io_path.io_mode == DDCA_IO_I2C);
    Display_Handle * dh = calloc(1, sizeof(Display_Handle));
    memcpy(dh->marker, DISPLAY_HANDLE_MARKER, 4);
    dh->fh = fh;
    dh->dref = dref;
-   dref->vcp_version = VCP_SPEC_UNQUERIED;
-   dh->repr = gaux_asprintf(
-                "Display_Handle[i2c: fh=%d, busno=%d]",
+   // dref->vcp_version = DDCA_VSPEC_UNQUERIED;
+   dh->repr = g_strdup_printf(
+                "[i2c: fh=%d, busno=%d]",
+                // "Display_Handle[i2c: fh=%d, busno=%d]",
                 dh->fh, dh->dref->io_path.path.i2c_busno);
    return dh;
 }
@@ -948,9 +937,10 @@ Display_Handle * create_adl_display_handle_from_display_ref(Display_Ref * dref)
    Display_Handle * dh = calloc(1, sizeof(Display_Handle));
    memcpy(dh->marker, DISPLAY_HANDLE_MARKER, 4);
    dh->dref = dref;
-   dref->vcp_version = VCP_SPEC_UNQUERIED;   // needed?
-   dh->repr = gaux_asprintf(
-                "Display_Handle[adl: display %d.%d]",
+   // dref->vcp_version = DDCA_VSPEC_UNQUERIED;   // needed?
+   dh->repr = g_strdup_printf(
+                "[adl: display %d.%d]",
+                // "Display_Handle[adl: display %d.%d]",
                  dh->dref->io_path.path.adlno.iAdapterIndex, dh->dref->io_path.path.adlno.iDisplayIndex);
    return dh;
 }
@@ -973,11 +963,12 @@ Display_Handle * create_usb_display_handle_from_display_ref(int fh, Display_Ref
    memcpy(dh->marker, DISPLAY_HANDLE_MARKER, 4);
    dh->fh = fh;
    dh->dref = dref;
-   dh->repr = gaux_asprintf(
-                "Display_Handle[usb: %d:%d, %s/hiddev%d]",
+   dh->repr = g_strdup_printf(
+                "[usb: %d:%d, %s/hiddev%d]",
+                // "Display_Handle[usb: %d:%d, %s/hiddev%d]",
                 dh->dref->usb_bus, dh->dref->usb_device,
                 usb_hiddev_directory(), dh->dref->io_path.path.hiddev_devno);
-   dref->vcp_version = VCP_SPEC_UNQUERIED;
+   // dref->vcp_version = DDCA_VSPEC_UNQUERIED;
    return dh;
 }
 #endif
@@ -1003,7 +994,7 @@ void dbgrpt_display_handle(Display_Handle * dh, const char * msg, int depth) {
          rpt_vstring(d1, "dref:                 %p", dh->dref);
          rpt_vstring(d1, "io mode:              %s",  io_mode_name(dh->dref->io_path.io_mode) );
          switch (dh->dref->io_path.io_mode) {
-         case (DDCA_IO_DEVI2C):
+         case (DDCA_IO_I2C):
             // rpt_vstring(d1, "ddc_io_mode = DDC_IO_DEVI2C");
             rpt_vstring(d1, "fh:                  %d", dh->fh);
             rpt_vstring(d1, "busno:               %d", dh->dref->io_path.path.i2c_busno);
@@ -1026,82 +1017,6 @@ void dbgrpt_display_handle(Display_Handle * dh, const char * msg, int depth) {
    }
 }
 
-#ifdef OLD
-/* Returns a string summarizing the specified #Display_Handle.
- *
- * The string is returned in a newly allocated buffer.
- * It is the responsibility of the caller to free this buffer.
- *
- * \param  dh    display handle
- *
- * \return  string representation of handle
- */
-char * dh_repr_a(Display_Handle * dh) {
-   assert(dh);
-   assert(dh->dref);
-
-   char * repr = NULL;
-
-      switch (dh->dref->io_mode) {
-      case DDCA_IO_DEVI2C:
-          repr = gaux_asprintf(
-                   "Display_Handle[i2c: fh=%d, busno=%d]",
-                   dh->fh, dh->dref->busno);
-          break;
-      case DDCA_IO_ADL:
-          repr = gaux_asprintf(
-                   "Display_Handle[adl: display %d.%d]",
-                   dh->dref->iAdapterIndex, dh->dref->iDisplayIndex);
-          break;
-      case DDCA_IO_USB:
-          repr = gaux_asprintf(
-                   "Display_Handle[usb: %d:%d, %s/hiddev%d]",
-                   dh->dref->usb_bus, dh->dref->usb_device,
-                   usb_hiddev_directory(), dh->dref->usb_hiddev_devno);
-          break;
-      }
-      return repr;
-   }
-#endif
-
-
-#ifdef OLD
-/* Returns a string summarizing the specified #Display_Handle.
- * The string is returned in a buffer provided by the caller.
- *
- * \param  dh    display handle
- * \param  buf   pointer to buffer
- * \param  bufsz buffer size
- *
- * \return  string representation of handle (buf)
- */
-static char * dh_repr_r(Display_Handle * dh, char * buf, int bufsz) {
-   assert(dh);
-   assert(dh->dref);
-
-   switch (dh->dref->io_mode) {
-   case DDCA_IO_DEVI2C:
-       snprintf(buf, bufsz,
-                "Display_Handle[i2c: fh=%d, busno=%d]",
-                dh->fh, dh->dref->busno);
-       break;
-   case DDCA_IO_ADL:
-       snprintf(buf, bufsz,
-                "Display_Handle[adl: display %d.%d]",
-                dh->dref->iAdapterIndex, dh->dref->iDisplayIndex);
-       break;
-   case DDCA_IO_USB:
-       snprintf(buf, bufsz,
-                "Display_Handle[usb: %d:%d, %s/hiddev%d]",
-                dh->dref->usb_bus, dh->dref->usb_device,
-                usb_hiddev_directory(), dh->dref->usb_hiddev_devno);
-       break;
-   }
-   buf[bufsz-1] = '\0';
-   return buf;
-}
-#endif
-
 
 /** Returns a string summarizing the specified #Display_Handle.
  *
@@ -1118,28 +1033,32 @@ char * dh_repr_t(Display_Handle * dh) {
    const int bufsz = 100;
    char * buf = get_thread_fixed_buffer(&dh_buf_key, bufsz);
 
-   assert(dh);
-   assert(dh->dref);
+   if (dh) {
+      assert(dh->dref);
 
-   switch (dh->dref->io_path.io_mode) {
-   case DDCA_IO_DEVI2C:
-        snprintf(buf, bufsz,
-                 "Display_Handle[i2c: fh=%d, busno=%d]",
-                 dh->fh, dh->dref->io_path.path.i2c_busno);
-        break;
-    case DDCA_IO_ADL:
-        snprintf(buf, bufsz,
-                 "Display_Handle[adl: display %d.%d]",
-                 dh->dref->io_path.path.adlno.iAdapterIndex, dh->dref->io_path.path.adlno.iDisplayIndex);
-        break;
-    case DDCA_IO_USB:
-        snprintf(buf, bufsz,
-                 "Display_Handle[usb: %d:%d, %s/hiddev%d]",
-                 dh->dref->usb_bus, dh->dref->usb_device,
-                 usb_hiddev_directory(), dh->dref->io_path.path.hiddev_devno);
-        break;
-    }
-    buf[bufsz-1] = '\0';
+      switch (dh->dref->io_path.io_mode) {
+      case DDCA_IO_I2C:
+           snprintf(buf, bufsz,
+                    "Display_Handle[i2c: fh=%d, busno=%d]",
+                    dh->fh, dh->dref->io_path.path.i2c_busno);
+           break;
+       case DDCA_IO_ADL:
+           snprintf(buf, bufsz,
+                    "Display_Handle[adl: display %d.%d]",
+                    dh->dref->io_path.path.adlno.iAdapterIndex, dh->dref->io_path.path.adlno.iDisplayIndex);
+           break;
+       case DDCA_IO_USB:
+           snprintf(buf, bufsz,
+                    "Display_Handle[usb: %d:%d, %s/hiddev%d]",
+                    dh->dref->usb_bus, dh->dref->usb_device,
+                    usb_hiddev_directory(), dh->dref->io_path.path.hiddev_devno);
+           break;
+       }
+       buf[bufsz-1] = '\0';
+   }
+   else {
+      strcpy(buf, "Display_Handle[NULL]");
+   }
 
    return buf;
 }
@@ -1206,7 +1125,7 @@ int hiddev_name_to_number(char * hiddev_name) {
       if (strlen(p) > 0) {
          // hiddev_number unchanged if error
          // n str_to_int() allows leading whitespace, not worth checking
-         bool ok = str_to_int(p, &hiddev_number);
+         bool ok = str_to_int(p, &hiddev_number, 10);
          if (!ok)
             hiddev_number = -1;   // not necessary, but makes coverity happy
       }
@@ -1225,7 +1144,7 @@ int hiddev_name_to_number(char * hiddev_name) {
  */
 char * hiddev_number_to_name(int hiddev_number) {
    assert(hiddev_number >= 0);
-   char * s = gaux_asprintf("%s/hiddev%d", usb_hiddev_directory(),hiddev_number);
+   char * s = g_strdup_printf("%s/hiddev%d", usb_hiddev_directory(),hiddev_number);
    // DBGMSG("hiddev_number=%d, returning: %s", hiddev_number, s);
    return s;
 }
