@@ -1,32 +1,10 @@
-/* execution_stats.c
- *
- * For recording and reporting the count and elapsed time of system calls.
- *
- * <copyright>
- * Copyright (C) 2014-2017 Sanford Rockowitz <rockowitz@minsoft.com>
- *
- * Licensed under the GNU General Public License Version 2
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- * </endcopyright>
- */
-
-/** \file
+/** \file execution_stats.c
  * Record execution statistics, mainly the count and elapsed time of system calls.
  */
 
+// Copyright (C) 2014-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// SPDX-License-Identifier: GPL-2.0-or-later
+
 /** \cond */
 #include <assert.h>
 #include <glib.h>
@@ -409,8 +387,8 @@ void show_specific_status_counts(Status_Code_Counts * pcounts) {
    if (debug) {
       DBGMSG("Keys.  keyct=%d", keyct);
       for (int ndx = 0; ndx < keyct; ndx++) {
-         DBGMSG( "keysp[%d]:  %d   %p  %d",
-                 ndx,  keysp[ndx], keysp[ndx], GPOINTER_TO_INT(keysp[ndx]) );
+         DBGMSG( "keysp[%d]:  %p  %d",
+                 ndx, keysp[ndx], GPOINTER_TO_INT(keysp[ndx]) );
       }
    }
    int summed_ct = 0;
@@ -680,11 +658,11 @@ void call_tuned_sleep(DDCA_IO_Mode io_mode, Sleep_Event_Type event_type) {
    bool debug = false || debug_sleep_stats_mutex;
    DBGMSF(debug, "Starting");
 
+   assert(event_type != SE_DDC_NULL);  // SE_DDC_NULL uses call_dynamic_tuned_sleep()
+
    int sleep_time_millis = 0;    // should be a default
    switch(io_mode) {
 
-   assert(event_type != SE_DDC_NULL);
-
    case DDCA_IO_I2C:
       switch(event_type) {
       case (SE_WRITE_TO_READ):
@@ -747,7 +725,7 @@ void call_tuned_sleep(DDCA_IO_Mode io_mode, Sleep_Event_Type event_type) {
       break;
 
    case DDCA_IO_USB:
-      printf("(%s) call_tuned_sleep() called for USB_IO\n", __func__);
+      PROGRAM_LOGIC_ERROR("call_tuned_sleep() called for USB_IO\n");
       break;
 
    }
