@@ -1,29 +1,10 @@
-/* feature_sets.h
+/* \file feature_sets.h
  *
- * <copyright>
- * Copyright (C) 2014-2018 Sanford Rockowitz <rockowitz@minsoft.com>
- *
- * Licensed under the GNU General Public License Version 2
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- * </endcopyright>
+ * Feature set identifiers
  */
 
-/** \file
- *  Feature set identifiers
- */
+// Copyright (C) 2014-2018 Sanford Rockowitz <rockowitz@minsoft.com>
+// SPDX-License-Identifier: GPL-2.0-or-later
 
 #ifndef FEATURE_SETS_H_
 #define FEATURE_SETS_H_
@@ -71,7 +52,7 @@ typedef enum {
 
    VCP_SUBSET_MFG             = 0x00000004,    // mfg specific codes
 
-// VCP_SUBSET_CUSTOM          = 0x00000002,    // possible future subset, aka DYNAMIC, aka USER
+   VCP_SUBSET_DYNAMIC         = 0x00000002,    // aka CUSTOM, DYNAMIC, USER
    VCP_SUBSET_SINGLE_FEATURE  = 0x00000001,
    VCP_SUBSET_NONE            = 0x00000000,
 } VCP_Feature_Subset;
@@ -98,8 +79,9 @@ typedef enum {
    FSF_FORCE                 = 0x20
 } Feature_Set_Flags;
 
-char * feature_set_flag_names(Feature_Set_Flags flags);
+char * feature_set_flag_names_t(Feature_Set_Flags flags);
 
 void dbgrpt_feature_set_ref(Feature_Set_Ref * fsref, int depth);
+char * fsref_repr_t(Feature_Set_Ref * fsref);
 
 #endif /* FEATURE_SETS_H_ */
