@@ -1,28 +1,11 @@
-/* cmd_parser_goption.c
+/** @file cmd_parser_goption.c
  *
- * Parse the command line using the glib goption functions.
- *
- * <copyright>
- * Copyright (C) 2014-2018 Sanford Rockowitz <rockowitz@minsoft.com>
- *
- * Licensed under the GNU General Public License Version 2
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- * </endcopyright>
+ *  Parse the command line using the glib goption functions.
  */
 
+// Copyright (C) 2014-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// SPDX-License-Identifier: GPL-2.0-or-later
+
 #include <config.h>
 
 #include <assert.h>
@@ -189,6 +172,10 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
    gboolean rw_only_flag   = false;
    gboolean ro_only_flag   = false;
    gboolean wo_only_flag   = false;
+   gboolean enable_udf_flag = false;
+#ifdef USE_USB
+   gboolean enable_usb_flag = true;
+#endif
    char *   mfg_id_work    = NULL;
    char *   modelwork      = NULL;
    char *   snwork         = NULL;
@@ -221,7 +208,13 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
       {"model",   'l',  0, G_OPTION_ARG_STRING,   &modelwork,        "Monitor model",               "model name"},
       {"sn",      'n',  0, G_OPTION_ARG_STRING,   &snwork,           "Monitor serial number",       "serial number"},
       {"edid",    'e',  0, G_OPTION_ARG_STRING,   &edidwork,         "Monitor EDID",            "256 char hex string" },
+#ifdef USE_USB
+//    {"disable-usb", '\0', G_OPTION_FLAG_REVERSE,
+//                             G_OPTION_ARG_NONE, &enable_usb_flag,  "Do not detect USB devices", NULL},
+      {"nousb",   '\0', G_OPTION_FLAG_REVERSE,
+                               G_OPTION_ARG_NONE, &enable_usb_flag,  "Do not detect USB devices", NULL},
 
+#endif
       // output control
       {"ddc",     '\0', 0, G_OPTION_ARG_NONE,     &ddc_flag,         "Report DDC protocol and data errors", NULL},
       {"verbose", 'v',  G_OPTION_FLAG_NO_ARG,
@@ -254,6 +247,10 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
       {"nodetect",'\0', 0, G_OPTION_ARG_NONE,     &nodetect_flag,    "Skip initial monitor detection",  NULL},
       {"async",   '\0', 0, G_OPTION_ARG_NONE,     &async_flag,       "Enable asynchronous display detection", NULL},
 
+      {"udf",     '\0', 0, G_OPTION_ARG_NONE,     &enable_udf_flag,  "Enable user defined feature support", NULL},
+      {"noudf",   '\0', G_OPTION_FLAG_REVERSE,
+                           G_OPTION_ARG_NONE,     &enable_udf_flag,  "Disable user defined feature support", NULL},
+
       // debugging
       {"excp",    '\0', 0, G_OPTION_ARG_NONE,     &report_freed_excp_flag,  "Report freed exceptions", NULL},
       {"trace",   '\0', 0, G_OPTION_ARG_STRING_ARRAY, &trace_classes, "Trace classes",         "trace class name" },
@@ -373,6 +370,10 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
    SET_CMDFLAG(CMD_FLAG_RO_ONLY,           ro_only_flag);
    SET_CMDFLAG(CMD_FLAG_WO_ONLY,           wo_only_flag);
    SET_CMDFLAG(CMD_FLAG_FORCE,             force_flag);
+   SET_CMDFLAG(CMD_FLAG_ENABLE_UDF,        enable_udf_flag);
+#ifdef USE_USB
+   SET_CMDFLAG(CMD_FLAG_ENABLE_USB,        enable_usb_flag);
+#endif
 
    if (failsim_fn_work) {
 #ifdef ENABLE_FAILSIM
@@ -573,7 +574,7 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
        if (debug)
           DBGMSG("tracework, argument = |%s|", tracework );
        strupper(tracework);
-       Trace_Group traceClasses = 0x00;
+       DDCA_Trace_Group traceClasses = 0x00;
 
        Null_Terminated_String_Array pieces = strsplit(tracework, ',');
        int ndx = 0;
@@ -585,7 +586,7 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
              traceClasses = 0xff;
           else {
              // DBGMSG("token: |%s|", token);
-             Trace_Group tg = trace_class_name_to_value(token);
+             DDCA_Trace_Group tg = trace_class_name_to_value(token);
              // DBGMSG("tg=0x%02x", tg);
              if (tg) {
                 traceClasses |= tg;
@@ -602,25 +603,25 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
        ntsa_free(pieces);
 
        DBGMSG("traceClasses = 0x%02x", traceClasses);
-       parsed_cmd->trace = traceClasses;
+       parsed_cmd->traced_groups = traceClasses;
        debug = saved_debug;
     }
 #endif
 
 // #ifdef MULTIPLE_TRACE
    if (trace_classes) {
-      Trace_Group traceClasses = 0x00;
+      DDCA_Trace_Group traceClasses = 0x00;
       int ndx = 0;
       for (;trace_classes[ndx] != NULL; ndx++) {
          char * token = trace_classes[ndx];
          strupper(token);
          // DBGMSG("token=|%s|", token);
          if (streq(token, "ALL") || streq(token, "*")) {
-            traceClasses = TRC_ALWAYS;   // 0xff
+            traceClasses = DDCA_TRC_ALL;   // 0xff
          }
          else {
             // DBGMSG("token: |%s|", token);
-            Trace_Group tg = trace_class_name_to_value(token);
+            DDCA_Trace_Group tg = trace_class_name_to_value(token);
             // DBGMSG("tg=0x%02x", tg);
             if (tg) {
                traceClasses |= tg;
@@ -632,7 +633,7 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
         }
       }
       // DBGMSG("traceClasses = 0x%02x", traceClasses);
-      parsed_cmd->trace = traceClasses;
+      parsed_cmd->traced_groups = traceClasses;
    }
 // #endif
 
