@@ -1,30 +1,11 @@
-/* ddc_displays.c
- *
- * <copyright>
- * Copyright (C) 2014-2018 Sanford Rockowitz <rockowitz@minsoft.com>
- *
- * Licensed under the GNU General Public License Version 2
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- * </endcopyright>
- */
-
-/** \file
+/** \file ddc_displays.c
  * Access displays, whether DDC, ADL, or USB
  */
 
+// Copyright (C) 2014-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// SPDX-License-Identifier: GPL-2.0-or-later
+
+
 #include <config.h>
 
 /** \cond */
@@ -35,6 +16,7 @@
 #include <time.h>
 
 #include "util/debug_util.h"
+#include "util/edid.h"
 #include "util/error_info.h"
 #include "util/failsim.h"
 #include "util/report_util.h"
@@ -45,7 +27,9 @@
 #include "base/adl_errors.h"
 #include "base/ddc_errno.h"
 #include "base/ddc_packets.h"
+#include "base/feature_metadata.h"
 #include "base/linux_errno.h"
+#include "base/monitor_model_key.h"
 #include "base/parms.h"
 
 #include "vcp/vcp_feature_codes.h"
@@ -62,7 +46,8 @@
 #include "usb/usb_displays.h"
 #endif
 
-#include "ddc/ddc_dynamic_features.h"
+#include <dynvcp/dyn_dynamic_features.h>
+
 #include "ddc/ddc_packet_io.h"
 #include "ddc/ddc_vcp.h"
 #include "ddc/ddc_vcp_version.h"
@@ -70,14 +55,18 @@
 #include "ddc/ddc_displays.h"
 
 
-// Trace class for this file
-// static Trace_Group TRACE_GROUP = TRC_DDC;   // currently unused
+// Default race class for this file
+static DDCA_Trace_Group TRACE_GROUP = DDCA_TRC_DDCIO;
 
 
 static GPtrArray * all_displays = NULL;    // all detected displays
 static int dispno_max = 0;                 // highest assigned display number
 static int async_threshold = DISPLAY_CHECK_ASYNC_THRESHOLD;
-
+#ifdef USE_USB
+static bool detect_usb_displays = true;
+#else
+static bool detect_usb_displays = false;
+#endif
 
 void ddc_set_async_threshold(int threshold) {
    // DBGMSG("threshold = %d", threshold);
@@ -85,161 +74,13 @@ void ddc_set_async_threshold(int threshold) {
 }
 
 
-#ifdef OLD
-// NOTICE EXTENDED REMARKS IN COMMENTS, MAY WANT TO PRESERVE
-/** Checks that DDC communication is working by trying to read the value
- *  of feature x10 (brightness).
- *
- *  \param dh  #Display_Handle of open display
- *  \retval true communication successful
- *  \retval false communication failed
- *
- *  \remark
- *  It has been observed that DDC communication can fail even if slave address x37
- *  is valid on the I2C bus.
- *  \remark
- *  ADL does not notice that a reported display, e.g. Dell 1905FP, does not support
- *  DDC.
- *  \remark
- *  If a validly structured DDC response is received, e.g. with the unsupported feature
- *  bit set or a DDC Null response, communication is considered successful.
- *  \remark
- *  Output level should have been set <= DDCA_OL_NORMAL prior to this call since
- *  verbose output is distracting.
- */
-static
-bool check_ddc_communication(Display_Handle * dh) {
-   bool debug = false;
-   DBGMSF(debug, "Starting. dh=%s", dh_repr_t(dh));
-
-   bool result = true;
-
-   Single_Vcp_Value * pvalrec;
-
-   // verbose output is distracting since this function is called when querying for other things
-   // DDCA_Output_Level olev = get_output_level();
-   // if (olev == DDCA_OL_VERBOSE)
-   //    set_output_level(DDCA_OL_NORMAL);
-
-   Public_Status_Code psc = ddc_get_vcp_value(dh, 0x10, DDCA_NON_TABLE_VCP_VALUE, &pvalrec);
-
-   // if (olev == DDCA_OL_VERBOSE)
-   //    set_output_level(olev);
-
-   if (psc != 0 && psc != DDCRC_REPORTED_UNSUPPORTED && psc != DDCRC_DETERMINED_UNSUPPORTED) {
-      result = false;
-      DBGMSF(debug, "Error getting value for brightness VCP feature 0x10. gsc=%s\n", psc_desc(psc) );
-   }
-
-   DBGMSF(debug, "Returning: %s", bool_repr(result));
-   return result;
-}
-#endif
-
-
-#ifdef OLD
-/** Checks whether the monitor uses a DDC Null response to report
- *  an unsupported VCP code by attempting to read feature 0x00.
- *
- *  \param dh  #Display_Handle of monitor
- *  \retval true  DDC Null Response was received
- *  \retval false any other response
- *
- *  \remark
- *  Monitors should set the unsupported feature bit in a valid DDC
- *  response, but a few monitors (mis)use the Null Response instead.
- *  \remark
- *  Note that this test is not perfect, as a Null Response might
- *  in fact indicate a transient error, but that is rare.
- *  \remark
- *  Output level should have been set <= DDCA_OL_NORMAL prior to this call since
- *  verbose output is distracting.
- */
-static
-bool check_monitor_ddc_null_response(Display_Handle * dh) {
-   assert(dh);
-   assert(dh->dref);
-   bool debug = false;
-   DBGMSF(debug, "Starting. dh=%s", dh_repr_t(dh));
-
-   bool result = false;
-
-   if (dh->dref->io_mode != DDCA_IO_USB) {
-
-      Single_Vcp_Value * pvalrec;
-
-      // // verbose output is distracting since this function is called when querying for other things
-      // DDCA_Output_Level olev = get_output_level();
-      // if (olev == DDCA_OL_VERBOSE)
-      //    set_output_level(DDCA_OL_NORMAL);
-
-      Public_Status_Code psc = ddc_get_vcp_value(dh, 0x00, DDCA_NON_TABLE_VCP_VALUE, &pvalrec);
-
-      // if (olev == DDCA_OL_VERBOSE)
-      //    set_output_level(olev);
-
-      if (psc == DDCRC_NULL_RESPONSE) {
-         result = true;
-      }
-      else if (psc != 0 && psc != DDCRC_REPORTED_UNSUPPORTED && psc != DDCRC_DETERMINED_UNSUPPORTED) {
-         DBGMSF(debug, "Unexpected status getting value for non-existent VCP feature 0x00. gsc=%s\n", psc_desc(psc) );
-      }
-
-   }
-
-   DBGMSF(debug, "Returning: %s", bool_repr(result));
-   return result;
-}
-#endif
-
-
-#ifdef OLD
-/** Collects most initial monitor checks to perform them on a single open of the
- *  monitor device, and to avoid repeating them.
- *
- *  Performs the following tests:
- *  - Checks that DDC communication is working.
- *  - Checks if the monitor uses DDC Null Response to indicate invalid VCP code
- *  - Queries the VCP (MCCS) version.
- *
- *  \param dh  pointer to #Display_Handle for open monitor device
- *  \return **true** if DDC communication with the display succeeded, **false** otherwise.
- */
-bool initial_checks_by_dh_old(Display_Handle * dh) {
-   bool debug = false;
-   DBGMSF(debug, "Starting. dh=%s", dh_repr_t(dh));
-
-   if (!(dh->dref->flags & DREF_DDC_COMMUNICATION_CHECKED)) {
-      if (check_ddc_communication(dh))
-         dh->dref->flags |= DREF_DDC_COMMUNICATION_WORKING;
-      dh->dref->flags |= DREF_DDC_COMMUNICATION_CHECKED;
-   }
-   bool communication_working = dh->dref->flags & DREF_DDC_COMMUNICATION_WORKING;
-
-   if (communication_working) {
-      if (!(dh->dref->flags & DREF_DDC_NULL_RESPONSE_CHECKED)) {
-         if (check_monitor_ddc_null_response(dh) )
-            dh->dref->flags |= DREF_DDC_USES_NULL_RESPONSE_FOR_UNSUPPORTED;
-         dh->dref->flags |= DREF_DDC_NULL_RESPONSE_CHECKED;
-      }
-      if ( vcp_version_is_unqueried(dh->dref->vcp_version)) {
-         dh->dref->vcp_version = get_vcp_version_by_display_handle(dh);
-         // dh->vcp_version = dh->dref->vcp_version;
-      }
-   }
-
-   DBGMSF(debug, "Returning: %s", bool_repr(communication_working));
-   return communication_working;
-}
-#endif
-
-
 /** Collects initial monitor checks to perform them on a single open of the
  *  monitor device, and to avoid repeating them.
  *
  *  Performs the following tests:
  *  - Checks that DDC communication is working.
  *  - Checks if the monitor uses DDC Null Response to indicate invalid VCP code
+ *  - Checks if the monitor uses mh=ml=sh=sl=0 to indicate invalid VCP code
  *
  *  \param dh  pointer to #Display_Handle for open monitor device
  *  \return **true** if DDC communication with the display succeeded, **false** otherwise.
@@ -255,7 +96,8 @@ bool initial_checks_by_dh_old(Display_Handle * dh) {
  *  \remark
  *  Monitors are supposed to set the unsupported feature bit in a valid DDC
  *  response, but a few monitors (mis)use the Null Response instead to indicate
- *  an unsupported feature.
+ *  an unsupported feature. Others return with the unsupported feature bit not
+ *  set, but all bytes (mh, ml, sh, sl) zero.
  *  \remark
  *  Note that the test here is not perfect, as a Null Response might
  *  in fact indicate a transient error, but that is rare.
@@ -265,47 +107,123 @@ bool initial_checks_by_dh_old(Display_Handle * dh) {
  */
 bool initial_checks_by_dh(Display_Handle * dh) {
    bool debug = false;
-   DBGMSF(debug, "Starting. dh=%s", dh_repr_t(dh));
+   DBGTRC(debug, TRACE_GROUP, "Starting. dh=%s", dh_repr_t(dh));
    assert(dh);
-   Single_Vcp_Value * pvalrec;
+   DDCA_Any_Vcp_Value * pvalrec;
 
    if (!(dh->dref->flags & DREF_DDC_COMMUNICATION_CHECKED)) {
-
       Public_Status_Code psc = 0;
       Error_Info * ddc_excp = ddc_get_vcp_value(dh, 0x00, DDCA_NON_TABLE_VCP_VALUE, &pvalrec);
       psc = (ddc_excp) ? ddc_excp->status_code : 0;
-      DBGMSF(debug, "get_vcp_value() for feature 0x00 returned: %s", psc_desc(psc));
+      DBGTRC(debug, TRACE_GROUP, "ddc_get_vcp_value() for feature 0x00 returned: %s, pvalrec=%p",
+                             //    psc_desc(psc),
+                                 errinfo_summary(ddc_excp),
+                                 pvalrec);
       if (psc == DDCRC_RETRIES && debug)
          DBGMSG("    Try errors: %s", errinfo_causes_string(ddc_excp));
       if (ddc_excp)
          errinfo_free(ddc_excp);
 
-      if (psc == DDCRC_NULL_RESPONSE ||
-          psc == DDCRC_ALL_RESPONSES_NULL ||
-          psc == 0                   ||
-          psc == DDCRC_REPORTED_UNSUPPORTED ||
-          psc == DDCRC_DETERMINED_UNSUPPORTED)
-      {
-         dh->dref->flags |= DREF_DDC_COMMUNICATION_WORKING;
+      DDCA_IO_Mode io_mode = dh->dref->io_path.io_mode;
+      if (io_mode == DDCA_IO_USB) {
+         if (psc == 0 || psc == DDCRC_DETERMINED_UNSUPPORTED) {
+            dh->dref->flags |= DREF_DDC_COMMUNICATION_WORKING;
 
-         if (psc == DDCRC_NULL_RESPONSE || psc == DDCRC_ALL_RESPONSES_NULL)
-            dh->dref->flags |= DREF_DDC_USES_NULL_RESPONSE_FOR_UNSUPPORTED;
+         }
+         assert( (psc == 0 && pvalrec) || (psc != 0 && !pvalrec) );
+
+      }
+      else {
+         assert(psc != DDCRC_DETERMINED_UNSUPPORTED);  // only set at higher levels, unless USB
+
+         // What if returns -EIO?  Dell AW3418D returns -EIO for unsupported features
+         // EXCEPT that it returns mh=ml=sh=sl=0 for feature 0x00  (2/2019)
+
+#ifdef FORCE_SUCCESS
+         // for testing:
+         if (psc == DDCRC_RETRIES) {
+            DBGTRC(debug, TRACE_GROUP, "Forcing DDC communication success");
+            psc = 0;
+            dh->dref->flags |= DREF_DDC_COMMUNICATION_WORKING;
+            dh->dref->flags |= DREF_DDC_DOES_NOT_INDICATE_UNSUPPORTED;
+            dh->dref->flags |= DREF_DDC_COMMUNICATION_CHECKED;
+            dh->dref->flags |= DREF_DDC_NULL_RESPONSE_CHECKED;    // redundant with refactoring
+            goto bye;
+         }
+#endif
+
+
+         if ( psc == DDCRC_NULL_RESPONSE        ||
+              psc == DDCRC_ALL_RESPONSES_NULL   ||
+              psc == 0                          ||
+              psc == DDCRC_REPORTED_UNSUPPORTED )
+         {
+            dh->dref->flags |= DREF_DDC_COMMUNICATION_WORKING;
+
+            if (psc == DDCRC_REPORTED_UNSUPPORTED)
+               dh->dref->flags |= DREF_DDC_USES_DDC_FLAG_FOR_UNSUPPORTED;
+
+            else if (psc == DDCRC_NULL_RESPONSE || psc == DDCRC_ALL_RESPONSES_NULL)
+               dh->dref->flags |= DREF_DDC_USES_NULL_RESPONSE_FOR_UNSUPPORTED;
+
+            else {
+               assert( psc == 0);
+               assert(pvalrec);
+
+            }
+            assert( (psc == 0 && pvalrec) || (psc != 0 && !pvalrec) );
+         }
+
+         if (psc == 0) {
+
+            assert( pvalrec->value_type == DDCA_NON_TABLE_VCP_VALUE );
+            if (debug || IS_TRACING()) {
+               DBGMSG("pvalrec:");
+               dbgrpt_single_vcp_value(pvalrec, 1);
+            }
+
+            DBGTRC(debug, TRACE_GROUP, "value_type=%d, mh=%d, ml=%d, sh=%d, sl=%d",
+                     pvalrec->value_type,
+                     pvalrec->val.c_nc.mh,
+                     pvalrec->val.c_nc.ml,
+                     pvalrec->val.c_nc.sh,
+                     pvalrec->val.c_nc.sl);
+            if ( pvalrec->val.c_nc.mh == 0 &&
+                 pvalrec->val.c_nc.ml == 0 &&
+                 pvalrec->val.c_nc.sh == 0 &&
+                 pvalrec->val.c_nc.sl == 0
+              )
+            {
+               DBGTRC(debug, TRACE_GROUP, "Setting DREF_DDC_USES_MH_ML_SH_SL_ZERO_FOR_UNSUPPORTED");
+               dh->dref->flags |= DREF_DDC_USES_MH_ML_SH_SL_ZERO_FOR_UNSUPPORTED;
+            }
+            else {
+               DBGTRC(debug, TRACE_GROUP, "Setting DREF_DDC_DOES_NOT_INDICATE_UNSUPPORTED");
+               dh->dref->flags |= DREF_DDC_DOES_NOT_INDICATE_UNSUPPORTED;
+            }
+         }
       }
       dh->dref->flags |= DREF_DDC_COMMUNICATION_CHECKED;
       dh->dref->flags |= DREF_DDC_NULL_RESPONSE_CHECKED;    // redundant with refactoring
    }
+
+#ifdef FORCE_SUCCESS
+bye:
+   ;   // o.w. error that a label can only be part of a statement
+#endif
    bool communication_working = dh->dref->flags & DREF_DDC_COMMUNICATION_WORKING;
 
-   // commented out - defer checking version until actually needed to avoid
+   // Would prefer to defer checking version until actually needed to avoid
    // additional DDC io during monitor detection
-   // if (communication_working) {
-      // if ( vcp_version_is_unqueried(dh->dref->vcp_version)) {
-      //    dh->dref->vcp_version = get_vcp_version_by_display_handle(dh);
-      //    // dh->vcp_version = dh->dref->vcp_version;
-      // }
-   // }
+   // Unfortunately, introduces ddc_open_display(), with possible error states,
+   // into other functions, e.g. ddca_get_feature_list_by_dref()
+   if (communication_working) {
+      if ( vcp_version_eq(dh->dref->vcp_version, DDCA_VSPEC_UNQUERIED)) {
+         dh->dref->vcp_version = get_vcp_version_by_display_handle(dh);
+      }
+   }
 
-   DBGMSF(debug, "Returning: %s", bool_repr(communication_working));
+   DBGTRC(debug, TRACE_GROUP, "Returning: %s", bool_repr(communication_working));
    return communication_working;
 }
 
@@ -374,7 +292,7 @@ static char * get_firmware_version_string_t(Display_Handle * dh) {
    static GPrivate  firmware_version_key = G_PRIVATE_INIT(g_free);
    char * version = get_thread_fixed_buffer(&firmware_version_key, 40);
 
-   Single_Vcp_Value * valrec = NULL;
+   DDCA_Any_Vcp_Value * valrec = NULL;
    Public_Status_Code psc = 0;
    Error_Info * ddc_excp = ddc_get_vcp_value(
                                dh,
@@ -390,7 +308,7 @@ static char * get_firmware_version_string_t(Display_Handle * dh) {
       }
    }
    else {
-      g_snprintf(version, 40, "%d.%d", valrec->val.nc.sh, valrec->val.nc.sl);
+      g_snprintf(version, 40, "%d.%d", valrec->val.c_nc.sh, valrec->val.c_nc.sl);
       free_single_vcp_value(valrec);
    }
    return version;
@@ -408,40 +326,46 @@ static char * get_firmware_version_string_t(Display_Handle * dh) {
  */
 static char * get_controller_mfg_string_t(Display_Handle * dh) {
    bool debug = false;
+   DBGMSF(debug, "Starting. dh = %s", dh_repr(dh));
 
    const int MFG_NAME_BUF_SIZE = 100;
 
-   static GPrivate  controller_mfg_key = G_PRIVATE_INIT(g_free);
-   char * mfg_name_buf = get_thread_fixed_buffer(&controller_mfg_key, MFG_NAME_BUF_SIZE);
+   static GPrivate  buf_key = G_PRIVATE_INIT(g_free);
+   char * mfg_name_buf = get_thread_fixed_buffer(&buf_key, MFG_NAME_BUF_SIZE);
 
    char * mfg_name = NULL;
-   Single_Vcp_Value * valrec;
+   DDCA_Any_Vcp_Value * valrec;
 
-   Public_Status_Code psc = 0;
+   DDCA_Status ddcrc = 0;
    Error_Info * ddc_excp = ddc_get_vcp_value(dh, 0xc8, DDCA_NON_TABLE_VCP_VALUE, &valrec);
-   psc = (ddc_excp) ? ddc_excp->status_code : 0;
+   ddcrc = (ddc_excp) ? ddc_excp->status_code : 0;
 
-   if (psc == 0) {
+   if (ddcrc == 0) {
       DDCA_Feature_Value_Entry * vals = pxc8_display_controller_type_values;
-      mfg_name =  get_feature_value_name(
+      mfg_name =  sl_value_table_lookup(
                             vals,
-                            valrec->val.nc.sl);
+                            valrec->val.c_nc.sl);
       if (!mfg_name) {
          g_snprintf(mfg_name_buf, MFG_NAME_BUF_SIZE,
-                       "Unrecognized manufacturer code 0x%02x", valrec->val.nc.sl);
+                       "Unrecognized manufacturer code 0x%02x",
+                       valrec->val.c_nc.sl);
+
          mfg_name = mfg_name_buf;
       }
       free_single_vcp_value(valrec);
    }
-   else if (psc == DDCRC_REPORTED_UNSUPPORTED || psc == DDCRC_DETERMINED_UNSUPPORTED) {
+   else if (ddcrc == DDCRC_REPORTED_UNSUPPORTED || ddcrc == DDCRC_DETERMINED_UNSUPPORTED) {
       mfg_name = "Unspecified";
    }
    else {
-      DBGMSF(debug, "get_nontable_vcp_value(0xc8) returned %s", psc_desc(psc));
-      if (debug)
+      if (debug) {
+         DBGMSG("get_nontable_vcp_value(0xc8) returned %s", psc_desc(ddcrc));
          DBGMSG("    Try errors: %s", errinfo_causes_string(ddc_excp));
+      }
       mfg_name = "DDC communication failed";
     }
+
+   DBGMSF(debug, "Returning: %s", mfg_name);
    return mfg_name;
 }
 
@@ -506,6 +430,7 @@ ddc_report_display_by_dref(Display_Ref * dref, int depth) {
       if (!(dref->flags & DREF_DDC_COMMUNICATION_WORKING) ) {
          rpt_vstring(d1, "DDC communication failed");
          if (output_level >= DDCA_OL_VERBOSE) {
+#ifdef OLD
             if (streq(dref->pedid->model_name,   "Unspecified") &&
                 streq(dref->pedid->serial_ascii, "Unspecified") )
             {
@@ -513,6 +438,19 @@ ddc_report_display_by_dref(Display_Ref * dref, int depth) {
             }
             else
                rpt_vstring(d1, "Is DDC/CI enabled in the monitor's on-screen display?");
+#endif
+
+            char * msg = "Is DDC/CI enabled in the monitor's on-screen display?";
+            if (dref->io_path.io_mode == DDCA_IO_I2C) {
+               I2C_Bus_Info * curinfo = dref->detail;
+               if (curinfo->flags & I2C_BUS_EDP)
+                  msg = "This is a eDP laptop display. Laptop displays do not support DDC/CI.";
+               else if ( is_embedded_parsed_edid(dref->pedid) )
+                  msg = "This appears to be a laptop display. Laptop displays do not support DDC/CI.";
+               else
+                  msg = "Is DDC/CI enabled in the monitor's on-screen display?";
+            }
+            rpt_vstring(d1, msg);
          }
       }
       else {
@@ -538,9 +476,12 @@ ddc_report_display_by_dref(Display_Ref * dref, int depth) {
                ddc_close_display(dh);
             }
 
-            if (dref->io_path.io_mode != DDCA_IO_USB)
+            if (dref->io_path.io_mode != DDCA_IO_USB) {
                rpt_vstring(d1, "Monitor returns DDC Null Response for unsupported features: %s",
                                   bool_repr(dref->flags & DREF_DDC_USES_NULL_RESPONSE_FOR_UNSUPPORTED));
+               // rpt_vstring(d1, "Monitor returns success with mh=ml=sh=sl=0 for unsupported features: %s",
+               //                    bool_repr(dref->flags & DREF_DDC_USES_MH_ML_SH_SL_ZERO_FOR_UNSUPPORTED));
+            }
          }
       }
    }
@@ -563,7 +504,7 @@ ddc_report_display_by_dref(Display_Ref * dref, int depth) {
 int
 ddc_report_displays(bool include_invalid_displays, int depth) {
    bool debug = false;
-   DBGMSF0(debug, "Starting");
+   DBGMSF(debug, "Starting");
 
    ddc_ensure_displays_detected();
 
@@ -792,7 +733,7 @@ bye:
  */
 void async_scan(GPtrArray * all_displays) {
    bool debug = false;
-   DBGMSF(debug, "Starting. all_displays=%p, display_count=%d", all_displays, all_displays->len);
+   DBGTRC(debug, TRACE_GROUP, "Starting. all_displays=%p, display_count=%d", all_displays, all_displays->len);
 
    GPtrArray * threads = g_ptr_array_new();
    for (int ndx = 0; ndx < all_displays->len; ndx++) {
@@ -811,7 +752,7 @@ void async_scan(GPtrArray * all_displays) {
       GThread * thread = g_ptr_array_index(threads, ndx);
       g_thread_join(thread);  // implicitly unrefs the GThread
    }
-   DBGMSF0(debug, "Threads joined");
+   DBGMSF(debug, "Threads joined");
    g_ptr_array_free(threads, true);
 
 #ifdef OLD
@@ -826,12 +767,13 @@ void async_scan(GPtrArray * all_displays) {
       }
    }
 #endif
-   DBGMSF0(debug, "Done");
+   DBGTRC(debug, TRACE_GROUP, "Done");
 }
 
+
 void non_async_scan(GPtrArray * all_displays) {
    bool debug = false;
-   DBGMSF(debug, "Starting. checking %d displays", all_displays->len);
+   DBGTRC(debug, TRACE_GROUP, "Starting. checking %d displays", all_displays->len);
 
    for (int ndx = 0; ndx < all_displays->len; ndx++) {
       Display_Ref * dref = g_ptr_array_index(all_displays, ndx);
@@ -847,7 +789,7 @@ void non_async_scan(GPtrArray * all_displays) {
       }
 #endif
    }
-   DBGMSF0(debug, "Done");
+   DBGTRC(debug, TRACE_GROUP, "Done");
 }
 
 
@@ -868,10 +810,11 @@ ddc_find_display_ref_by_criteria(Display_Criteria * criteria) {
 
 
 /** Searches the master display list for a display matching the
- * specified #Display_Identifier, returning its #Display_Ref
+ *  specified #Display_Identifier, returning its #Display_Ref
  *
- * \param did display identifier to search for
- * \return #Display_Ref for the display.
+ *  \param did display identifier to search for
+ *  \return #Display_Ref for the display, NULL if not found or
+ *          display doesn't support DDC
  *
  * \remark
  * The returned value is a pointer into an internal data structure
@@ -880,7 +823,7 @@ ddc_find_display_ref_by_criteria(Display_Criteria * criteria) {
 Display_Ref *
 ddc_find_display_ref_by_display_identifier(Display_Identifier * did) {
    bool debug = false;
-   DBGMSF0(debug, "Starting");
+   DBGMSF(debug, "Starting");
    if (debug)
       dbgrpt_display_identifier(did, 1);
 
@@ -913,14 +856,13 @@ ddc_find_display_ref_by_display_identifier(Display_Identifier * did) {
       break;
    case DISP_ID_HIDDEV:
       criteria->hiddev = did->hiddev_devno;
-
    }
 
    result = ddc_find_display_ref_by_criteria(criteria);
 
    // Is this the best location in the call chain to make this check?
    if (result && (result->dispno < 0)) {
-      DBGMSF0(debug, "Found a display that doesn't support DDC.  Ignoring.");
+      DBGMSF(debug, "Found a display that doesn't support DDC.  Ignoring.");
       result = NULL;
    }
 
@@ -969,10 +911,11 @@ get_display_ref_for_display_identifier(
  *
  * \return array of #Display_Ref
  */
+static
 GPtrArray *
 ddc_detect_all_displays() {
    bool debug = false;
-   DBGMSF0(debug, "Starting");
+   DBGTRC(debug, TRACE_GROUP, "Starting");
 
    GPtrArray * display_list = g_ptr_array_new();
 
@@ -985,6 +928,11 @@ ddc_detect_all_displays() {
          Display_Ref * dref = create_bus_display_ref(businfo->busno);
          dref->dispno = -1;
          dref->pedid = businfo->edid;    // needed?
+         dref->mmid  = monitor_model_key_new(
+                          dref->pedid->mfg_id,
+                          dref->pedid->model_name,
+                          dref->pedid->product_code);
+
          // drec->detail.bus_detail = businfo;
          dref->detail = businfo;
          dref->flags |= DREF_DDC_IS_MONITOR_CHECKED;
@@ -1000,6 +948,10 @@ ddc_detect_all_displays() {
      Display_Ref * dref = create_adl_display_ref(detail->iAdapterIndex, detail->iDisplayIndex);
      dref->dispno = -1;
      dref->pedid = detail->pEdid;   // needed?
+     dref->mmid  = monitor_model_key_new(
+                      dref->pedid->mfg_id,
+                      dref->pedid->model_name,
+                      dref->pedid->product_code);
      // drec->detail.adl_detail = detail;
      dref->detail = detail;
      dref->flags |= DREF_DDC_IS_MONITOR_CHECKED;
@@ -1012,25 +964,35 @@ ddc_detect_all_displays() {
   g_ptr_array_free(all_adl_details, true);
 
 #ifdef USE_USB
-   GPtrArray * usb_monitors = get_usb_monitor_list();
-   // DBGMSF(debug, "Found %d USB displays", usb_monitors->len);
-   for (int ndx=0; ndx<usb_monitors->len; ndx++) {
-      Usb_Monitor_Info  * curmon = g_ptr_array_index(usb_monitors,ndx);
-      assert(memcmp(curmon->marker, USB_MONITOR_INFO_MARKER, 4) == 0);
-      Display_Ref * dref = create_usb_display_ref(
-                                curmon->hiddev_devinfo->busnum,
-                                curmon->hiddev_devinfo->devnum,
-                                curmon->hiddev_device_name);
-      dref->dispno = -1;
-      dref->pedid = curmon->edid;
-      // drec->detail.usb_detail = curmon;
-      dref->detail = curmon;
-      dref->flags |= DREF_DDC_IS_MONITOR_CHECKED;
-      dref->flags |= DREF_DDC_IS_MONITOR;
-      g_ptr_array_add(display_list, dref);
+   if (detect_usb_displays) {
+      GPtrArray * usb_monitors = get_usb_monitor_list();
+      // DBGMSF(debug, "Found %d USB displays", usb_monitors->len);
+      for (int ndx=0; ndx<usb_monitors->len; ndx++) {
+         Usb_Monitor_Info  * curmon = g_ptr_array_index(usb_monitors,ndx);
+         assert(memcmp(curmon->marker, USB_MONITOR_INFO_MARKER, 4) == 0);
+         Display_Ref * dref = create_usb_display_ref(
+                                   curmon->hiddev_devinfo->busnum,
+                                   curmon->hiddev_devinfo->devnum,
+                                   curmon->hiddev_device_name);
+         dref->dispno = -1;
+         dref->pedid = curmon->edid;
+         if (dref->pedid)
+            dref->mmid  = monitor_model_key_new(
+                             dref->pedid->mfg_id,
+                             dref->pedid->model_name,
+                             dref->pedid->product_code);
+         else
+            dref->mmid = monitor_model_key_new("UNK", "UNK", 0);
+         // drec->detail.usb_detail = curmon;
+         dref->detail = curmon;
+         dref->flags |= DREF_DDC_IS_MONITOR_CHECKED;
+         dref->flags |= DREF_DDC_IS_MONITOR;
+         g_ptr_array_add(display_list, dref);
+      }
    }
 #endif
 
+
    // verbose output is distracting within scans
    // saved and reset here so that async threads are not adjusting output level
    DDCA_Output_Level olev = get_output_level();
@@ -1041,7 +1003,6 @@ ddc_detect_all_displays() {
                  display_list->len, async_threshold, adlct);
    // ADL displays do not support async scan.  Not worth implementing.
    if (display_list->len >= async_threshold && adlct == 0)
-   // if (true)
       async_scan(display_list);
    else
       non_async_scan(display_list);
@@ -1068,7 +1029,7 @@ ddc_detect_all_displays() {
    //    DBGMSG("Displays detected:");
    //    report_display_recs(display_list, 1);
    // }
-   DBGMSF(debug, "Done. Detected %d valid displays", dispno_max);
+   DBGTRC(debug, TRACE_GROUP, "Done. Detected %d valid displays", dispno_max);
    return display_list;
 }
 
@@ -1085,3 +1046,52 @@ ddc_ensure_displays_detected() {
    }
 }
 
+
+/** Indicates whether displays have already been detected
+ *
+ *  @return true/false
+ */
+bool
+ddc_displays_already_detected()
+{
+   return all_displays;
+}
+
+
+/** Controls whether USB displays are to be detected.
+ *
+ *  Must be called before any function that triggers display detection.
+ *
+ *  @param  onoff  true if USB displays are to be detected, false if not
+ *  @retval DDCRC_OK  normal
+ *  @retval DDCRC_INVALID_OPERATION function called after displays have been detected
+ *  @retval DDCRC_UNIMPLEMENTED ddcutil was not built with USB support
+ *
+ *  @remark
+ *  If this function is not called, the default (if built with USB support) is on
+ */
+DDCA_Status
+ddc_enable_usb_display_detection(bool onoff) {
+#ifdef USE_USB
+   DDCA_Status rc = DDCRC_INVALID_OPERATION;
+   if (!ddc_displays_already_detected()) {
+      detect_usb_displays = onoff;
+      rc = DDCRC_OK;
+      // DBGMSG("detect_usb_displays = %s", sbool(detect_usb_displays));
+   }
+   return rc;
+#else
+   return DDCRC_UNIMPLEMENTED;
+#endif
+}
+
+
+/** Indicates whether USB displays are to be detected
+ *
+ *  @return true/false
+ */
+bool
+ddc_is_usb_display_detection_enabled() {
+   return detect_usb_displays;
+}
+
