@@ -4,7 +4,7 @@
  * Capabilities.
  *
  * <copyright>
- * Copyright (C) 2014-2018 Sanford Rockowitz <rockowitz@minsoft.com>
+ * Copyright (C) 2014-2019 Sanford Rockowitz <rockowitz@minsoft.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -53,7 +53,7 @@
 
 // Trace management
 
-static Trace_Group TRACE_GROUP = TRC_DDC;
+static DDCA_Trace_Group TRACE_GROUP = DDCA_TRC_DDC;
 
 
 // Retry management and statistics
@@ -79,6 +79,7 @@ void ddc_reset_multi_part_read_stats() {
       multi_part_read_stats_rec = try_data_create("multi-part read exchange", max_multi_part_read_tries);
 }
 
+
 /** Resets the statistics for multi-part writes */
 void ddc_reset_multi_part_write_stats() {
    if (multi_part_write_stats_rec)
@@ -87,12 +88,14 @@ void ddc_reset_multi_part_write_stats() {
       multi_part_write_stats_rec = try_data_create("multi-part write exchange", max_multi_part_write_tries);
 }
 
+
 /** Reports the statistics for multi-part reads */
 void ddc_report_multi_part_read_stats(int depth) {
    assert(multi_part_read_stats_rec);
    try_data_report(multi_part_read_stats_rec, depth);
 }
 
+
 /** Reports the statistics for multi-part writes */
 void ddc_report_multi_part_write_stats(int depth) {
    assert(multi_part_write_stats_rec);
@@ -128,6 +131,7 @@ int ddc_get_max_multi_part_read_tries() {
    return max_multi_part_read_tries;
 }
 
+
 /** Gets the current maximum number of multi-part write exchange tries allowed
   * @return maximum number of tries
   */
@@ -147,7 +151,7 @@ int ddc_get_max_multi_part_write_tries() {
 *
 * @return status code
 */
-Error_Info *
+static Error_Info *
 try_multi_part_read(
       Display_Handle * dh,
       Byte             request_type,
@@ -174,7 +178,7 @@ try_multi_part_read(
    int  cur_offset = 0;
    bool complete   = false;
    while (!complete && !excp) {         // loop over fragments
-      DBGTRC(debug, TRC_NONE, "Top of fragment loop", NULL);
+      DBGTRC(debug, DDCA_TRC_NONE, "Top of fragment loop");
 
       int fragment_size;
       update_ddc_multi_part_read_request_packet_offset(request_packet_ptr, cur_offset);
@@ -193,7 +197,7 @@ try_multi_part_read(
            &response_packet_ptr
           );
       psc = (excp) ? excp->status_code : 0;
-      DBGTRC(debug, TRC_NONE,
+      DBGTRC(debug, DDCA_TRC_NONE,
              "ddc_write_read_with_retry() request_type=0x%02x, request_subtype=0x%02x, returned %s",
              request_type, request_subtype, errinfo_summary(excp));
 
@@ -216,17 +220,17 @@ try_multi_part_read(
 
       int display_current_offset = aux_data_ptr->fragment_offset;
       if (display_current_offset != cur_offset) {
-         DBGTRC(debug, TRC_NONE,
+         DBGTRC(debug, DDCA_TRC_NONE,
                 "display_current_offset %d != cur_offset %d", display_current_offset, cur_offset);
          psc = DDCRC_MULTI_PART_READ_FRAGMENT;
          excp = errinfo_new(psc, __func__);
          COUNT_STATUS_CODE(psc);
       }
       else {
-         DBGTRC(debug, TRC_NONE, "display_current_offset = %d matches cur_offset", display_current_offset);
+         DBGTRC(debug, DDCA_TRC_NONE, "display_current_offset = %d matches cur_offset", display_current_offset);
 
          fragment_size = aux_data_ptr->fragment_length;         // ***
-         DBGTRC(debug, TRC_NONE, "fragment_size = %d", fragment_size);
+         DBGTRC(debug, DDCA_TRC_NONE, "fragment_size = %d", fragment_size);
          if (fragment_size == 0) {
             complete = true;   // redundant
          }
@@ -288,7 +292,7 @@ multi_part_read_with_retry(
    Buffer * accumulator = buffer_new(2048, "multi part read buffer");
 
    while (tryctr < max_multi_part_read_tries && rc < 0 && can_retry) {
-      DBGTRC(debug, TRC_NEVER,
+      DBGTRC(debug, DDCA_TRC_NONE,
              "Start of while loop. try_ctr=%d, max_multi_part_read_tries=%d",
              tryctr, max_multi_part_read_tries);
 
@@ -328,7 +332,7 @@ multi_part_read_with_retry(
       tryctr++;
    }
    assert( (rc<0 && ddc_excp) || (rc==0 && !ddc_excp) );
-   DBGTRC(debug, TRC_NEVER, "After try loop. tryctr=%d, rc=%d. ddc_excp=%p",
+   DBGTRC(debug, DDCA_TRC_NONE, "After try loop. tryctr=%d, rc=%d. ddc_excp=%p",
                             tryctr, rc, ddc_excp);
 
    if (debug) {
@@ -372,7 +376,7 @@ multi_part_read_with_retry(
 *
 *   @return status code
 */
-Error_Info *
+static Error_Info *
 try_multi_part_write(
       Display_Handle * dh,
       Byte             vcp_code,
