@@ -1,28 +1,9 @@
-/* ddc_output.c
- *
- * <copyright>
- * Copyright (C) 2014-2018 Sanford Rockowitz <rockowitz@minsoft.com>
- *
- * Licensed under the GNU General Public License Version 2
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- * </endcopyright>
+/** \file ddc_output.c
  */
 
-/** \file
- */
+// Copyright (C) 2014-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// SPDX-License-Identifier: GPL-2.0-or-later
+
 
 #include <config.h>
 
@@ -56,16 +37,19 @@
 
 #include "vcp/parse_capabilities.h"
 
+#include "dynvcp/dyn_feature_set.h"
+#include "dynvcp/dyn_feature_codes.h"
+
 #include "ddc/ddc_multi_part_io.h"
 #include "ddc/ddc_packet_io.h"
 #include "ddc/ddc_vcp.h"
 #include "ddc/ddc_vcp_version.h"
 
-#include <ddc/ddc_output.h>
+#include "ddc/ddc_output.h"
 
 
 // Trace class for this file
-static Trace_Group TRACE_GROUP = TRC_DDC;
+static DDCA_Trace_Group TRACE_GROUP = DDCA_TRC_DDC;
 
 
 // Standard format strings for reporting feature codes.
@@ -76,6 +60,7 @@ const char* FMT_CODE_NAME_DETAIL_W_NL  = "VCP code 0x%02x (%-30s): %s\n";
 // VCP Feature Table inquiry
 //
 
+#ifdef OLD
 /* Checks if a feature is a table type feature, given
  * the VCP version of a monitor.
  *
@@ -105,8 +90,10 @@ is_table_feature_by_display_handle(
    // DBGMSF(debug, "returning: %d", result);
    return result;
 }
+#endif
 
 
+#ifdef FUTURE
 // For possible future use - currently unused
 Public_Status_Code
 check_valid_operation_by_feature_rec_and_version(
@@ -142,6 +129,7 @@ check_valid_operation_by_feature_id_and_dh(
    }
    return result;
 }
+#endif
 
 
 //
@@ -162,12 +150,161 @@ check_valid_operation_by_feature_id_and_dh(
  * Returns:
  *    status code
  */
-Public_Status_Code
-get_raw_value_for_feature_table_entry(
+static Public_Status_Code
+get_raw_value_for_feature_metadata(
       Display_Handle *           dh,
-      VCP_Feature_Table_Entry *  frec,
+      Display_Feature_Metadata *    frec,
       bool                       ignore_unsupported,
-      Single_Vcp_Value **   pvalrec,
+      DDCA_Any_Vcp_Value ** pvalrec,
+      FILE *                     msg_fh)
+{
+   bool debug = false;
+   DBGTRC(debug, TRACE_GROUP, "Starting");
+
+   assert(dh);
+   assert(dh->dref);
+
+   Public_Status_Code psc = 0;
+   Error_Info * ddc_excp = NULL;
+
+   // DDCA_MCCS_Version_Spec vspec = get_vcp_version_by_display_handle(dh);
+   // char * feature_name = get_version_sensitive_feature_name(frec, vspec);
+   char * feature_name = frec->feature_name;
+
+   Byte feature_code = frec->feature_code;
+   // bool is_table_feature = is_table_feature_by_display_handle(frec, dh);
+   bool is_table_feature = frec->feature_flags & DDCA_TABLE;
+   DDCA_Vcp_Value_Type feature_type = (is_table_feature) ? DDCA_TABLE_VCP_VALUE : DDCA_NON_TABLE_VCP_VALUE;
+   DDCA_Output_Level output_level = get_output_level();
+   DDCA_Any_Vcp_Value * valrec = NULL;
+   if (dh->dref->io_path.io_mode == DDCA_IO_USB) {
+#ifdef USE_USB
+     psc = usb_get_vcp_value(
+              dh,
+              feature_code,
+              feature_type,
+              &valrec);
+     if (psc != 0)
+        ddc_excp = errinfo_new(psc, __func__);
+#else
+      PROGRAM_LOGIC_ERROR("ddcutil not built with USB support");
+#endif
+   }
+   else {
+      ddc_excp = ddc_get_vcp_value(
+              dh,
+              feature_code,
+              feature_type,
+              &valrec);
+      psc = ERRINFO_STATUS(ddc_excp);
+   }
+   assert ( (psc==0 && valrec) || (psc!=0 && !valrec) );
+
+   // For now, only regard -EIO as unsupported feature for the
+   // single model on which this has been observed
+   if (psc ==  -EIO &&
+       dh->dref->io_path.io_mode == DDCA_IO_I2C &&
+       streq(dh->dref->pedid->mfg_id, "DEL")    &&
+       streq(dh->dref->pedid->model_name, "AW3418DW") )
+   {
+      // Dell AW3418DW returns -EIO for unsupported features
+      // (except for feature 0x00, which returns mh=ml=sh=sl=0) (2/2019)
+      if (!ignore_unsupported) {
+         f0printf(msg_fh, FMT_CODE_NAME_DETAIL_W_NL,
+                        feature_code, feature_name, "Unsupported feature code (EIO)");
+      }
+      psc = DDCRC_DETERMINED_UNSUPPORTED;
+      COUNT_STATUS_CODE(DDCRC_DETERMINED_UNSUPPORTED);
+   }
+
+   else {
+      switch(psc) {
+      case 0:
+         break;
+
+      case DDCRC_DDC_DATA:           // was DDCRC_INVALID_DATA
+         if (output_level >= DDCA_OL_NORMAL)
+            f0printf(msg_fh, FMT_CODE_NAME_DETAIL_W_NL,
+                            feature_code, feature_name, "Invalid response");
+         break;
+
+      case DDCRC_NULL_RESPONSE:
+         // for unsupported features, some monitors return null response rather than a valid response
+         // with unsupported feature indicator set
+         if (!ignore_unsupported) {
+            f0printf(msg_fh, FMT_CODE_NAME_DETAIL_W_NL,
+                           feature_code, feature_name, "Unsupported feature code (Null response)");
+         }
+         COUNT_STATUS_CODE(DDCRC_DETERMINED_UNSUPPORTED);
+         psc = DDCRC_DETERMINED_UNSUPPORTED;
+         break;
+
+      case DDCRC_READ_ALL_ZERO:
+         // treat as invalid response if not table type?
+         if (!ignore_unsupported) {
+            f0printf(msg_fh, FMT_CODE_NAME_DETAIL_W_NL,
+                           feature_code, feature_name, "Unsupported feature code (All zero response)");
+         }
+         psc = DDCRC_DETERMINED_UNSUPPORTED;
+         COUNT_STATUS_CODE(DDCRC_DETERMINED_UNSUPPORTED);
+         break;
+
+#ifdef FUTURE
+      case -EIO:
+         // Dell AW3418DW returns -EIO for unsupported features
+         // (except for feature 0x00, which returns mh=ml=sh=sl=0) (2/2019)
+         if (!ignore_unsupported) {
+            f0printf(msg_fh, FMT_CODE_NAME_DETAIL_W_NL,
+                           feature_code, feature_name, "Unsupported feature code (EIO)");
+         }
+         psc = DDCRC_DETERMINED_UNSUPPORTED;
+         COUNT_STATUS_CODE(DDCRC_DETERMINED_UNSUPPORTED);
+         break;
+#endif
+
+      case DDCRC_RETRIES:
+         f0printf(msg_fh, FMT_CODE_NAME_DETAIL_W_NL,
+                         feature_code, feature_name, "Maximum retries exceeded");
+         break;
+
+      case DDCRC_REPORTED_UNSUPPORTED:
+      case DDCRC_DETERMINED_UNSUPPORTED:
+         if (!ignore_unsupported) {
+            f0printf(msg_fh, FMT_CODE_NAME_DETAIL_W_NL,
+                            feature_code, feature_name, "Unsupported feature code");
+         }
+         break;
+
+      default:
+      {
+         char buf[200];
+         snprintf(buf, 200, "Invalid response. status code=%s, %s", psc_desc(psc), dh_repr_t(dh));
+         f0printf(msg_fh, FMT_CODE_NAME_DETAIL_W_NL,
+                          feature_code, feature_name, buf);
+      }
+      }
+
+   }
+
+   *pvalrec = valrec;
+   DBGTRC(debug, TRACE_GROUP, "Done.  Returning: %s, *pvalrec=%p", psc_desc(psc), *pvalrec);
+   assert( (psc == 0 && *pvalrec) || (psc != 0 && !*pvalrec) );
+   if (*pvalrec && (debug || IS_TRACING())) {
+      dbgrpt_single_vcp_value(*pvalrec, 1);
+   }
+   if (ddc_excp) {
+      ERRINFO_FREE_WITH_REPORT(ddc_excp, debug || IS_TRACING() || report_freed_exceptions);
+   }
+   return psc;
+}
+
+#ifdef IN_PROGREESS
+Public_Status_Code
+get_raw_value_for_feature_metadata_dfm(
+      Display_Handle *           dh,
+      DDCA_Feature_Metadata *    frec,
+      bool                       ignore_unsupported,
+      DDCA_Any_Vcp_Value ** pvalrec,
       FILE *                     msg_fh)
 {
    bool debug = false;
@@ -179,14 +316,16 @@ get_raw_value_for_feature_table_entry(
    Public_Status_Code psc = 0;
    Error_Info * ddc_excp = NULL;
 
-   DDCA_MCCS_Version_Spec vspec = get_vcp_version_by_display_handle(dh);
-   char * feature_name = get_version_sensitive_feature_name(frec, vspec);
+   // DDCA_MCCS_Version_Spec vspec = get_vcp_version_by_display_handle(dh);
+   // char * feature_name = get_version_sensitive_feature_name(frec, vspec);
+   char * feature_name = frec->feature_name;
 
-   Byte feature_code = frec->code;
-   bool is_table_feature = is_table_feature_by_display_handle(frec, dh);
+   Byte feature_code = frec->feature_code;
+   // bool is_table_feature = is_table_feature_by_display_handle(frec, dh);
+   bool is_table_feature = frec->feature_flags & DDCA_TABLE;
    DDCA_Vcp_Value_Type feature_type = (is_table_feature) ? DDCA_TABLE_VCP_VALUE : DDCA_NON_TABLE_VCP_VALUE;
    DDCA_Output_Level output_level = get_output_level();
-   Single_Vcp_Value * valrec = NULL;
+   DDCA_Any_Vcp_Value * valrec = NULL;
    if (dh->dref->io_path.io_mode == DDCA_IO_USB) {
 #ifdef USE_USB
      psc = usb_get_vcp_value(
@@ -281,6 +420,8 @@ get_raw_value_for_feature_table_entry(
    }
    return psc;
 }
+#endif
+
 
 
 /* Gather values for the features in a feature set.
@@ -296,32 +437,37 @@ get_raw_value_for_feature_table_entry(
  *    status code
  */
 Public_Status_Code
-collect_raw_feature_set_values(
+collect_raw_feature_set_values2_dfm(
       Display_Handle *      dh,
-      VCP_Feature_Set       feature_set,
+      Dyn_Feature_Set*       feature_set,
       Vcp_Value_Set         vset,
       bool                  ignore_unsupported,  // if false, is error if unsupported
       FILE *                msg_fh)
 {
-   Public_Status_Code master_status_code = 0;
    bool debug = false;
-   DBGMSF0(debug, "Starting.");
-   int features_ct = get_feature_set_size(feature_set);
+   DBGMSF(debug, "Starting.");
+
+   Public_Status_Code master_status_code = 0;
+   int features_ct = dyn_get_feature_set_size2_dfm(feature_set);
    // needed when called from C API, o.w. get get NULL response for first feature
    // DBGMSG("Inserting sleep() before first call to get_raw_value_for_feature_table_entry()");
    // sleep_millis_with_trace(DDC_TIMEOUT_MILLIS_DEFAULT, __func__, "initial");
    int ndx;
    for (ndx=0; ndx< features_ct; ndx++) {
-      VCP_Feature_Table_Entry * entry = get_feature_set_entry(feature_set, ndx);
-      DBGMSF(debug,"ndx=%d, feature = 0x%02x", ndx, entry->code);
-      Single_Vcp_Value *    pvalrec;
+      Display_Feature_Metadata * dfm = dyn_get_feature_set_entry2_dfm(feature_set, ndx);
+      DBGMSF(debug,"ndx=%d, feature = 0x%02x", ndx, dfm->feature_code);
+      DDCA_Any_Vcp_Value *  pvalrec;
+      // DDCA_Feature_Metadata * ddca_meta = dfm_to_ddca_feature_metadata(dfm);
+
       Public_Status_Code cur_status_code =
-       get_raw_value_for_feature_table_entry(
-         dh,
-         entry,
-         ignore_unsupported,
-         &pvalrec,
-         msg_fh);
+            get_raw_value_for_feature_metadata(
+                  dh,
+                  dfm,    // ddca_meta,
+                  ignore_unsupported,
+                  &pvalrec,
+                   msg_fh);
+      // todo: free ddca_meta
+
       if (cur_status_code == 0) {
          vcp_value_set_add(vset, pvalrec);
       }
@@ -355,31 +501,36 @@ collect_raw_feature_set_values(
  *    status code
  */
 Public_Status_Code
-collect_raw_subset_values(
+ddc_collect_raw_subset_values(
         Display_Handle *    dh,
         VCP_Feature_Subset  subset,
         Vcp_Value_Set       vset,
         bool                ignore_unsupported,
         FILE *              msg_fh)
 {
-   Public_Status_Code psc = 0;
    bool debug = false;
    DBGMSF(debug, "Starting.  subset=%d  dh=%s", subset, dh_repr(dh) );
-   DDCA_MCCS_Version_Spec vcp_version = get_vcp_version_by_display_handle(dh);
+
+   Public_Status_Code psc = 0;
+
+   // DDCA_MCCS_Version_Spec vcp_version = get_vcp_version_by_display_handle(dh);
    // DBGMSG("VCP version = %d.%d", vcp_version.major, vcp_version.minor);
-   VCP_Feature_Set feature_set = create_feature_set(
+
+   Dyn_Feature_Set * feature_set = dyn_create_feature_set2_dfm(
                                      subset,
-                                     vcp_version,
+                                     dh->dref,          // vcp_version,
                                      FSF_NOTABLE);
                                 //   false);      // exclude_table_features
    if (debug)
-      report_feature_set(feature_set, 0);
+      dbgrpt_dyn_feature_set(feature_set, true, 0);
 
-   psc = collect_raw_feature_set_values(
+   psc = collect_raw_feature_set_values2_dfm(
             dh, feature_set, vset,
             ignore_unsupported, msg_fh);
-   free_vcp_feature_set(feature_set);
-   DBGMSF0(debug, "Done");
+
+   dyn_free_feature_set(feature_set);
+
+   DBGMSF(debug, "Returning: %s", psc_desc(psc));
    return psc;
 }
 
@@ -392,7 +543,7 @@ collect_raw_subset_values(
  *  a formatted interpretation of the value.
  *
  * \param  dh         handle for open display
- * \param  vcp_entry  feature table entry
+ * \param  internal_metadata
  * \param  suppress_unsupported
  *                    if true, do not report unsupported features
  * \param  prefix_value_with_feature_code
@@ -408,24 +559,25 @@ collect_raw_subset_values(
  * earlier code.  It needs refactoring.
  */
 Public_Status_Code
-get_formatted_value_for_feature_table_entry(
-      Display_Handle *           dh,
-      VCP_Feature_Table_Entry *  vcp_entry,
-      bool                       suppress_unsupported,
-      bool                       prefix_value_with_feature_code,
-      char **                    formatted_value_loc,
-      FILE *                     msg_fh)
+ddc_get_formatted_value_for_display_feature_metadata(
+      Display_Handle *            dh,
+      Display_Feature_Metadata *  dfm,
+      bool                        suppress_unsupported,
+      bool                        prefix_value_with_feature_code,
+      char **                     formatted_value_loc,
+      FILE *                      msg_fh)
 {
    bool debug = false;
-   DBGTRC(debug, TRACE_GROUP, "Starting. suppress_unsupported=%s", bool_repr(suppress_unsupported));
+   DBGTRC(debug, TRACE_GROUP, "Starting. suppress_unsupported=%s", sbool(suppress_unsupported));
 
    Public_Status_Code psc = 0;
    *formatted_value_loc = NULL;
 
    DDCA_MCCS_Version_Spec vspec = get_vcp_version_by_display_handle(dh);
-   Byte feature_code = vcp_entry->code;
-   char * feature_name = get_version_sensitive_feature_name(vcp_entry, vspec);
-   bool is_table_feature = is_table_feature_by_display_handle(vcp_entry, dh);
+   // DDCA_Feature_Metadata* extmeta = dfm_to_ddca_feature_metadata(dfm);
+   Byte feature_code = dfm->feature_code;
+   char * feature_name = dfm->feature_name;
+   bool is_table_feature = dfm->feature_flags & DDCA_TABLE;
    DDCA_Vcp_Value_Type feature_type = (is_table_feature) ? DDCA_TABLE_VCP_VALUE : DDCA_NON_TABLE_VCP_VALUE;
    DDCA_Output_Level output_level = get_output_level();
    if (output_level >= DDCA_OL_VERBOSE) {
@@ -435,14 +587,14 @@ get_formatted_value_for_feature_table_entry(
                             feature_name);
    }
 
-   Single_Vcp_Value *    pvalrec = NULL;
+   DDCA_Any_Vcp_Value *  pvalrec = NULL;
 
    // bool ignore_unsupported = !(output_level >= DDCA_OL_NORMAL && !suppress_unsupported);
    bool ignore_unsupported = suppress_unsupported;
 
-   psc = get_raw_value_for_feature_table_entry(
+   psc = get_raw_value_for_feature_metadata(
             dh,
-            vcp_entry,
+            dfm,    // extmeta,
             ignore_unsupported,
             &pvalrec,
             (output_level == DDCA_OL_TERSE) ? NULL : msg_fh);
@@ -473,27 +625,27 @@ get_formatted_value_for_feature_table_entry(
             free(hexbuf);
          }
          else {                                // OL_PROGRAM, not table feature
-            DDCA_Version_Feature_Flags vflags =
-               get_version_sensitive_feature_flags(vcp_entry, vspec);
+            DDCA_Version_Feature_Flags vflags = dfm->feature_flags;
+            // =   get_version_sensitive_feature_flags(vcp_entry, vspec);
             char buf[200];
             assert(vflags & (DDCA_CONT | DDCA_SIMPLE_NC | DDCA_COMPLEX_NC | DDCA_NC_CONT));
             if (vflags & DDCA_CONT) {
                snprintf(buf, 200, "VCP %02X C %d %d",
-                                  vcp_entry->code,
-                                  pvalrec->val.c.cur_val, pvalrec->val.c.max_val);
+                                  feature_code,
+               VALREC_CUR_VAL(pvalrec), VALREC_MAX_VAL(pvalrec));
             }
             else if (vflags & DDCA_SIMPLE_NC) {
                snprintf(buf, 200, "VCP %02X SNC x%02x",
-                                   vcp_entry->code, pvalrec->val.nc.sl);
+               feature_code, pvalrec->val.c_nc.sl);
             }
             else {
                assert(vflags & (DDCA_COMPLEX_NC|DDCA_NC_CONT));
                snprintf(buf, 200, "VCP %02X CNC x%02x x%02x x%02x x%02x",
-                                  vcp_entry->code,
-                                  pvalrec->val.nc.mh,
-                                  pvalrec->val.nc.ml,
-                                  pvalrec->val.nc.sh,
-                                  pvalrec->val.nc.sl
+                                  feature_code,
+                                  pvalrec->val.c_nc.mh,
+                                  pvalrec->val.c_nc.ml,
+                                  pvalrec->val.c_nc.sh,
+                                  pvalrec->val.c_nc.sl
                                   );
             }
             *formatted_value_loc = strdup(buf);
@@ -504,8 +656,8 @@ get_formatted_value_for_feature_table_entry(
          bool ok;
          char * formatted_data = NULL;
 
-         ok = vcp_format_feature_detail(
-                 vcp_entry,
+         ok = dyn_format_feature_detail_dfm(
+                 dfm,
                  vspec,
                  pvalrec,
                  &formatted_data);
@@ -535,7 +687,7 @@ get_formatted_value_for_feature_table_entry(
    else {   // error
       // if output_level >= DDCA_OL_NORMAL, get_raw_value_for_feature_table_entry() already issued message
       if (output_level == DDCA_OL_TERSE && !suppress_unsupported) {
-         f0printf(msg_fh, "VCP %02X ERR\n", vcp_entry->code);
+         f0printf(msg_fh, "VCP %02X ERR\n", feature_code);
       }
    }
 
@@ -543,29 +695,29 @@ get_formatted_value_for_feature_table_entry(
       free_single_vcp_value(pvalrec);
 
    DBGTRC(debug, TRACE_GROUP,
-          "Done.  Returning: %s, *formatted_value_loc=|%s|",
+          "Done.  Returning: %s, *formatted_value_loc=%p",
           psc_desc(psc), formatted_value_loc);
    return psc;
 }
 
 
 Public_Status_Code
-show_feature_set_values(
+show_feature_set_values2_dfm(
       Display_Handle *      dh,
-      VCP_Feature_Set       feature_set,
+      Dyn_Feature_Set*      feature_set,
       GPtrArray *           collector,     // if null, write to current stdout device
       Feature_Set_Flags     flags,
       Byte_Value_Array      features_seen)     // if non-null, collect list of features seen
 {
-   Public_Status_Code master_status_code = 0;
    bool debug = false;
-   char * s0 = feature_set_flag_names(flags);
+   char * s0 = feature_set_flag_names_t(flags);
    DBGMSF(debug, "Starting.  flags=%s, collector=%p", s0, collector);
-   free(s0);
+
+   Public_Status_Code master_status_code = 0;
 
    FILE * outf = fout();
 
-   VCP_Feature_Subset subset_id = get_feature_set_subset_id(feature_set);
+   VCP_Feature_Subset subset_id = feature_set->subset;
    DDCA_Output_Level output_level = get_output_level();
    bool show_unsupported = false;
    if ( (flags & FSF_SHOW_UNSUPPORTED)  ||
@@ -575,23 +727,23 @@ show_feature_set_values(
        show_unsupported = true;
    bool suppress_unsupported = !show_unsupported;
 
-   DDCA_MCCS_Version_Spec vcp_version = get_vcp_version_by_display_handle(dh);
+   // DDCA_MCCS_Version_Spec vcp_version = get_vcp_version_by_display_handle(dh);
    bool prefix_value_with_feature_code = true;    // TO FIX
    FILE * msg_fh = outf;                        // TO FIX
-   int features_ct = get_feature_set_size(feature_set);
+   int features_ct = dyn_get_feature_set_size2_dfm(feature_set);
    DBGMSF(debug, "features_ct=%d", features_ct);
    int ndx;
    for (ndx=0; ndx< features_ct; ndx++) {
-      VCP_Feature_Table_Entry * entry = get_feature_set_entry(feature_set, ndx);
-      DBGMSF(debug,"ndx=%d, feature = 0x%02x", ndx, entry->code);
-      if (!is_feature_readable_by_vcp_version(entry, vcp_version)) {
+      Display_Feature_Metadata * dfm = dyn_get_feature_set_entry2_dfm(feature_set, ndx);
+      // DDCA_Feature_Metadata * extmeta = ifm->external_metadata;
+      DBGMSF(debug,"ndx=%d, feature = 0x%02x", ndx, dfm->feature_code);
+      if ( !(dfm->feature_flags & DDCA_READABLE) ) {
          // confuses the output if suppressing unsupported
          if (show_unsupported) {
-            char * feature_name =  get_version_sensitive_feature_name(entry, vcp_version);
-            DDCA_Version_Feature_Flags vflags = get_version_sensitive_feature_flags(entry, vcp_version);
-            char * msg = (vflags & DDCA_DEPRECATED) ? "Deprecated" : "Write-only feature";
+            char * feature_name =  dfm->feature_name;
+            char * msg = (dfm->feature_flags & DDCA_DEPRECATED) ? "Deprecated" : "Write-only feature";
             f0printf(outf, FMT_CODE_NAME_DETAIL_W_NL,
-                          entry->code, feature_name, msg);
+                          dfm->feature_code, feature_name, msg);
          }
       }
       else {
@@ -599,7 +751,7 @@ show_feature_set_values(
 #ifdef NO
          if (subset_id != VCP_SUBSET_SINGLE_FEATURE &&
              is_feature_table_by_vcp_version(entry, vcp_version) &&
-             (flags & FSF_NOTABLE) )
+             (feature_flags & FSF_NOTABLE) )
          {
             skip_feature = true;
          }
@@ -608,9 +760,9 @@ show_feature_set_values(
 
             char * formatted_value = NULL;
             Public_Status_Code psc =
-            get_formatted_value_for_feature_table_entry(
+            ddc_get_formatted_value_for_display_feature_metadata(
                   dh,
-                  entry,
+                  dfm,
                   suppress_unsupported,
                   prefix_value_with_feature_code,
                   &formatted_value,
@@ -623,11 +775,11 @@ show_feature_set_values(
                   f0printf(outf, "%s\n", formatted_value);
                free(formatted_value);
                if (features_seen)
-                  bbf_set(features_seen, entry->code);  // note that feature was read
+                  bbf_set(features_seen, dfm->feature_code);  // note that feature was read
             }
             else {
                // or should I check features_ct == 1?
-               VCP_Feature_Subset subset_id = get_feature_set_subset_id(feature_set);
+               VCP_Feature_Subset subset_id = feature_set->subset;
                if (subset_id == VCP_SUBSET_SINGLE_FEATURE)
                   master_status_code = psc;
                else {
@@ -639,13 +791,15 @@ show_feature_set_values(
             }
          }   // !skip_feature
       }
-      DBGMSF(debug,"ndx=%d, feature = 0x%02x Done", ndx, entry->code);
+      DBGMSF(debug,"ndx=%d, feature = 0x%02x Done", ndx, dfm->feature_code);
    }   // loop over features
 
    DBGMSF(debug, "Returning: %s", psc_desc(master_status_code));
    return master_status_code;
 }
 
+
+
 #ifdef FUTURE
 //typedef bool (*VCP_Feature_Set_Filter_Func)(VCP_Feature_Table_Entry * ventry);
 bool hack42(VCP_Feature_Table_Entry * ventry) {
@@ -665,6 +819,7 @@ bool hack42(VCP_Feature_Table_Entry * ventry) {
 #endif
 
 
+
 /* Shows the VCP values for all features in a VCP feature subset.
  *
  * Arguments:
@@ -678,28 +833,29 @@ bool hack42(VCP_Feature_Table_Entry * ventry) {
  *    status code
  */
 Public_Status_Code
-show_vcp_values(
+ddc_show_vcp_values(
         Display_Handle *    dh,
         VCP_Feature_Subset  subset,
         GPtrArray *         collector,    // not used
         Feature_Set_Flags   flags,
         Byte_Bit_Flags      features_seen)
 {
-   Public_Status_Code psc = 0;
    bool debug = false;
-   if (debug) {
-      char * s0 = feature_set_flag_names(flags);
+   if (debug || IS_TRACING()) {
+      char * s0 = feature_set_flag_names_t(flags);
       DBGMSG("Starting.  subset=%d, flags=%s,  dh=%s", subset, s0, dh_repr(dh) );
-      free(s0);
    }
 
-   DDCA_MCCS_Version_Spec vcp_version = get_vcp_version_by_display_handle(dh);
+   Public_Status_Code psc = 0;
+
+   // DDCA_MCCS_Version_Spec vcp_version = get_vcp_version_by_display_handle(dh);
    // DBGMSG("VCP version = %d.%d", vcp_version.major, vcp_version.minor);
-   VCP_Feature_Set feature_set = create_feature_set(
+
+   Dyn_Feature_Set* feature_set = dyn_create_feature_set2_dfm(
                                     subset,
-                                    vcp_version,
+                                    dh->dref,   // vcp_version,
                                     flags);
-                               //   flags & FSF_NOTABLE);
+
 #ifdef FUTURE
    Parsed_Capabilities * pcaps = NULL;   // TODO: HOW TO GET Parsed_Capabilities?, will only be set for probe/interrogate
    // special case, if scanning, don't try to do a table read of manufacturer specific
@@ -710,13 +866,13 @@ show_vcp_values(
       filter_feature_set(feature_set, hack42);
    }
 #endif
-   if (debug)
-      report_feature_set(feature_set, 0);
-
-   psc = show_feature_set_values(
+   if (debug || IS_TRACING()) {
+      DBGMSG("feature_set:");
+      dbgrpt_dyn_feature_set(feature_set, true, 0);
+   }
+   psc = show_feature_set_values2_dfm(
             dh, feature_set, collector, flags, features_seen);
-   free_vcp_feature_set(feature_set);
-   DBGMSF0(debug, "Done");
+   dyn_free_feature_set(feature_set);
+   DBGTRC(debug, TRACE_GROUP, "Done. Returning %s", psc_desc(psc));
    return psc;
 }
-
