@@ -1,34 +1,12 @@
-/* ddc_packet_io.c
+/** \file ddc_packet_io.c
  *
- *  Functions for performing DDC packet IO, using either the I2C bus API
- *  or the ADL API, as appropriate.
- *
- * <copyright>
- * Copyright (C) 2014-2018 Sanford Rockowitz <rockowitz@minsoft.com>
- *
- * Licensed under the GNU General Public License Version 2
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- * </endcopyright>
- */
-
-/** \file
  *  Functions for performing DDC packet IO, using either the I2C bus API
  *  or the ADL API, as appropriate.  Handles I2C bus retry.
  */
 
+// Copyright (C) 2014-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// SPDX-License-Identifier: GPL-2.0-or-later
+
 // N. ddc_open_display() and ddc_close_display() handle case USB, but the
 // packet functions are for I2C and ADL only.  Consider splitting.
 
@@ -42,7 +20,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <sys/ioctl.h>
+// #include <sys/ioctl.h>
 #include <unistd.h>
 
 #include "util/debug_util.h"
@@ -72,7 +50,7 @@
 
 
 // Trace class for this file
-static Trace_Group TRACE_GROUP = TRC_DDC;
+static DDCA_Trace_Group TRACE_GROUP = DDCA_TRC_DDCIO;
 
 #ifdef DEPRECATED
 // Deprecated - use all_bytes_zero() in string_util.c
@@ -110,79 +88,86 @@ bool is_ddc_null_message(Byte * packet) {
  *
  *  \param  dref            display reference
  *  \param  callopts        call option flags
- *  \param  pdh             address at which to return display handle
+ *  \param  dh_loc          address at which to return display handle
  *  \return status code     as from #i2c_open_bus(), #usb_open_hiddev_device()
  *  \retval DDCRC_LOCKED    display open in another thread
+ *  \retval DDCRC_ALREADY_OPEN display already open in current thread
  *
  *  **Call_Option** flags recognized:
  *  - CALLOPT_WAIT
  *  - CALLOPT_ERR_MSG
  */
-Public_Status_Code ddc_open_display(
+DDCA_Status
+ddc_open_display(
       Display_Ref *    dref,
       Call_Options     callopts,
-      Display_Handle** pdh)
+      Display_Handle** dh_loc)
 {
    bool debug = false;
    DBGMSF(debug, "Opening display %s, callopts=%s",
                  dref_repr_t(dref), interpret_call_options_t(callopts) );
+   assert(dh_loc);
 
    Display_Handle * dh = NULL;
-   Public_Status_Code psc = 0;
+   DDCA_Status ddcrc = 0;
 
-   Distinct_Display_Ref display_id = get_distinct_display_ref(dref);
+   Distinct_Display_Ref ddisp_ref = get_distinct_display_ref(dref);
    Distinct_Display_Flags ddisp_flags = DDISP_NONE;
    if (callopts & CALLOPT_WAIT)
       ddisp_flags |= DDISP_WAIT;
 
-   bool locked = lock_distinct_display(display_id, ddisp_flags);
-   if (!locked) {
-      psc = DDCRC_LOCKED;          // is there an appropriate errno value?  EBUSY? EACCES?
+   DDCA_Status lockrc = lock_distinct_display(ddisp_ref, ddisp_flags);
+   if (lockrc == DDCRC_LOCKED) {     // locked in another thread
+      ddcrc = DDCRC_LOCKED;          // is there an appropriate errno value?  EBUSY? EACCES?
       goto bye;
    }
 
+   if (dref->flags & DREF_OPEN) {
+      assert(ddcrc == DDCRC_ALREADY_OPEN);   // from lock_distinct_display()
+      ddcrc = DDCRC_ALREADY_OPEN;
+      goto bye;
+   }
+
+   assert(ddcrc == 0);
    switch (dref->io_path.io_mode) {
 
    case DDCA_IO_I2C:
       {
          int fd = i2c_open_bus(dref->io_path.path.i2c_busno, callopts);
          if (fd < 0) {
-            psc = fd;
-            goto bye;
+            ddcrc = fd;
          }
-
-         DBGMSF(debug, "Calling set_addr(0x37) for %s", dref_repr_t(dref));
-         Status_Errno base_rc =  i2c_set_addr(fd, 0x37, callopts);
-         if (base_rc != 0) {
-            assert(base_rc < 0);
-            close(fd);
-            psc = base_rc;
-            goto bye;
-         }
-
-         // Is this needed?
-         // 10/24/15, try disabling:
-         // sleepMillisWithTrace(DDC_TIMEOUT_MILLIS_DEFAULT, __func__, NULL);
-
-         dh = create_bus_display_handle_from_display_ref(fd, dref);    // n. sets dh->dref = dref
-
-         I2C_Bus_Info * bus_info = dref->detail;
-         assert(bus_info);   // need to convert to a test?
-         assert( memcmp(bus_info, I2C_BUS_INFO_MARKER, 4) == 0);
-
-         dref->pedid = bus_info->edid;
-         if (!dref->pedid) {
-            // How is this even possible?
-            // 1/2017:  Observed with x260 laptop and Ultradock, See ddcutil user report.
-            //          close(fd) fails
-            DBGMSG("No EDID for device on bus /dev/i2c-%d", dref->io_path.path.i2c_busno);
-            // if (!(callopts & CALLOPT_FORCE)) {
+         else {
+            DBGMSF(debug, "Calling set_addr(0x37) for %s", dref_repr_t(dref));
+            ddcrc =  i2c_set_addr(fd, 0x37, callopts);
+            if (ddcrc != 0) {
+               assert(ddcrc < 0);
                close(fd);
-               psc = DDCRC_EDID;
-               goto bye;
-            // }
-            // else
-            //    DBGMSG0("Continuing");
+            }
+
+            else {
+               // Is this needed?
+               // 10/24/15, try disabling:
+               // sleepMillisWithTrace(DDC_TIMEOUT_MILLIS_DEFAULT, __func__, NULL);
+
+               dh = create_bus_display_handle_from_display_ref(fd, dref);    // n. sets dh->dref = dref
+
+               I2C_Bus_Info * bus_info = dref->detail;
+               assert(bus_info);   // need to convert to a test?
+               assert( memcmp(bus_info, I2C_BUS_INFO_MARKER, 4) == 0);
+
+               dref->pedid = bus_info->edid;
+               if (!dref->pedid) {
+                  // How is this even possible?
+                  // 1/2017:  Observed with x260 laptop and Ultradock, See ddcutil user report.
+                  //          close(fd) fails
+                  DBGMSG("No EDID for device on bus /dev/i2c-%d", dref->io_path.path.i2c_busno);
+                  close(fd);
+                  ddcrc = DDCRC_EDID;
+                  free_display_handle(dh);
+                  dh = NULL;
+               }
+            }
          }
       }
       break;
@@ -203,11 +188,12 @@ Public_Status_Code ddc_open_display(
          // }
          int fd = usb_open_hiddev_device(dref->usb_hiddev_name, callopts);
          if (fd < 0) {
-            psc = fd;
-            goto bye;
+            ddcrc = fd;
+         }
+         else {
+            dh = create_usb_display_handle_from_display_ref(fd, dref);
+            dref->pedid = usb_get_parsed_edid_by_display_handle(dh);
          }
-         dh = create_usb_display_handle_from_display_ref(fd, dref);
-         dref->pedid = usb_get_parsed_edid_by_display_handle(dh);
       }
 #else
       PROGRAM_LOGIC_ERROR("ddcutil not built with USB support");
@@ -215,47 +201,53 @@ Public_Status_Code ddc_open_display(
       break;
    } // switch
    assert(!dh || dh->dref->pedid);
-   // needed?  for both or just I2C?
-   // sleep_millis_with_trace(DDC_TIMEOUT_MILLIS_DEFAULT, __func__, NULL);
-   if (dref->io_path.io_mode != DDCA_IO_USB)
-      call_tuned_sleep_i2c(SE_POST_OPEN);
-   // dbgrpt_display_handle(dh, __func__, 1);
+
+   if (ddcrc == 0) {
+      if (dref->io_path.io_mode != DDCA_IO_USB)
+         call_tuned_sleep_i2c(SE_POST_OPEN);
+      dref->flags |= DREF_OPEN;
+   }
+   else {
+      unlock_distinct_display(ddisp_ref);
+   }
 
 bye:
-   if (psc != 0) {
-      if (locked)
-         unlock_distinct_display(display_id);
-      COUNT_STATUS_CODE(psc);
+   if (ddcrc != 0) {
+      COUNT_STATUS_CODE(ddcrc);
    }
-   *pdh = dh;
-   assert(psc <= 0);
+   *dh_loc = dh;
+   assert(ddcrc <= 0);
+   assert( (ddcrc == 0 && *dh_loc) || (ddcrc < 0 && !*dh_loc) );
    // dbgrpt_distinct_display_descriptors(0);
-   DBGMSF(debug, "Done.  Returning: %s, *pdh=%d", psc_desc(psc), *pdh);
-   return psc;
+   DBGMSF(debug, "Done.  Returning: %s, *dh_loc=%s", psc_desc(ddcrc), dh_repr_t(*dh_loc));
+   return ddcrc;
 }
 
 
-/* Closes a DDC display.
+/** Closes a DDC display.
  *
- * Arguments:
- *    dh            display handle
+ *  \param  dh            display handle
+ *  \return 0 if success, or -errno if error
  *
- * Logs status code but continues execution if error.
+ *  \remark
+ *  Logs underlying status code if error.
  */
-void ddc_close_display(Display_Handle * dh) {
+Status_Errno
+ddc_close_display(Display_Handle * dh) {
    bool debug = false;
    if (debug) {
       DBGMSG0("Starting.");
       dbgrpt_display_handle(dh, __func__, 1);
    }
+   Status_Errno rc = 0;
 
    switch(dh->dref->io_path.io_mode) {
    case DDCA_IO_I2C:
       {
-         Status_Errno rc = i2c_close_bus(dh->fh, dh->dref->io_path.path.i2c_busno,  CALLOPT_NONE);    // return error if failure
+         rc = i2c_close_bus(dh->fh, dh->dref->io_path.path.i2c_busno,  CALLOPT_NONE);    // return error if failure
          if (rc != 0) {
             assert(rc < 0);
-            DBGMSG("close_i2c_bus returned %d", rc);
+            DBGMSG("i2c_close_bus returned %d", rc);
             COUNT_STATUS_CODE(rc);
          }
          dh->fh = -1;    // indicate invalid, in case we try to continue using dh
@@ -267,7 +259,7 @@ void ddc_close_display(Display_Handle * dh) {
    case DDCA_IO_USB:
 #ifdef USE_USB
       {
-         Status_Errno rc = usb_close_device(dh->fh, dh->dref->usb_hiddev_name, CALLOPT_NONE); // return error if failure
+         rc = usb_close_device(dh->fh, dh->dref->usb_hiddev_name, CALLOPT_NONE); // return error if failure
          if (rc != 0) {
             assert(rc < 0);
             DBGMSG("usb_close_device returned %d", rc);
@@ -281,10 +273,12 @@ void ddc_close_display(Display_Handle * dh) {
 #endif
    } //switch
 
+   dh->dref->flags &= (~DREF_OPEN);
    Distinct_Display_Ref display_id = get_distinct_display_ref(dh->dref);
    unlock_distinct_display(display_id);
 
    free_display_handle(dh);
+   return rc;
 }
 
 
@@ -361,7 +355,7 @@ int ddc_get_max_write_read_exchange_tries() {
 
 
 typedef
-Public_Status_Code (*Write_Read_Raw_Function)(
+DDCA_Status (*Write_Read_Raw_Function)(
          Display_Handle * dh,
          DDC_Packet *     request_packet_ptr,
          int              max_read_bytes,
@@ -390,7 +384,7 @@ Public_Status_Code (*Write_Read_Raw_Function)(
  *   -errno if error in write
  *   DDCRC_READ_ALL_ZERO
  */
-static Public_Status_Code ddc_i2c_write_read_raw(
+static DDCA_Status ddc_i2c_write_read_raw(
          Display_Handle * dh,
          DDC_Packet *     request_packet_ptr,
          int              max_read_bytes,
@@ -466,7 +460,7 @@ static Public_Status_Code ddc_i2c_write_read_raw(
  *   additional information.  Never seen.  How to handle?
  */
 
-static Public_Status_Code ddc_adl_write_read_raw(
+static DDCA_Status ddc_adl_write_read_raw(
       Display_Handle * dh,
       DDC_Packet *     request_packet_ptr,
       int              max_read_bytes,
@@ -481,7 +475,7 @@ static Public_Status_Code ddc_adl_write_read_raw(
    assert(dh && dh->dref && dh->dref->io_path.io_mode == DDCA_IO_ADL);
    // ASSERT_DISPLAY_IO_MODE(dh, DDCA_IO_ADL);
 
-   Public_Status_Code psc = adlshim_ddc_write_only(
+   DDCA_Status psc = adlshim_ddc_write_only(
                                dh,
                                get_packet_start(request_packet_ptr),   // n. no adjustment, unlike i2c version
                                get_packet_len(request_packet_ptr)
@@ -502,7 +496,7 @@ static Public_Status_Code ddc_adl_write_read_raw(
       else {
          if ( all_bytes_zero(readbuf+1, max_read_bytes-1)) {
                  psc = DDCRC_READ_ALL_ZERO;
-                 DDCMSG(debug, "All zero response.", NULL);
+                 DDCMSG(debug, "All zero response.");
                  COUNT_STATUS_CODE(psc);
          }
          else if (memcmp(get_packet_start(request_packet_ptr), readbuf, get_packet_len(request_packet_ptr)) == 0) {
@@ -523,7 +517,7 @@ static Public_Status_Code ddc_adl_write_read_raw(
 }
 
 
-static Public_Status_Code ddc_write_read_raw(
+static DDCA_Status ddc_write_read_raw(
       Display_Handle * dh,
       DDC_Packet *     request_packet_ptr,
       int              max_read_bytes,
@@ -538,7 +532,7 @@ static Public_Status_Code ddc_write_read_raw(
       DBGMSG0("request_packer_ptr->raw_bytes:");
       dbgrpt_buffer(request_packet_ptr->raw_bytes, 1);
    }
-   Public_Status_Code psc;
+   DDCA_Status psc;
 
    // This function should not be called for USB
    assert(dh->dref->io_path.io_mode == DDCA_IO_I2C || dh->dref->io_path.io_mode == DDCA_IO_ADL);
@@ -567,8 +561,6 @@ static Public_Status_Code ddc_write_read_raw(
       DBGTRC(debug, TRACE_GROUP,
              "      readbuf: %s",
              hexstring3_t(readbuf, *p_rcvd_bytes_ct, " ", 4, false));
-
-            // hexstring_t(readbuf, *pbytes_received));
    }
    return psc;
 }
@@ -584,11 +576,8 @@ static Public_Status_Code ddc_write_read_raw(
  *  \param expected_subtype    expected subtype to check for
  *  \param response_packet_ptr_loc  where to write address of response packet received
  *
- *  \return >= 0 if success (positive values possible for ADL?)\n
- *          modulated ADL status code if ADL error or special success case\n
- *          -errno for Linux errors\n
- *          as from #create_ddc_typed_response_packet()
- * \remark
+ *  \return pointer to #Error_Info struct if failure, NULL if success
+ *  \remark
  *  Issue: positive ADL codes, need to handle?
  */
 Error_Info *
@@ -606,7 +595,7 @@ ddc_write_read(
 
    Byte * readbuf = calloc(1, max_read_bytes);
    int    bytes_received = max_read_bytes;
-   Public_Status_Code    psc;
+   DDCA_Status    psc;
    *response_packet_ptr_loc = NULL;
 
    psc =  ddc_write_read_raw(
@@ -628,7 +617,7 @@ ddc_write_read(
               response_packet_ptr_loc);
        DBGTRC(debug, TRACE_GROUP,
               "create_ddc_typed_response_packet() returned %s, *response_packet_ptr_loc=%p",
-              ddcrc_desc(psc), *response_packet_ptr_loc );
+              ddcrc_desc_t(psc), *response_packet_ptr_loc );
 
        if (psc != 0 && *response_packet_ptr_loc) {  // paranoid,  should never occur
           free(*response_packet_ptr_loc);
@@ -668,7 +657,7 @@ ddc_write_read(
  *  \param all_zero_response_ok treat a response of all 0s as valid
  *  \param response_packet_ptr_loc  where to write address of response packet received
  *
- *  \return pointer to #Ddc_Error if failure, NULL if success
+ *  \return pointer to #Error_Info struct if failure, NULL if success
  *
  *  \remark
  *  status code from #ddc_write_read() may be positive for positive ADL status code ??
@@ -676,7 +665,7 @@ ddc_write_read(
  *            DDCRC_RETRIES, DDCRC_ALL_TRIES_ZERO, DDCRC_ALL_RESPONES_NULL if maximum tries exceeded
  *
  *\remark
- *   Issue: positive ADL codes, need to handle?
+ * Issue: positive ADL codes, need to handle?
  * \remark
  * The maximum number of tries is set in global variable max_write_read_exchange_tries.
  */
@@ -697,11 +686,9 @@ ddc_write_read_with_retry(
    assert(dh->dref->io_path.io_mode != DDCA_IO_USB);
    // show_backtrace(1);
 
-   // will be false on initial call to verify DDC communication
-   // bool null_response_checked = dh->dref->flags & DREF_DDC_NULL_RESPONSE_CHECKED;   // unused
    bool retry_null_response = !(dh->dref->flags & DREF_DDC_USES_NULL_RESPONSE_FOR_UNSUPPORTED);
 
-   Public_Status_Code  psc;
+   DDCA_Status  psc;
    int  tryctr;
    bool retryable;
    int  ddcrc_read_all_zero_ct = 0;
@@ -791,7 +778,7 @@ ddc_write_read_with_retry(
             ddcrc_read_all_zero_ct++;
       }    // rc < 0
    }
-   DBGTRC(debug, TRC_NEVER, "After try loop. tryctr=%d, psc=%d, retryable=%s",
+   DBGTRC(debug, DDCA_TRC_NONE, "After try loop. tryctr=%d, psc=%d, retryable=%s",
          tryctr, psc, bool_repr(retryable));
    if (debug) {
       for (int ndx = 0; ndx < tryctr; ndx++) {
@@ -799,7 +786,6 @@ ddc_write_read_with_retry(
       }
    }
 
-   // Using new Ddc_Error mechanism
    Error_Info * ddc_excp = NULL;
 
    if (psc < 0) {
@@ -842,7 +828,7 @@ ddc_write_read_with_retry(
  *   0 if success
  *   -errno if error
  */
-Public_Status_Code
+static Status_Errno_DDC
 ddc_i2c_write_only(
          int           fh,
          DDC_Packet *  request_packet_ptr
@@ -864,7 +850,7 @@ ddc_i2c_write_only(
             ? SE_POST_SAVE_SETTINGS
             : SE_POST_WRITE;
    call_tuned_sleep_i2c(sleep_type);
-   DBGTRC(debug, TRACE_GROUP, "Done. rc=%s\n", psc_desc(rc) );
+   DBGTRC(debug, TRACE_GROUP, "Done. rc=%s", psc_desc(rc) );
    return rc;
 }
 
@@ -873,7 +859,7 @@ ddc_i2c_write_only(
  *
  * \param  dh                  Display_Handle for open I2C or ADL device
  * \param  request_packet_ptr  DDC packet to write
- * \return NULL if success, #Ddc_Error if error
+ * \return NULL if success, #Error_Info struct if error
  */
 Error_Info *
 ddc_write_only(
@@ -883,7 +869,7 @@ ddc_write_only(
    bool debug = false;
    DBGTRC0(debug, TRACE_GROUP, "Starting.");
 
-   Public_Status_Code psc = 0;
+   DDCA_Status psc = 0;
    assert(dh->dref->io_path.io_mode != DDCA_IO_USB);
    if (dh->dref->io_path.io_mode == DDCA_IO_I2C) {
       psc = ddc_i2c_write_only(dh->fh, request_packet_ptr);
@@ -915,7 +901,7 @@ ddc_write_only(
  *
  *  \param  dh                  display handle (for either I2C or ADL device)
  *  \param  request_packet_ptr  DDC packet to write
- *  \return pointer to #Ddc_Error if failure, NULL if success
+ *  \return pointer to #Error_Info struct if failure, NULL if success
  *
  *  The maximum number of tries allowed has been set in global variable
  *  max_write_only_exchange_tries.
@@ -923,18 +909,17 @@ ddc_write_only(
 Error_Info *
 ddc_write_only_with_retry(
       Display_Handle * dh,
-      DDC_Packet *     request_packet_ptr
-     )
+      DDC_Packet *     request_packet_ptr)
 {
    bool debug = false;
    DBGTRC0(debug, TRACE_GROUP, "Starting." );
 
    assert(dh->dref->io_path.io_mode != DDCA_IO_USB);
 
-   Public_Status_Code psc;
+   DDCA_Status        psc;
    int                tryctr;
    bool               retryable;
-   Error_Info *        try_errors[MAX_MAX_TRIES];
+   Error_Info *       try_errors[MAX_MAX_TRIES];
 
    assert(max_write_only_exchange_tries > 0);
    for (tryctr=0, psc=-999, retryable=true;
@@ -991,8 +976,11 @@ ddc_write_only_with_retry(
          COUNT_STATUS_CODE(psc);     // new status code, count it
    }
    else {
-      for (int ndx = 0; ndx < tryctr; ndx++) {
-         // errinfo_free(try_errors[ndx]);
+      // 2 possibilities:
+      //   succeeded after retries, there will be some errors (tryctr > 1)
+      //   no errors (tryctr == 1)
+      // int last_bad_try_index = tryctr-2;
+      for (int ndx = 0; ndx < tryctr-1; ndx++) {
          ERRINFO_FREE_WITH_REPORT(try_errors[ndx], debug || IS_TRACING() || report_freed_exceptions);
       }
    }
