@@ -1,29 +1,12 @@
-/* ddc_packet_io.h
+/** \file ddc_packet_io.h
  *
  *  Functions for performing DDC packet IO, using either the I2C bus API
- *  or the ADL API, as appropriate.
- *
- * <copyright>
- * Copyright (C) 2014-2016 Sanford Rockowitz <rockowitz@minsoft.com>
- *
- * Licensed under the GNU General Public License Version 2
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- * </endcopyright>
+ *  or the ADL API, as appropriate.  Handles I2C bus retry.
  */
 
+// Copyright (C) 2014-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// SPDX-License-Identifier: GPL-2.0-or-later
+
 #ifndef DDC_PACKET_IO_H_
 #define DDC_PACKET_IO_H_
 
@@ -38,11 +21,11 @@
 
 // bool all_zero(Byte * bytes, int bytec);
 
-Public_Status_Code ddc_open_display(
+DDCA_Status ddc_open_display(
       Display_Ref *    dref,
       Call_Options     callopts,
-      Display_Handle** pdh);
-void ddc_close_display(Display_Handle * dh);
+      Display_Handle** dh_loc);
+Status_Errno ddc_close_display(Display_Handle * dh);
 
 
 // Retry management
