@@ -1,29 +1,10 @@
-/* ddc_services.c
+/** @file ddc_services.c
  *
- * <copyright>
- * Copyright (C) 2014-2018 Sanford Rockowitz <rockowitz@minsoft.com>
- *
- * Licensed under the GNU General Public License Version 2
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- * </endcopyright>
+ * ddc layer initialization and configuration, statistics management
  */
 
-/** \file
- *  DDC initialization and configuration, statistics management
- */
+// Copyright (C) 2014-2018 Sanford Rockowitz <rockowitz@minsoft.com>
+// SPDX-License-Identifier: GPL-2.0-or-later
 
 /** \cond */
 #include <stdio.h>
@@ -35,9 +16,12 @@
 #include "base/base_init.h"
 #include "base/parms.h"
 #include "base/sleep.h"
+#include "base/feature_metadata.h"
 
 #include "vcp/vcp_feature_codes.h"
 
+#include "dynvcp/dyn_feature_codes.h"
+
 #include "i2c/i2c_do_io.h"
 
 #include "adl/adl_shim.h"
@@ -150,5 +134,7 @@ void init_ddc_services() {
    // ddc:
    ddc_reset_ddc_stats();
    init_vcp_feature_codes();
+   init_dyn_feature_codes();    // must come after init_vcp_feature_codes()
+   // dbgrpt_func_name_table(1);
    init_ddc_display_lock();
 }
