@@ -1,31 +1,9 @@
-/* ddc_vcp.c
- *
- * Virtual Control Panel access
- *
- * <copyright>
- * Copyright (C) 2014-2018 Sanford Rockowitz <rockowitz@minsoft.com>
- *
- * Licensed under the GNU General Public License Version 2
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- * </endcopyright>
+/** \file ddc_vcp.c
+ *  Virtual Control Panel access
  */
 
-/** \file
- * Virtual Control Panel access
- */
+// Copyright (C) 2014-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// SPDX-License-Identifier: GPL-2.0-or-later
 
 #include <config.h>
 
@@ -59,6 +37,8 @@
 
 #include "vcp/vcp_feature_codes.h"
 
+#include <dynvcp/dyn_feature_codes.h>
+
 #include "ddc/ddc_multi_part_io.h"
 #include "ddc/ddc_packet_io.h"
 #include "ddc/ddc_vcp_version.h"
@@ -67,7 +47,7 @@
 
 
 // Trace class for this file
-static Trace_Group TRACE_GROUP = TRC_DDC;
+static DDCA_Trace_Group TRACE_GROUP = DDCA_TRC_DDC;
 
 
 
@@ -95,7 +75,6 @@ static Thread_Vcp_Settings *  get_thread_vcp_settings() {
 }
 
 
-
 //
 //  Save Control Settings
 //
@@ -103,7 +82,7 @@ static Thread_Vcp_Settings *  get_thread_vcp_settings() {
 /** Executes the DDC Save Control Settings command.
  *
  * \param  dh handle of open display device
- * \return NULL if success, pointer to #Ddc_Error if failure
+ * \return NULL if success, pointer to #Error_Info if failure
  */
 Error_Info *
 ddc_save_current_settings(
@@ -117,7 +96,7 @@ ddc_save_current_settings(
 
    if (dh->dref->io_path.io_mode == DDCA_IO_USB) {
       // command line parser should block this case
-      PROGRAM_LOGIC_ERROR("MCCS over USB does not have Save Current Settings command", NULL);
+      PROGRAM_LOGIC_ERROR("MCCS over USB does not have Save Current Settings command");
       ddc_excp = errinfo_new(DDCRC_UNIMPLEMENTED, __func__);
    }
    else {
@@ -150,7 +129,7 @@ ddc_save_current_settings(
  *  \param  feature_code  VCP feature code
  *  \param  new_value     new value
  *  \return NULL if success,
- *          pointer to #Ddc_Error from #ddc_write_only_with_retry() if failure
+ *          pointer to #Error_Info from #ddc_write_only_with_retry() if failure
  */
 Error_Info *
 ddc_set_nontable_vcp_value(
@@ -200,7 +179,7 @@ ddc_set_nontable_vcp_value(
  *  \param  bytect        number of bytes
  *  \return NULL  if success
  *          DDCL_UNIMPLEMENTED if io mode is USB
- *          #Ddc_Error from #multi_part_write_with_retry() otherwise
+ *          #Error_Info from #multi_part_write_with_retry() otherwise
  */
 Error_Info *
 set_table_vcp_value(
@@ -276,7 +255,6 @@ is_rereadable_feature(
 {
    bool debug = false;
    DBGMSF(debug, "Starting opcode = 0x%02x", opcode);
-   bool result = false;
 
    // readable features that should not be read after write
    DDCA_Vcp_Feature_Code unrereadable_features[] = {
@@ -285,31 +263,25 @@ is_rereadable_feature(
          0x60,        // input source ???
    };
 
-   VCP_Feature_Table_Entry * vfte = vcp_find_feature_by_hexid(opcode);
-   DBGMSF(debug, "vfte=%p", vfte);
-   if (vfte) {
-      assert(opcode < 0xe0);
-      DDCA_MCCS_Version_Spec vspec = get_vcp_version_by_display_handle(dh);  // ensure dh->vcp_version set
-      DBGMSF(debug, "vspec = %d.%d", vspec.major, vspec.minor);
-      // hack, make a guess
-      if ( vcp_version_eq(vspec, DDCA_VSPEC_UNKNOWN)   ||
-           vcp_version_eq(vspec, DDCA_VSPEC_UNQUERIED ))
-         vspec = DDCA_VSPEC_V22;
-
-      // if ( !vcp_version_eq(vspec, VCP_SPEC_UNKNOWN) &&
-      //      !vcp_version_eq(vspec, VCP_SPEC_UNQUERIED ))
-      // {
-         result = is_feature_readable_by_vcp_version(vfte, vspec);
-         DBGMSF(debug, "vspec=%d.%d, readable feature = %s", vspec.major, vspec.minor, bool_repr(result));
-      // }
+   bool result = true;
+   for (int ndx = 0; ndx < ARRAY_SIZE(unrereadable_features); ndx++) {
+      if ( unrereadable_features[ndx] == opcode ) {
+          result = false;
+          DBGMSF(debug, "Unreadable opcode 0x%02x", opcode);
+          break;
+      }
    }
+
    if (result) {
-      for (int ndx = 0; ndx < ARRAY_SIZE(unrereadable_features); ndx++) {
-         if ( unrereadable_features[ndx] == opcode ) {
-            result = false;
-            DBGMSF(debug, "Unreadable opcode 0x%02x", opcode);
-            break;
-         }
+      Display_Feature_Metadata * dfm = dyn_get_feature_metadata_by_dh_dfm(
+            opcode,
+            dh,
+            false    //                  with_default
+            );
+      // if not found, assume readable  ??
+      if (dfm) {
+         result = dfm->feature_flags & DDCA_READABLE;
+         dfm_free(dfm);
       }
    }
 
@@ -320,8 +292,8 @@ is_rereadable_feature(
 
 static bool
 single_vcp_value_equal(
-      Single_Vcp_Value * vrec1,
-      Single_Vcp_Value * vrec2)
+      DDCA_Any_Vcp_Value * vrec1,
+      DDCA_Any_Vcp_Value * vrec2)
 {
    assert(vrec1 && vrec2);  // no implementation for degenerate cases
    bool debug = false;
@@ -333,7 +305,7 @@ single_vcp_value_equal(
       switch(vrec1->value_type) {
       case(DDCA_NON_TABLE_VCP_VALUE):
             // only check SL byte which would be set for any VCP, monitor
-            result = (vrec1->val.nc.sl == vrec2->val.nc.sl);
+      result = (vrec1->val.c_nc.sl == vrec2->val.c_nc.sl);
             break;
       case(DDCA_TABLE_VCP_VALUE):
             result = (vrec1->val.t.bytect == vrec2->val.t.bytect) &&
@@ -354,7 +326,7 @@ single_vcp_value_equal(
  *  \param  dh            display handle for open display
  *  \param  vrec          pointer to value record
  *  \param  newval_loc    if non-null, address at which to return verified value
- *  \return NULL if success, pointer to #Ddc_Error if failure
+ *  \return NULL if success, pointer to #Error_Info if failure
  *
  *  If write verification is turned on, reads the feature value after writing it
  *  to ensure the display has actually changed the value.
@@ -367,8 +339,8 @@ single_vcp_value_equal(
 Error_Info *
 ddc_set_vcp_value(
       Display_Handle *    dh,
-      Single_Vcp_Value *  vrec,
-      Single_Vcp_Value ** newval_loc)
+      DDCA_Any_Vcp_Value *  vrec,
+      DDCA_Any_Vcp_Value ** newval_loc)
 {
    bool debug = false;
    DBGMSF0(debug, "Starting. ");
@@ -381,7 +353,7 @@ ddc_set_vcp_value(
    if (newval_loc)
       *newval_loc = NULL;
    if (vrec->value_type == DDCA_NON_TABLE_VCP_VALUE) {
-      ddc_excp = ddc_set_nontable_vcp_value(dh, vrec->opcode, vrec->val.c.cur_val);
+      ddc_excp = ddc_set_nontable_vcp_value(dh, vrec->opcode, VALREC_CUR_VAL(vrec));
       psc = (ddc_excp) ? ddc_excp->status_code : 0;
    }
    else {
@@ -393,7 +365,7 @@ ddc_set_vcp_value(
    if (!ddc_excp && ddc_get_verify_setvcp()) {
       if (is_rereadable_feature(dh, vrec->opcode) ) {
          f0printf(verbose_msg_dest, "Verifying that value of feature 0x%02x successfully set...\n", vrec->opcode);
-         Single_Vcp_Value * newval = NULL;
+         DDCA_Any_Vcp_Value * newval = NULL;
          ddc_excp = ddc_get_vcp_value(
              dh,
              vrec->opcode,
@@ -436,6 +408,61 @@ ddc_set_vcp_value(
 }
 
 
+/** Possibly returns a mock value for a non-table feature
+ *
+ *  \param  feature_code  VCP Feature Code
+ *  \param  resp_loc      where to return pointer to pseudo-response,
+ *                        NULL if no pseudo-response
+ *  \return pseudo error information, if simulating a failure
+ *
+ *  \remark
+ *  If return NULL and *resp_loc == NULL, then no pseudo response was generated
+ */
+Error_Info *
+mock_get_nontable_vcp_value(
+      DDCA_Vcp_Feature_Code   feature_code,
+      Parsed_Nontable_Vcp_Response** resp_loc)
+{
+   bool debug = false;
+   DBGMSF(debug, "Starting. feature_code = 0x%02x, resp_loc = %p",
+                 feature_code, resp_loc);
+   Error_Info * pseudo_errinfo = NULL;
+   *resp_loc = NULL;
+
+#ifdef TEST_X72
+   if (feature_code == 0x72) {
+      Parsed_Nontable_Vcp_Response * resp = calloc(1, sizeof(Parsed_Nontable_Vcp_Response));
+      resp->vcp_code = feature_code;
+      resp->valid_response = true;
+      resp->supported_opcode = true;
+      resp->mh = 0;
+      resp->ml = 0;
+      resp->sh = 0x78;
+      resp->sl = 0x00;
+      resp->max_value = resp->mh << 8 | resp->ml;
+      resp->cur_value = resp->sh << 8 | resp->sl;
+
+      *resp_loc = resp;
+   }
+#endif
+
+#ifdef TEST_EIO
+   if (feature_code == 0x45) {
+      pseudo_errinfo = errinfo_new2(-EIO, __func__, "Pseudo EIO error");
+   }
+#endif
+
+   if (debug) {
+      DBGMSG("Feature 0x%02x, *resp_loc = %p, returning: %s",
+          feature_code,
+          *resp_loc,
+          errinfo_summary(pseudo_errinfo) );
+      if (*resp_loc)
+         dbgrpt_interpreted_nontable_vcp_response(*resp_loc, 2);
+   }
+   return pseudo_errinfo;
+}
+
 //
 // Get VCP values
 //
@@ -444,8 +471,8 @@ ddc_set_vcp_value(
  *
  *  \param  dh                 handle for open display
  *  \param  feature_code       VCP feature code
- *  \param  ppInterpretedCode  where to return parsed respons
- *  \return NULL if success, pointer to #Ddc_Error if failure
+ *  \param  ppInterpretedCode  where to return parsed response
+ *  \return NULL if success, pointer to #Error_Info if failure
  *
  * It is the responsibility of the caller to free the parsed response.
  *
@@ -463,11 +490,18 @@ ddc_get_nontable_vcp_value(
    Public_Status_Code psc = 0;
    Error_Info * excp = NULL;
    Parsed_Nontable_Vcp_Response * parsed_response = NULL;
+   *ppInterpretedCode = NULL;
+
+   Error_Info * mock_errinfo = mock_get_nontable_vcp_value(feature_code, ppInterpretedCode);
+   if (mock_errinfo || *ppInterpretedCode) {
+      DBGMSF(debug, "Returning mock response for feature 0x%02x", feature_code);
+      return mock_errinfo;
+   }
 
    DDC_Packet * request_packet_ptr  = NULL;
    DDC_Packet * response_packet_ptr = NULL;
    request_packet_ptr = create_ddc_getvcp_request_packet(
-                           feature_code, "get_vcp_by_DisplayRef:request packet");
+                           feature_code, "ddc_get_nontable_vcp_value:request packet");
    // dump_packet(request_packet_ptr);
 
    Byte expected_response_type = DDC_PACKET_TYPE_QUERY_VCP_RESPONSE;
@@ -536,11 +570,15 @@ ddc_get_nontable_vcp_value(
       if (excp) {
          DBGMSG("Error reading feature x%02x.  Returning exception: ", feature_code);
          errinfo_report(excp, 1);
+         DBGMSG("Done");
       }
       else {
          DBGMSG("Success reading feature x%02x. *ppinterpreted_code=%p", feature_code, parsed_response);
-         DBGMSG("  mh=0x%02x, ml=0x%02x, sh=0x%02x, sl=0x%02x",
-                parsed_response->mh, parsed_response->ml, parsed_response->sh, parsed_response->sl);
+         DBGMSG("  mh=0x%02x, ml=0x%02x, sh=0x%02x, sl=0x%02x, max value=%d, cur value=%d",
+                parsed_response->mh, parsed_response->ml,
+                parsed_response->sh, parsed_response->sl,
+                (parsed_response->mh<<8) | parsed_response->ml,
+                (parsed_response->sh<<8) | parsed_response->sl);
       }
    }
    *ppInterpretedCode = parsed_response;
@@ -555,7 +593,7 @@ ddc_get_nontable_vcp_value(
  *  \param  dh              display handle
  *  \param  feature_code    VCP feature code
  *  \param  pp_table_bytes  location at which to save address of newly allocated Buffer
- *  \return NULL if success, pointer to #Ddc_Error if failure
+ *  \return NULL if success, pointer to #Error_Info if failure
  */
 Error_Info * ddc_get_table_vcp_value(
        Display_Handle *       dh,
@@ -603,7 +641,7 @@ Error_Info * ddc_get_table_vcp_value(
  * \param  feature_code    feature code id
  * \param  call_type       indicates whether table or non-table
  * \param  pvalrec         location where to return newly allocated #Single_Vcp_Value
- * \return NULL if success, pointer to #Ddc_Error if failure
+ * \return NULL if success, pointer to #Error_Info if failure
  *
  * The value pointed to by pvalrec is non-null iff the returned status code is 0.
  *
@@ -614,7 +652,7 @@ ddc_get_vcp_value(
        Display_Handle *       dh,
        Byte                   feature_code,
        DDCA_Vcp_Value_Type    call_type,
-       Single_Vcp_Value **    valrec_loc)
+       DDCA_Any_Vcp_Value **  valrec_loc)
 {
    bool debug = false;
    DBGTRC(debug, TRACE_GROUP, "Starting. Reading feature 0x%02x, dh=%s, dh->fh=%d",
@@ -624,7 +662,7 @@ ddc_get_vcp_value(
    Error_Info * ddc_excp = NULL;
    Buffer * buffer = NULL;
    Parsed_Nontable_Vcp_Response * parsed_nontable_response = NULL;  // vs interpreted ..
-   Single_Vcp_Value * valrec = NULL;
+   DDCA_Any_Vcp_Value * valrec = NULL;
 
    // why are we coming here for USB?
    if (dh->dref->io_path.io_mode == DDCA_IO_USB) {
@@ -647,6 +685,8 @@ ddc_get_vcp_value(
                                parsed_nontable_response->sl);
                    free(parsed_nontable_response);
                 }
+                else
+                   ddc_excp = errinfo_new(psc, __func__);
                 break;
 
           case (DDCA_TABLE_VCP_VALUE):
@@ -696,105 +736,12 @@ ddc_get_vcp_value(
    *valrec_loc = valrec;
 
    DBGTRC(debug, TRACE_GROUP, "Done. psc=%s", psc_desc(psc) );
-   if (psc == 0 && debug)
-      report_single_vcp_value(valrec,1);
+   assert( (psc == 0 && valrec) || (psc != 0 && !valrec) );
+
+   if (psc == 0 && ( debug || IS_TRACING_GROUP(TRACE_GROUP)))
+      dbgrpt_single_vcp_value(valrec,1);
    assert( (psc == 0 && *valrec_loc) || (psc != 0 && !*valrec_loc) );
    DBGTRC(debug, TRACE_GROUP, "Done. Returning: %s", errinfo_summary(ddc_excp));
    return ddc_excp;
 }
 
-#ifdef ALT
-Error_Info *
-ddc_get_vcp_value2(
-       Display_Handle *       dh,
-       Byte                   feature_code,
-       DDCA_Vcp_Value_Type    call_type,
-       DDCA_Any_Vcp_Value **  valrec_loc)
-{
-   bool debug = false;
-   DBGTRC(debug, TRACE_GROUP, "Starting. Reading feature 0x%02x, dh=%s, dh->fh=%d",
-            feature_code, dh_repr_t(dh), dh->fh);
-
-   Public_Status_Code psc = 0;
-   Error_Info * ddc_excp = NULL;
-   Buffer * buffer = NULL;
-   Parsed_Nontable_Vcp_Response * parsed_nontable_response = NULL;  // vs interpreted ..
-   DDCA_Any_Vcp_Value * valrec = NULL;
-
-   // why are we coming here for USB?
-   if (dh->dref->io_path.io_mode == DDCA_IO_USB) {
-#ifdef USE_USB
-      DBGMSF0(debug, "USB case");
-
-      switch (call_type) {
-
-          case (DDCA_NON_TABLE_VCP_VALUE):
-                psc = usb_get_nontable_vcp_value(
-                      dh,
-                      feature_code,
-                      &parsed_nontable_response);    //
-                if (psc == 0) {
-                   valrec = create_nontable_vcp_value2(
-                               feature_code,
-                               parsed_nontable_response->mh,
-                               parsed_nontable_response->ml,
-                               parsed_nontable_response->sh,
-                               parsed_nontable_response->sl);
-                   free(parsed_nontable_response);
-                }
-                break;
-
-          case (DDCA_TABLE_VCP_VALUE):
-                psc = DDCRC_UNIMPLEMENTED;
-                ddc_excp = errinfo_new(DDCRC_UNIMPLEMENTED, __func__);
-                break;
-          }
-#else
-      PROGRAM_LOGIC_ERROR("ddcutil not built with USB support");
-#endif
-   }
-   else {
-      switch (call_type) {
-
-      case (DDCA_NON_TABLE_VCP_VALUE):
-            ddc_excp = ddc_get_nontable_vcp_value(
-                          dh,
-                          feature_code,
-                          &parsed_nontable_response);
-            psc = (ddc_excp) ? ddc_excp->status_code : 0;
-            if (!ddc_excp) {
-               valrec = create_nontable_vcp_value2(
-                           feature_code,
-                           parsed_nontable_response->mh,
-                           parsed_nontable_response->ml,
-                           parsed_nontable_response->sh,
-                           parsed_nontable_response->sl);
-               free(parsed_nontable_response);
-            }
-            break;
-
-      case (DDCA_TABLE_VCP_VALUE):
-            ddc_excp = ddc_get_table_vcp_value(
-                    dh,
-                    feature_code,
-                    &buffer);
-            psc = ERRINFO_STATUS(ddc_excp);
-            if (!ddc_excp) {
-               valrec = create_table_vcp_value_by_buffer2(feature_code, buffer);
-               buffer_free(buffer, __func__);
-            }
-            break;
-      }
-
-   } // non USB
-
-   *valrec_loc = valrec;
-
-   DBGTRC(debug, TRACE_GROUP, "Done. psc=%s", psc_desc(psc) );
-   if (psc == 0 && debug)
-      report_any_vcp_value(valrec,1);
-   assert( (psc == 0 && *valrec_loc) || (psc != 0 && !*valrec_loc) );
-   DBGTRC(debug, TRACE_GROUP, "Done. Returning: %s", errinfo_summary(ddc_excp));
-   return ddc_excp;
-}
-#endif
