@@ -1,25 +1,7 @@
-/* i2c_bus_core.c
- *
- * <copyright>
- * Copyright (C) 2014-2018 Sanford Rockowitz <rockowitz@minsoft.com>
- *
- * Licensed under the GNU General Public License Version 2
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- * </endcopyright>
- */
+// i2c_bus_core.c
+
+// Copyright (C) 2014-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// SPDX-License-Identifier: GPL-2.0-or-later
 
 /** \file
  * I2C bus detection and inspection
@@ -42,8 +24,11 @@
 #include "util/debug_util.h"
 #include "util/failsim.h"
 #include "util/file_util.h"
+#include "util/glib_string_util.h"
 #include "util/report_util.h"
+#include "util/edid.h"
 #include "util/string_util.h"
+#include "util/subprocess_util.h"
 #include "util/sysfs_util.h"
 #include "util/udev_i2c_util.h"
 #include "util/utilrpt.h"
@@ -62,10 +47,10 @@
 
 
 // Trace class for this file
-static Trace_Group TRACE_GROUP = TRC_I2C;
+static DDCA_Trace_Group TRACE_GROUP = DDCA_TRC_I2C;
 
-/** All I2C buses.  GPtrArray of pointers to #I2C_Bus_Info */
-static GPtrArray * i2c_buses = NULL;
+/** All I2C buses.  GPtrArray of pointers to #I2C_Bus_Info - shared with i2c_bus_selector.c */
+/* static */ GPtrArray * i2c_buses = NULL;
 
 /** Global variable.  Controls whether function #i2c_set_addr() attempts retry
  *  after EBUSY error by changing ioctl op I2C_SLAVE to I2C_SLAVE_FORCE.
@@ -202,7 +187,8 @@ Status_Errno i2c_set_addr(int file, int addr, Call_Options callopts) {
    bool force_i2c_slave_failure = false;
 #endif
    callopts |= CALLOPT_ERR_MSG;    // temporary
-   DBGMSF(debug, "file=%d, addr=0x%02x, i2c_force_slave_addr_flag=%s, callopts=%s",
+   DBGTRC(debug, TRACE_GROUP,
+                 "file=%d, addr=0x%02x, i2c_force_slave_addr_flag=%s, callopts=%s",
                  file, addr,
                  bool_repr(i2c_force_slave_addr_flag),
                  interpret_call_options_t(callopts) );
@@ -249,10 +235,9 @@ retry:
       result = -errsv;
    }
 
-   if (result || debug) {
-      printf("(%s) addr = 0x%02x. Returning %d\n", __func__, addr, result);
+   DBGTRC((result || debug), TRACE_GROUP,
+           "addr = 0x%02x. Returning %s", addr, psc_desc(result));
       // show_backtrace(1);
-   }
 
    assert(result <= 0);
    return result;
@@ -263,34 +248,6 @@ retry:
 // I2C Bus Inspection - Functionality Flags
 //
 
-// Functions and data structures for interpreting the I2C bus functionality flags.
-// They are overly complex for production use.  They were created during development
-// to facilitate exploratory programming.
-
-// Note 2 entries for I2C_FUNC_I2C.  Usage must take this into account.
-Value_Name_Title_Table functionality_table2 = {
-      //  flag                              I2C function name
-      VNT(I2C_FUNC_I2C                    , "ioctl_write"),
-      VNT(I2C_FUNC_I2C                    , "ioctl_read"),
-      VNT(I2C_FUNC_10BIT_ADDR             , NULL),
-      VNT(I2C_FUNC_PROTOCOL_MANGLING      , NULL),
-      VNT(I2C_FUNC_SMBUS_PEC              , "i2c_smbus_pec"),
-      VNT(I2C_FUNC_SMBUS_BLOCK_PROC_CALL  , "i2c_smbus_block_proc_call"),
-      VNT(I2C_FUNC_SMBUS_QUICK            , "i2c_smbus_quick"),
-      VNT(I2C_FUNC_SMBUS_READ_BYTE        , "i2c_smbus_read_byte"),
-      VNT(I2C_FUNC_SMBUS_WRITE_BYTE       , "i2c_smbus_write_byte"),
-      VNT(I2C_FUNC_SMBUS_READ_BYTE_DATA   , "i2c_smbus_read_byte_data"),
-      VNT(I2C_FUNC_SMBUS_WRITE_BYTE_DATA  , "i2c_smbus_write_byte_data"),
-      VNT(I2C_FUNC_SMBUS_READ_WORD_DATA   , "i2c_smbus_read_word_data"),
-      VNT(I2C_FUNC_SMBUS_WRITE_WORD_DATA  , "i2c_smbus_write_word_data"),
-      VNT(I2C_FUNC_SMBUS_PROC_CALL        , "i2c_smbus_proc_call"),
-      VNT(I2C_FUNC_SMBUS_READ_BLOCK_DATA  , "i2c_smbus_read_block_data"),
-      VNT(I2C_FUNC_SMBUS_WRITE_BLOCK_DATA , "i2c_smbus_write_block_data"),
-      VNT(I2C_FUNC_SMBUS_READ_I2C_BLOCK   , "i2c_smbus_read_i2c_block_data"),
-      VNT(I2C_FUNC_SMBUS_WRITE_I2C_BLOCK  , "i2c_smbus_write_i2c_block_data"),
-      VNT_END
-};
-
 
 // Separate table for interpreting functionality flags.
 
@@ -330,7 +287,7 @@ Value_Name_Table functionality_flag_table = {
  */
 unsigned long i2c_get_functionality_flags_by_fd(int fd) {
    bool debug = false;
-   DBGMSF(debug, "Starting.", NULL);
+   DBGMSF(debug, "Starting.");
 
    unsigned long funcs;
    int rc;
@@ -372,7 +329,7 @@ char * i2c_interpret_functionality_flags(unsigned long functionality) {
  */
 void i2c_report_functionality_flags(long functionality, int maxline, int depth) {
    bool debug = false;
-   DBGMSF(debug, "Starting.  functionality=0x%016x, maxline=%d", functionality, maxline);
+   DBGMSF(debug, "Starting.  functionality=0x%lx, maxline=%d", functionality, maxline);
 
    char * buf0 = i2c_interpret_functionality_flags(functionality);
    DBGMSF(debug, "buf0=|%s|", buf0);
@@ -397,7 +354,7 @@ void i2c_report_functionality_flags(long functionality, int maxline, int depth)
    free(buf0);
    ntsa_free(ntsa, /* free_strings */ true);
 
-   DBGMSF(debug, "Done", NULL);
+   DBGMSF(debug, "Done");
 }
 
 
@@ -405,6 +362,35 @@ void i2c_report_functionality_flags(long functionality, int maxline, int depth)
 // I2C Bus Inspection - Slave Addresses
 //
 
+// Attempting to recode using opendir() and readdir() produced
+// a complicated mess.  Using execute_shell_cmd_collect() is simple.
+// Simplicity has its virtues.
+static bool is_edp_device(int busno) {
+   bool debug = false;
+   // DBGMSF(debug, "Starting.  busno=%d", busno);
+   bool result = false;
+
+   char cmd[100];
+   snprintf(cmd, 100, "ls -d /sys/class/drm/card*/card*/i2c-%d", busno);
+   // DBGMSG("cmd: %s", cmd);
+
+   GPtrArray * lines = execute_shell_cmd_collect(cmd);
+   if (lines)  {    // command should never fail, but just in case
+      for (int ndx = 0; ndx < lines->len; ndx++) {
+         char * s = g_ptr_array_index(lines, ndx);
+         if (strstr(s, "-eDP-")) {
+            result = true;
+            break;
+         }
+      }
+      g_ptr_array_free(lines, true);
+   }
+
+   DBGTRC(debug, TRACE_GROUP, "busno=%d, returning: %s", busno, sbool(result));
+   return result;
+}
+
+
 #ifdef UNUSED
 /* Checks each address on an I2C bus to see if a device exists.
  * The bus device has already been opened.
@@ -475,6 +461,7 @@ bool * i2c_detect_all_slave_addrs(int busno) {
 }
 #endif
 
+#ifdef DETECT_SLAVE_ADDRS
 
 /** Checks DDC related addresses on an I2C bus to see if the addresses are active.
  *  The bus device has already been opened.
@@ -492,15 +479,17 @@ bool * i2c_detect_all_slave_addrs(int busno) {
 // static
 Status_Errno_DDC i2c_detect_ddc_addrs_by_fd(int fd, Byte * presult) {
    bool debug = false;
-   DBGMSF(debug, "Starting. fd=%d", fd);
+   DBGTRC(debug, TRACE_GROUP, "Starting. fd=%d", fd);
    assert(fd >= 0);
    unsigned char result = 0x00;
 
    Byte    readbuf;  //  1 byte buffer
-   Byte    writebuf = {0x00};
+   // Byte    writebuf = {0x00};
    Status_Errno_DDC base_rc = 0;
 
-   base_rc = i2c_set_addr(fd, 0x30, CALLOPT_ERR_MSG);   // CALLOPT_ERR_MSG temporary
+#ifdef DISABLE
+   // Causes screen corruption on Dell XPS 13, which has a QHD+ eDP screen
+   base_rc = i2c_set_addr(fd, 0x30, CALLOPT_NONE);
    if (base_rc < 0) {
       goto bye;
    }
@@ -508,8 +497,11 @@ Status_Errno_DDC i2c_detect_ddc_addrs_by_fd(int fd, Byte * presult) {
    // DBGMSG("invoke_i2c_writer() returned %s", psc_desc(rc));
    if (base_rc == 0)
       result |= I2C_BUS_ADDR_0X30;
+#else
+   // DBGMSG("Skipping probe of slave address x30.");
+#endif
 
-   base_rc = i2c_set_addr(fd, 0x50, CALLOPT_ERR_MSG);   // CALLOPT_ERR_MSG temporary
+   base_rc = i2c_set_addr(fd, 0x50, CALLOPT_NONE);
    if (base_rc < 0) {
       goto bye;
    }
@@ -517,15 +509,26 @@ Status_Errno_DDC i2c_detect_ddc_addrs_by_fd(int fd, Byte * presult) {
    if (base_rc == 0)
       result |= I2C_BUS_ADDR_0X50;
 
-   base_rc = i2c_set_addr(fd, 0x37, CALLOPT_ERR_MSG);   // CALLOPT_ERR_MSG temporary
+#ifdef DISABLE
+   // 10/2018: Causes temporary screen corruption on Dell XPS 13, which has a QHD+ eDP screen
+   base_rc = i2c_set_addr(fd, 0x37, CALLOPT_NONE);
    if (base_rc < 0) {
       goto bye;
    }
-   base_rc = invoke_i2c_reader(fd, 1, &readbuf);
-   // DBGMSG("call_read() returned %d", rc);
-   // 11/2015: DDCRC_READ_ALL_ZERO currently set only in ddc_packet_io.c:
-   if (base_rc == 0 || base_rc == DDCRC_READ_ALL_ZERO)
+   // 7/2018: changed from invoke_i2c_reader() to invoke_i2c_writer()
+   //         Dell P2715Q does not always respond to read of single byte
+   base_rc = invoke_i2c_writer(fd, 1, &writebuf);
+   DBGTRC(debug, TRACE_GROUP,"invoke_i2c_writer() for slave address x37 returned %s", psc_desc(base_rc));
+   if (base_rc == 0) {
+      base_rc = invoke_i2c_reader(fd, 1, &readbuf);
+      DBGTRC(debug, TRACE_GROUP,"invoke_i2c_reader() for slave address x37 returned %s", psc_desc(base_rc));
+   }
+
+   if (base_rc == 0)
       result |= I2C_BUS_ADDR_0X37;
+#else
+   // DBGMSG("Skipping probe of slave address x37.");
+#endif
 
    base_rc = 0;
 
@@ -535,10 +538,12 @@ bye:
 
    *presult = result;
 
-   DBGMSF(debug, "Done.  Returning base_rc=%d, *presult = 0x%02x", base_rc, *presult);
+   DBGTRC(debug, TRACE_GROUP,
+          "Done.  Returning base_rc=%s, *presult = 0x%02x", psc_desc(base_rc), *presult);
    return base_rc;
 }
 
+#endif
 
 //
 // I2C Bus Inspection - EDID Retrieval
@@ -552,14 +557,14 @@ bye:
  * @retval  0        success
  * @retval  <0       error
  */
-Public_Status_Code i2c_get_raw_edid_by_fd(int fd, Buffer * rawedid) {
+Status_Errno_DDC i2c_get_raw_edid_by_fd(int fd, Buffer * rawedid) {
    bool debug = false;
    DBGTRC(debug, TRACE_GROUP, "Getting EDID for file %d", fd);
 
    bool conservative = true;
 
    assert(rawedid->buffer_size >= 128);
-   Public_Status_Code rc;
+   Status_Errno_DDC rc;
 
    rc = i2c_set_addr(fd, 0x50, CALLOPT_ERR_MSG);
    if (rc < 0) {
@@ -605,8 +610,8 @@ bye:
    if (rc < 0)
       rawedid->len = 0;
 
-   DBGTRC(debug, TRACE_GROUP, "Returning %s.  edidbuf contents:", psc_desc(rc));
    if (debug || IS_TRACING()) {
+      DBGMSG("Returning %s.  edidbuf contents:", psc_desc(rc));
       buffer_dump(rawedid);
    }
    return rc;
@@ -621,20 +626,20 @@ bye:
  *
  * @return status code
  */
-Public_Status_Code i2c_get_parsed_edid_by_fd(int fd, Parsed_Edid ** edid_ptr_loc) {
+Status_Errno_DDC i2c_get_parsed_edid_by_fd(int fd, Parsed_Edid ** edid_ptr_loc) {
    bool debug = false;
-   DBGTRC(debug, TRACE_GROUP, "Starting. fd=%d\n", fd);
+   DBGTRC(debug, TRACE_GROUP, "Starting. fd=%d", fd);
    Parsed_Edid * edid = NULL;
    Buffer * rawedidbuf = buffer_new(128, NULL);
 
-   Public_Status_Code rc = i2c_get_raw_edid_by_fd(fd, rawedidbuf);
+   Status_Errno_DDC rc = i2c_get_raw_edid_by_fd(fd, rawedidbuf);
    if (rc == 0) {
       edid = create_parsed_edid(rawedidbuf->bytes);
       if (debug) {
          if (edid)
             report_parsed_edid(edid, false /* dump hex */, 0);
          else
-            DBGMSG("create_parsed_edid() returned NULL", NULL);
+            DBGMSG("create_parsed_edid() returned NULL");
       }
       if (!edid)
          rc = DDCRC_INVALID_EDID;
@@ -684,18 +689,42 @@ static void i2c_check_bus(I2C_Bus_Info * bus_info) {
    int file = 0;
 
    if (!(bus_info->flags & I2C_BUS_PROBED)) {
-      DBGMSF(debug, "Probing", NULL);
+      DBGMSF(debug, "Probing");
       bus_info->flags |= I2C_BUS_PROBED;
 
+      if ( is_edp_device(bus_info->busno) ) {
+         DBGMSF(debug, "eDP device detected");
+         bus_info->flags |= I2C_BUS_EDP;
+         // goto bye;
+      }
 
-      // unnecessary, bus_info is already filtered
       // probing hangs on PowerMac if i2c device is SMU
-      // if (!is_ignorable_i2c_device(bus_info->busno)) {
+      // but, bus_info has already been filtered for ignorable devices
+      assert( !is_ignorable_i2c_device(bus_info->busno) );
 
+#ifdef OLD
+      // tests no longer needed, see comment on above assert()
+      if ( !(bus_info->flags & I2C_BUS_VALID_NAME_CHECKED) ) {
+         bus_info->flags |= I2C_BUS_VALID_NAME_CHECKED;
+         if ( !is_ignorable_i2c_device(bus_info->busno) )
+            bus_info->flags |= I2C_BUS_HAS_VALID_NAME;
+      }
+#endif
+      bus_info->flags |= I2C_BUS_VALID_NAME_CHECKED;
+      bus_info->flags |= I2C_BUS_HAS_VALID_NAME;
+
+      if (bus_info->flags & I2C_BUS_VALID_NAME_CHECKED &&
+          bus_info->flags & I2C_BUS_HAS_VALID_NAME )
+      {
          file = i2c_open_bus(bus_info->busno, CALLOPT_ERR_MSG);  // returns if failure
 
          if (file >= 0) {
             bus_info->flags |= I2C_BUS_ACCESSIBLE;
+
+            bus_info->functionality = i2c_get_functionality_flags_by_fd(file);
+            // DBGMSF(debug, "i2c_get_functionality_flags_by_fd() returned %lu", bus_info->functionality);
+
+#ifdef DETECT_SLAVE_ADDRS
             Byte ddc_addr_flags = 0x00;
             Status_Errno_DDC psc = i2c_detect_ddc_addrs_by_fd(file, &ddc_addr_flags);
             if (psc != 0) {
@@ -705,9 +734,7 @@ static void i2c_check_bus(I2C_Bus_Info * bus_info) {
                goto bye;
             }
             bus_info->flags |= ddc_addr_flags;
-            // DBGMSF(debug, "Calling i2c_get_functionality_flags_by_fd()...");
-            bus_info->functionality = i2c_get_functionality_flags_by_fd(file);
-            // DBGMSF(debug, "i2c_get_functionality_flags_by_fd() returned %lu", bus_info->functionality);
+
             if (bus_info->flags & I2C_BUS_ADDR_0X50) {
                // Have seen case of nouveau driver with Quadro card where
                // there's a bus that has no monitor but responds to the X50 probe
@@ -729,25 +756,37 @@ static void i2c_check_bus(I2C_Bus_Info * bus_info) {
             if (bus_info->flags & I2C_BUS_ADDR_0X37) {
                // have seen case where laptop display reports addr 37 active, but
                // it doesn't respond to DDC
+               // 8/2017: If DDC turned off on U3011 monitor, addr x37 still detected
                // TODO: sanity check for DDC goes here
                // or make this check at a higher level, since I2c doesn't understand DDC
 
             }
    #endif
-
+#else
+            Public_Status_Code psc = i2c_get_parsed_edid_by_fd(file, &bus_info->edid);
+            DBGMSF(debug, "i2c_get_parsed_edid_by_fd() returned %d", psc);
+            if (psc == 0)
+               bus_info->flags |= I2C_BUS_ADDR_0X50;
+            else
+               goto bye;     // not logically necessary, avoids iftesting label bye
+#endif
          }
       }
-   // }
+   }
 
 bye:
    if (file >= 0)
       i2c_close_bus(file, bus_info->busno,  CALLOPT_ERR_MSG);
 
-   DBGTRC(debug, TRACE_GROUP, "Done. flags=0x%02x", bus_info->flags );
+   // DBGTRC(debug, TRACE_GROUP, "Done. flags=0x%02x", bus_info->flags );
+   if (debug || IS_TRACING() ) {
+      DBGMSG("Done. flags=0x%04x", bus_info->flags );
+      i2c_dbgrpt_bus_info(bus_info, 1);
+   }
 }
 
 
-static void i2c_free_bus_info(I2C_Bus_Info * bus_info) {
+void i2c_free_bus_info(I2C_Bus_Info * bus_info) {
    if (bus_info) {
       if ( memcmp(bus_info->marker, "BINx", 4) != 0) {   // just ignore if already freed
          assert( memcmp(bus_info->marker, I2C_BUS_INFO_MARKER, 4) == 0);
@@ -800,13 +839,18 @@ void i2c_dbgrpt_bus_info(I2C_Bus_Info * bus_info, int depth) {
    DBGMSF(debug, "bus_info=%p", bus_info);
    assert(bus_info);
 
-   // rpt_nl();
-   rpt_vstring(depth, "Bus /dev/i2c-%d found:    %s", bus_info->busno, bool_repr(bus_info->flags&I2C_BUS_EXISTS));
-   rpt_vstring(depth, "Bus /dev/i2c-%d probed:   %s", bus_info->busno, bool_repr(bus_info->flags&I2C_BUS_PROBED ));
+   rpt_vstring(depth, "Bus /dev/i2c-%d found:   %s", bus_info->busno, sbool(bus_info->flags&I2C_BUS_EXISTS));
+   rpt_vstring(depth, "Bus /dev/i2c-%d probed:  %s", bus_info->busno, sbool(bus_info->flags&I2C_BUS_PROBED ));
    if ( bus_info->flags & I2C_BUS_PROBED ) {
-      rpt_vstring(depth, "Address 0x30 present:    %s", bool_repr(bus_info->flags & I2C_BUS_ADDR_0X30));
-      rpt_vstring(depth, "Address 0x37 present:    %s", bool_repr(bus_info->flags & I2C_BUS_ADDR_0X37));
-      rpt_vstring(depth, "Address 0x50 present:    %s", bool_repr(bus_info->flags & I2C_BUS_ADDR_0X50));
+      rpt_vstring(depth, "Bus accessible:          %s", sbool(bus_info->flags&I2C_BUS_ACCESSIBLE ));
+      rpt_vstring(depth, "Bus is eDP:              %s", sbool(bus_info->flags&I2C_BUS_EDP ));
+      rpt_vstring(depth, "Valid bus name checked:  %s", sbool(bus_info->flags & I2C_BUS_VALID_NAME_CHECKED));
+      rpt_vstring(depth, "I2C bus has valid name:  %s", sbool(bus_info->flags & I2C_BUS_HAS_VALID_NAME));
+#ifdef DETECT_SLAVE_ADDRS
+      rpt_vstring(depth, "Address 0x30 present:    %s", sbool(bus_info->flags & I2C_BUS_ADDR_0X30));
+      rpt_vstring(depth, "Address 0x37 present:    %s", sbool(bus_info->flags & I2C_BUS_ADDR_0X37));
+#endif
+      rpt_vstring(depth, "Address 0x50 present:    %s", sbool(bus_info->flags & I2C_BUS_ADDR_0X50));
       i2c_report_functionality_flags(bus_info->functionality, /* maxline */ 90, depth);
       if ( bus_info->flags & I2C_BUS_ADDR_0X50) {
          if (bus_info->edid) {
@@ -815,7 +859,7 @@ void i2c_dbgrpt_bus_info(I2C_Bus_Info * bus_info, int depth) {
       }
    }
 
-   DBGMSF(debug, "Done", NULL);
+   DBGMSF(debug, "Done");
 }
 
 
@@ -831,13 +875,20 @@ void i2c_report_active_display(I2C_Bus_Info * businfo, int depth) {
    DDCA_Output_Level output_level = get_output_level();
    rpt_vstring(depth, "I2C bus:             /dev/i2c-%d", businfo->busno);
 
-   if (output_level >= DDCA_OL_NORMAL)
-   rpt_vstring(depth, "Supports DDC:        %s", bool_repr(businfo->flags & I2C_BUS_ADDR_0X37));
+   // 08/2018 Disable.
+   // Test for DDC communication is now done more sophisticatedly at the DDC level
+   // The simple X37 test can have both false positives (DDC turned off in monitor but
+   // X37 responsive), and false negatives (Dell P2715Q)
+   // if (output_level >= DDCA_OL_NORMAL)
+   // rpt_vstring(depth, "Supports DDC:        %s", bool_repr(businfo->flags & I2C_BUS_ADDR_0X37));
 
    if (output_level >= DDCA_OL_VERBOSE) {
-      rpt_vstring(depth+1, "I2C address 0x30 (EDID block#)  present: %-5s", bool_repr(businfo->flags & I2C_BUS_ADDR_0X30));
-      rpt_vstring(depth+1, "I2C address 0x37 (DDC)          present: %-5s", bool_repr(businfo->flags & I2C_BUS_ADDR_0X37));
-      rpt_vstring(depth+1, "I2C address 0x50 (EDID)         present: %-5s", bool_repr(businfo->flags & I2C_BUS_ADDR_0X50));
+#ifdef DETECT_SLAVE_ADDRS
+      rpt_vstring(depth+1, "I2C address 0x30 (EDID block#)  present: %-5s", srepr(businfo->flags & I2C_BUS_ADDR_0X30));
+      rpt_vstring(depth+1, "I2C address 0x37 (DDC)          present: %-5s", srepr(businfo->flags & I2C_BUS_ADDR_0X37));
+#endif
+      rpt_vstring(depth+1, "I2C address 0x50 (EDID) present: %-5s", bool_repr(businfo->flags & I2C_BUS_ADDR_0X50));
+      rpt_vstring(depth+1, "Is eDP device:                   %-5s", sbool(businfo->flags & I2C_BUS_EDP));
 
       char fn[PATH_MAX];     // yes, PATH_MAX is dangerous, but not as used here
       sprintf(fn, "/sys/bus/i2c/devices/i2c-%d/name", businfo->busno);
@@ -924,32 +975,33 @@ int i2c_device_count() {
 
 int i2c_detect_buses() {
    bool debug = false;
-   DBGMSF(debug, "Starting.  i2c_buses = %p", i2c_buses);
+   DBGTRC(debug, DDCA_TRC_I2C, "Starting.  i2c_buses = %p", i2c_buses);
    if (!i2c_buses) {
+      // only returns buses with valid name (arg=false)
       Byte_Value_Array i2c_bus_bva = get_i2c_device_numbers_using_udev(false);
       // TODO: set free function
       i2c_buses = g_ptr_array_sized_new(bva_length(i2c_bus_bva));
       for (int ndx = 0; ndx < bva_length(i2c_bus_bva); ndx++) {
          int busno = bva_get(i2c_bus_bva, ndx);
-         DBGMSF(debug, "busno = %d", busno);
+         DBGMSF(debug, "Checking busno = %d", busno);
          I2C_Bus_Info * businfo = i2c_new_bus_info(busno);
-         businfo->flags = I2C_BUS_EXISTS;
+         businfo->flags = I2C_BUS_EXISTS | I2C_BUS_VALID_NAME_CHECKED | I2C_BUS_HAS_VALID_NAME;
          i2c_check_bus(businfo);
-         if (debug)
-            i2c_dbgrpt_bus_info(businfo, 0);
+         // if (debug || IS_TRACING() )
+         //    i2c_dbgrpt_bus_info(businfo, 0);
          g_ptr_array_add(i2c_buses, businfo);
       }
       bva_free(i2c_bus_bva);
    }
    int result = i2c_buses->len;
-   DBGMSF(debug, "Returning: %d", result);
+   DBGTRC(debug, DDCA_TRC_I2C, "Returning: %d", result);
    return result;
 }
 
 
 I2C_Bus_Info * detect_single_bus(int busno) {
    bool debug = false;
-   DBGMSF(debug, "Starting.  busno = %d", busno);
+   DBGTRC(debug, DDCA_TRC_I2C, "Starting.  busno = %d", busno);
    I2C_Bus_Info * businfo = NULL;
 
    if (i2c_device_exists(busno) ) {
@@ -960,7 +1012,7 @@ I2C_Bus_Info * detect_single_bus(int busno) {
          i2c_dbgrpt_bus_info(businfo, 0);
    }
 
-   DBGMSF(debug, "Done.  busnp=%d, returning: %p", busno, businfo);
+   DBGTRC(debug, DDCA_TRC_I2C, "Done.  busno=%d, returning: %p", busno, businfo);
    return businfo;
 }
 
@@ -992,7 +1044,7 @@ I2C_Bus_Info * i2c_get_bus_info_by_index(int busndx) {
    bus_info = g_ptr_array_index(i2c_buses, busndx);
    // report_businfo(busInfo);
    if (debug) {
-      DBGMSG("flags=0x%02x", bus_info->flags);
+      DBGMSG("flags=0x%04x", bus_info->flags);
       DBGMSG("flags & I2C_BUS_PROBED = 0x%02x", (bus_info->flags & I2C_BUS_PROBED) );
    }
    assert( bus_info->flags & I2C_BUS_PROBED );
@@ -1036,203 +1088,6 @@ I2C_Bus_Info * i2c_find_bus_info_by_busno(int busno) {
 }
 
 
-// Generalized Bus_Info search
-//
-// Overkill for current use.
-// Was coded at the time when selecting display by criteria occurred at the i2c/ad/usb
-// level rather than in ddc_displays.c.
-// Still used by USB layer as a fallback to find the EDID by model etc.
-// if the EDID can't be gotten from USB services.
-
-typedef struct {
-   int           busno;
-   const char *  mfg_id;
-   const char *  model_name;
-   const char *  serial_ascii;
-   const Byte *  edidbytes;
-   Byte          options;
-} I2C_Bus_Selector;
-
-
-static
-void report_i2c_bus_selector(I2C_Bus_Selector * sel, int depth) {
-   int d1 = depth+1;
-   int d2 = depth+2;
-   rpt_structure_loc("I2C_Bus_Selector", sel, depth);
-   rpt_int("busno",        NULL, sel->busno, d1);
-   rpt_str("mfg_id",       NULL, sel->mfg_id, d1);
-   rpt_str("model_name",   NULL, sel->model_name, d1);
-   rpt_str("serial_ascii", NULL, sel->serial_ascii, d1);
-   rpt_structure_loc("edidbytes", sel->edidbytes, d1);
-   if (sel->edidbytes)
-      rpt_hex_dump(sel->edidbytes, 128, d2);
-}
-
-static
-void init_i2c_bus_selector(I2C_Bus_Selector* sel) {
-   assert(sel);
-   memset(sel, 0, sizeof(I2C_Bus_Selector));
-   sel->busno = -1;
-}
-
-// Note: No need for free_i2c_bus_selector() function since strings and memory
-// pointed to by selector are always in other data structures
-
-
-/* Tests if a bus_info table entry matches the criteria of a selector.
- *
- * Arguments:
- *   bus_info    pointer to Bus_Info instance to test
- *   sel         selection criteria
- *
- * Returns:      true/false
- */
-static
-bool bus_info_matches_selector(I2C_Bus_Info * bus_info, I2C_Bus_Selector * sel) {
-   bool debug = false;
-   if (debug) {
-      DBGMSG("Starting", NULL);
-      i2c_dbgrpt_bus_info(bus_info, 1);
-   }
-
-   assert( bus_info && sel);
-   assert( sel->busno >= 0   ||
-           sel->mfg_id       ||
-           sel->model_name   ||
-           sel->serial_ascii ||
-           sel->edidbytes);
-
-   bool result = false;
-   // does the bus represent a valid display?
-   if (sel->options & DISPSEL_VALID_ONLY) {
-      if (!(bus_info->flags & I2C_BUS_ADDR_0X37))
-         goto bye;
-   }
-   bool some_test_passed = false;
-
-   if (sel->busno >= 0) {
-      DBGMSF(debug, "bus_info->busno = %d", bus_info->busno);
-      if (sel->busno != bus_info->busno)  {
-         result = false;
-         goto bye;
-      }
-      DBGMSF(debug, "busno test passed", NULL);
-      some_test_passed = true;
-   }
-
-   Parsed_Edid * edid = bus_info->edid;  // will be NULL for I2C bus with no monitor
-
-   if (sel->mfg_id && strlen(sel->mfg_id) > 0) {
-      if ((!edid) || strlen(edid->mfg_id) == 0 || !streq(sel->mfg_id, edid->mfg_id) ) {
-         result = false;
-         goto bye;
-      }
-      some_test_passed = true;
-   }
-   if (sel->model_name && strlen(sel->model_name) > 0) {
-      if ((!edid) || strlen(edid->model_name) == 0 || !streq(sel->model_name, edid->model_name) ) {
-         result = false;
-         goto bye;
-      }
-      some_test_passed = true;
-   }
-   if (sel->serial_ascii && strlen(sel->serial_ascii) > 0) {
-      if ((!edid) || strlen(edid->serial_ascii) == 0 || !streq(sel->serial_ascii, edid->serial_ascii) ) {
-         result = false;
-         goto bye;
-      }
-      some_test_passed = true;
-   }
-   if (sel->edidbytes) {
-      if ((!edid) || !memcmp(sel->edidbytes, edid->bytes, 128) != 0  ) {
-         result = false;
-         goto bye;
-      }
-      some_test_passed = true;
-   }
-   if (some_test_passed)
-      result = true;
-
-bye:
-   DBGMSF(debug, "Returning: %s", bool_repr(result));
-   return result;
-}
-
-
-/* Finds the first Bus_Info instance that matches a selector
- *
- * Arguments:
- *   sel       pointer to selection criteria
- *
- * Returns:    pointer to Bus_Info instance if found, NULL if not
- */
-static
-I2C_Bus_Info * find_bus_info_by_selector(I2C_Bus_Selector * sel) {
-   assert(sel);
-   bool debug = false;
-   if (debug) {
-      DBGMSG("Starting.", NULL);
-      report_i2c_bus_selector(sel, 1);
-   }
-
-   I2C_Bus_Info * bus_info = NULL;
-   assert(i2c_buses);
-   int busct = i2c_buses->len;
-
-   for (int ndx = 0; ndx < busct; ndx++) {
-      I2C_Bus_Info * cur_info = g_ptr_array_index(i2c_buses, ndx);
-      if (bus_info_matches_selector(cur_info, sel)) {
-         bus_info = cur_info;
-         break;
-      }
-   }
-
-    DBGMSF(debug, "returning %p", bus_info );
-    if (debug && bus_info) {
-       i2c_dbgrpt_bus_info(bus_info, 1);
-    }
-    return bus_info;
- }
-
-
-// Finally, functions that use the generalized bus selection mechanism
-
-/** Retrieves bus information by some combination of the monitor's
- * mfg id, model name and/or serial number.
- *
- *  @param  mfg_id  3 character manufacturer id
- *  @param  model     monitor model (as listed in the EDID)
- *  @param  sn        monitor ascii serial number (as listed in the EDID)
- *  @param  findopts  selector options
- *
- * @return pointer to Bus_Info struct for the bus,\n
- *         NULL if not found
- *
- * @remark used by get_fallback_hiddev_edid() in usb_edid.c
- */
-I2C_Bus_Info *
-i2c_find_bus_info_by_mfg_model_sn(
-      const char * mfg_id,
-      const char * model,
-      const char * sn,
-      Byte findopts)
-{
-   bool debug = false;
-   DBGMSF(debug, "Starting. mfg_id=|%s|, model=|%s|, sn=|%s|", mfg_id, model, sn );
-   assert(mfg_id || model || sn);    // loosen the requirements
-
-   I2C_Bus_Selector sel;
-   init_i2c_bus_selector(&sel);
-   sel.mfg_id       = mfg_id;
-   sel.model_name   = model;
-   sel.serial_ascii = sn;
-   sel.options      = findopts;
-   I2C_Bus_Info * result = find_bus_info_by_selector(&sel);
-
-   DBGMSF(debug, "Returning: %p", result );
-   return result;
-}
-
 
 //
 // I2C Bus Inquiry
@@ -1333,70 +1188,11 @@ int i2c_report_buses(bool report_all, int depth) {
 }
 
 
-//
-// For test driver use only
-//
-
-static bool is_function_supported(int busno, char * funcname) {
-   bool debug = false;
-   DBGMSF(debug, "Starting. busno=%d, funcname=%s", busno, funcname);
-
-   bool result = true;
-   if ( !streq(funcname, "read") &&  !streq(funcname, "write") ) {
-      uint32_t func_bit = vnt_find_id(
-                               functionality_table2,
-                               funcname,
-                               true,      //  search title field
-                               false,     //  ignore_case,
-                               0x00);     //   default_id);
-
-      if (!func_bit) {
-         DBGMSG("Unrecognized function name: %s", funcname);
-         result = false;
-         goto bye;
-      }
-
-      // DBGMSG("functionality=0x%lx, func_table_entry->bit=-0x%lx", bus_infos[busno].functionality, func_table_entry->bit);
-      // Bus_Info * bus_info = i2c_get_bus_info(busno, DISPSEL_NONE);
-      // Bus_Info * bus_info = i2c_get_bus_info_new(busno);
-      I2C_Bus_Info * bus_info = detect_single_bus(busno);
-      if ( !bus_info ) {
-         DBGMSG("Invalid bus: /dev/i2c-%d", busno);
-         result = false;
-      }
-      else   // add unneeded else clause to avoid clang warning
-         result = (bus_info->functionality & func_bit) != 0;
-      i2c_free_bus_info(bus_info);
-   }
-
-bye:
-   DBGMSF(debug, "busno=%d, funcname=%s, returning %d", busno, funcname, result);
+#ifdef UNUSED
+bool is_probably_laptop_display(I2C_Bus_Info * businfo) {
+   assert(businfo);
+   bool result = (businfo->flags & I2C_BUS_EDP) || is_embedded_parsed_edid(businfo->edid);
    return result;
 }
+#endif
 
-
-/** Verify that the specified I2C write and read functions are supported.
- *
- *  This function is used in test management.
- *
- *  @param  busno   I2C bus number
- *  @param  write_func_name  write function name
- *  @param  read_func_name   read function name
- *
- *  @return true/false
- */
-bool i2c_verify_functions_supported(int busno, char * write_func_name, char * read_func_name) {
-   // printf("(%s) Starting. busno=%d, write_func_name=%s, read_func_name=%s\n",
-   //        __func__, busno, write_func_name, read_func_name);
-   bool write_supported = is_function_supported(busno, write_func_name);
-   bool read_supported  = is_function_supported(busno, read_func_name);
-
-   if (!write_supported)
-      printf("Unsupported write function: %s\n", write_func_name );
-   if (!read_supported)
-      printf("Unsupported read function: %s\n", read_func_name );
-
-   bool result =write_supported && read_supported;
-   // DBGMSG("returning %d", result);
-   return result;
-}
