@@ -1,32 +1,11 @@
-/* ddcutil_c_api.h
+/** @file ddcutil_c_api.h
  *
- * Public C APi for ddcutil. .
- *
- * <copyright>
- * Copyright (C) 2014-2018 Sanford Rockowitz <rockowitz@minsoft.com>
- *
- * Licensed under the GNU General Public License Version 2
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- * </endcopyright>
- */
-
-/** \file ddcutil_c_api.h
  *  Public C API for ddcutil
  */
 
+// Copyright (C) 2014-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// SPDX-License-Identifier: GPL-2.0-or-later
+
 #ifndef DDCUTIL_C_API_H_
 #define DDCUTIL_C_API_H_
 
@@ -46,7 +25,7 @@ extern "C" {
  *  @brief ddcutil public C API
  *
  *  Function names in the public C API begin with "ddca_"\n
- *  Status codes begin with "DDCRC_".
+ *  Status codes begin with "DDCRC_".\n
  *  Typedefs, other constants, etc. begin with "DDCA_".
  */
 
@@ -78,31 +57,18 @@ extern "C" {
  *
  * @return version numbers
  */
-DDCA_Ddcutil_Version_Spec ddca_ddcutil_version(void);       // ddcutil version
+DDCA_Ddcutil_Version_Spec
+ddca_ddcutil_version(void);
 
 /**
  * Returns the ddcutil version as a string in the form "major.minor.micro".
  *
  * @return version string.
  */
-const char * ddca_ddcutil_version_string(void);
-
+const char *
+ddca_ddcutil_version_string(void);
 
 // Bit ids for ddca_get_build_options() - how to make connection in doxygen?
-/** Build option flags
- *
- *  The enum values are defined as 1,2,4 etc so that they can be or'd.
- */
-typedef enum {
-   /** @brief ddcutil was built with support for AMD Display Library connected monitors */
-   DDCA_BUILT_WITH_ADL     = 0x01,
-   /** @brief ddcutil was built with support for USB connected monitors */
-   DDCA_BUILT_WITH_USB     = 0x02,
-  /** @brief ddcutil was built with support for failure simulation */
-   DDCA_BUILT_WITH_FAILSIM = 0x04
-} DDCA_Build_Option_Flags;
-
-
 #ifdef ALT
  * Defined Bits
  * <table>
@@ -123,7 +89,8 @@ typedef enum {
  * |#DDCA_BUILT_WITH_FAILSIM | built with failure simulation
  *
  */
-DDCA_Build_Option_Flags ddca_build_options(void);
+DDCA_Build_Option_Flags
+ddca_build_options(void);
 
 
  //
@@ -134,11 +101,12 @@ DDCA_Build_Option_Flags ddca_build_options(void);
   *  API call, if the call supports detailed error information (only a
   *  few do).
   *
-  *  @return  copy of detailed error information (user most free)
+  *  @return  copy of detailed error information (user must free)
   *
   *  @since 0.9.0
   */
- DDCA_Error_Detail * ddca_get_error_detail();
+ DDCA_Error_Detail *
+ ddca_get_error_detail(void);
 
  /** Frees a detailed error information record
   *
@@ -146,7 +114,19 @@ DDCA_Build_Option_Flags ddca_build_options(void);
   *
   *  @since 0.9.0
   */
- void ddca_free_error_detail(DDCA_Error_Detail * ddca_erec);
+ void
+ ddca_free_error_detail(
+       DDCA_Error_Detail * ddca_erec);
+
+ /** Issues a detailed report of a #DDCA_Error_Detail instance.
+  *
+  *  @param ddca_erec  error information record
+  *  @param depth      logical indentation depth
+  */
+ void
+ ddca_report_error_detail(
+       DDCA_Error_Detail * ddca_erec,
+       int                 depth);
 
 
 //
@@ -161,9 +141,11 @@ DDCA_Build_Option_Flags ddca_build_options(void);
  *
  * @remark
  * The returned value is a pointer into internal persistent
- * data structures and should not be free'd by the caller.
+ * data structures and should not be freed by the caller.
  *  */
-char * ddca_rc_name(DDCA_Status status_code);
+const char *
+ddca_rc_name(
+      DDCA_Status status_code);
 
 /** Returns a description of a ddcutil status code
  *
@@ -175,39 +157,9 @@ char * ddca_rc_name(DDCA_Status status_code);
  * The returned value is a pointer into internal persistent
  * data structures and should not be free'd by the caller.
  */
-char * ddca_rc_desc(DDCA_Status status_code);
-
-
-//
-// MCCS Version Id
-//
-
-/** \deprecated
- *  Returns the symbolic name of a #DDCA_MCCS_Version_Id,
- *  e.g. "DDCA_MCCS_V20."
- *
- *  @param[in]  version_id  version id value
- *  @return symbolic name (do not free)
- */
-char *
-__attribute__ ((deprecated))
-ddca_mccs_version_id_name(
-      DDCA_MCCS_Version_Id  version_id) __attribute__ ((deprecated));
-
-/** \deprecated
- *  Returns the descriptive name of a #DDCA_MCCS_Version_Id,
- *  e.g. "2.0".
- *
- *  @param[in]  version_id  version id value
- *  @return descriptive name (do not free)
- *
- *  @remark added to replace ddca_mccs_version_id_desc() during 0.9
- *  development, but then use of DDCA_MCCS_Version_Id deprecated
- */
-char *
-__attribute__ ((deprecated))
-ddca_mccs_version_id_desc(
-      DDCA_MCCS_Version_Id  version_id) ;
+const char *
+ddca_rc_desc(
+      DDCA_Status status_code);
 
 
 //
@@ -232,7 +184,8 @@ There are 3 retry contexts:
  *
  * @return maximum max tries value allowed on set_max_tries()
  */
-int ddca_max_max_tries(void);
+int
+ddca_max_max_tries(void);
 
 /** Gets the maximum number of I2C retries for the specified operation type.
  * @param[in]  retry_type   I2C operation type
@@ -241,21 +194,22 @@ int ddca_max_max_tries(void);
  * @remark
  * This setting is global, not thread-specific.
  */
-int ddca_get_max_tries(
-       DDCA_Retry_Type retry_type);
+int
+ddca_get_max_tries(
+    DDCA_Retry_Type retry_type);
 
 /** Sets the maximum number of I2C retries for the specified operation type
  * @param[in] retry_type    I2C operation type
  * @param[in] max_tries     maximum count to set
- * @retval   -EINVAL        max_tries < 1 or > #ddca_get_max_tries()
+ * @retval    DDCRC_ARG     max_tries < 1 or > #ddca_get_max_tries()
  *
  * @remark
  * This setting is global, not thread-specific.
  */
 DDCA_Status
 ddca_set_max_tries(
-      DDCA_Retry_Type retry_type,
-      int             max_tries);
+    DDCA_Retry_Type retry_type,
+    int             max_tries);
 ///@}
 
 /** Controls whether VCP values are read after being set.
@@ -265,7 +219,9 @@ ddca_set_max_tries(
  *
  * \remark This setting is thread-specific.
  */
-bool ddca_enable_verify(bool onoff);
+bool
+ddca_enable_verify(
+      bool onoff);
 
 /** Query whether VCP values are read after being set.
  * \retval true values are verified after being set
@@ -273,7 +229,8 @@ bool ddca_enable_verify(bool onoff);
  *
  * \remark This setting is thread-specific.
  */
-bool ddca_is_verify_enabled(void);
+bool
+ddca_is_verify_enabled(void);
 
 
 //
@@ -281,18 +238,22 @@ bool ddca_is_verify_enabled(void);
 //
 
 /** Redirects output on the current thread that normally would go to **stdout**  */
-void ddca_set_fout(
-        FILE * fout);   /**< where to write normal messages, if NULL, suppress output */
+void
+ddca_set_fout(
+    FILE * fout);   /**< where to write normal messages, if NULL, suppress output */
 
 /** Redirects output on the current thread that normally goes to **stdout** back to **stdout** */
-void ddca_set_fout_to_default(void);
+void
+ddca_set_fout_to_default(void);
 
 /** Redirects output on the current thread that normally would go to **stderr**  */
-void ddca_set_ferr(
-        FILE * ferr);   /**< where to write error messages, If NULL, suppress output */
+void
+ddca_set_ferr(
+    FILE * ferr);   /**< where to write error messages, If NULL, suppress output */
 
 /** Redirects output on the current thread that normally goes to **stderr** back to **stderr** */
-void ddca_set_ferr_to_default(void);
+void
+ddca_set_ferr_to_default(void);
 
 
 //
@@ -300,24 +261,15 @@ void ddca_set_ferr_to_default(void);
 // to an in-memory buffer.
 //
 
-/** Capture option flags
- *
- *  The enum values are defined as 1,2,4 etc so that they can be or'd.
- *
- *  @since 0.9.0
- */
-typedef enum {
-   DDCA_CAPTURE_NOOPTS     = 0,     ///< @brief no options specified
-   DDCA_CAPTURE_STDERR     = 1      ///< @brief capture **stderr** as well as **stdout**
-} DDCA_Capture_Option_Flags;
-
 /** Begins capture of **stdout** and optionally **stderr** output on the
  *  current thread to a thread-specific in-memory buffer.
  *
  *  @note  If output is already being captured, this function has no effect.
  *  @since 0.9.0
  */
-void ddca_start_capture(DDCA_Capture_Option_Flags flags);
+void
+ddca_start_capture(
+      DDCA_Capture_Option_Flags flags);
 
 /** Ends capture of **stdout** output and returns the contents of the
  *  in-memory buffer.
@@ -328,10 +280,14 @@ void ddca_start_capture(DDCA_Capture_Option_Flags flags);
  *
  *  @return captured output as a string.
  *
+ *  @note
+ *  If output is not currently being captured, returns a 0 length string.
+ *
  *  @note  Writes messages to actual **stderr** in case of error.
  *  @since 0.9.0
  */
-char * ddca_end_capture(void);
+char *
+ddca_end_capture(void);
 
 
 //
@@ -356,8 +312,9 @@ ddca_set_output_level(
  *  @param[in]  val  output level id
  *  @return     output level name (do not free)
  */
-char * ddca_output_level_name(
-          DDCA_Output_Level val);   /**< output level id */
+char *
+ddca_output_level_name(
+      DDCA_Output_Level val);   /**< output level id */
 
 
 /** Controls whether messages describing DDC protocol errors are output
@@ -366,13 +323,60 @@ char * ddca_output_level_name(
  *
  *  This setting is global to all threads.
  */
-bool ddca_enable_report_ddc_errors(bool onoff);
+bool
+ddca_enable_report_ddc_errors(
+      bool onoff);
 
-/** Checks whether messages describing DDC protocol errors are output.
+/** Indicates whether messages describing DDC protocol errors are output.
  *
  *  This setting is global to all threads.
  */
-bool ddca_is_report_ddc_errors_enabled(void);
+bool
+ddca_is_report_ddc_errors_enabled(void);
+
+
+//
+// Tracing
+//
+
+/** Turn on tracing for a specific function.
+ *
+ *  \param  funcname   function name
+ *
+ *  \remark
+ *  The function must include trace calls.
+ */
+void
+ddca_add_traced_function(
+      const char * funcname);
+
+/** Turn on all tracing in a specific source file.
+ *
+ *  \param filename  simple file name, with or without the ".c" extension,
+ *                   e.g. vcp_feature_set.c, vcp_feature_set
+ */
+void
+ddca_add_traced_file(
+      const char * filename);
+
+/* Specify one or more trace groups.
+ *
+ *  \param trace_flags  bitfield indicating groups to trace
+ */
+void
+ddca_set_trace_groups(
+      DDCA_Trace_Group trace_flags);
+
+
+/** Given a trace group name, returns its identifier.
+ *  Case is ignored.
+ *
+ *  \param   name trace group name
+ *  \return  trace group identifier
+ *  \retval  TRC_NEVER unrecognized name
+ */
+DDCA_Trace_Group
+ddca_trace_group_name_to_value(char * name);
 
 
 //
@@ -382,14 +386,18 @@ bool ddca_is_report_ddc_errors_enabled(void);
 //
 
 /** Resets all **ddcutil** statistics */
-void ddca_reset_stats(void);
+void
+ddca_reset_stats(void);
 
 /** Show execution statistics.
  *
  *  \param[in] stats  bitflags of statistics types to show
  *  \param[in] depth  logical indentation depth
  */
-void ddca_show_stats(DDCA_Stats_Type stats, int depth);
+void
+ddca_show_stats(
+      DDCA_Stats_Type stats,
+      int             depth);
 
 // TODO: Add functions to get stats
 
@@ -398,25 +406,39 @@ void ddca_show_stats(DDCA_Stats_Type stats, int depth);
  *  @param  enable   true/false
  *  @return prior value
  */
-bool ddca_enable_error_info(bool enable);
-
-
+bool
+ddca_enable_error_info(
+      bool enable);
 
 
 //
 // Display Descriptions
 //
 
-/** @deprecated use #ddca_get_display_info_list2()
- * Gets a list of the detected displays.
+/** Controls whether USB devices are checked during display detection
  *
- *  Displays that do not support DDC are not included.
+ *  Must be called before any API call that triggers display detection.
  *
- *  @return list of display summaries
+ *  @param onoff
+ *  @retval  DDCRC_OK                success
+ *  @retval  DDCRC_INVALID_OPERATION display detection has already occurred
+ *  @retval  DDCRC_UNIMPLEMENTED     ddcutil not built with USB monitor support
+ *
+ *  @remark
+ *  The default is to check USB devices.
+ *
+ *  This setting is global to all threads.
  */
-__attribute__ ((deprecated))
-DDCA_Display_Info_List *
-ddca_get_display_info_list(void);
+DDCA_Status
+ddca_enable_usb_display_detection(bool onoff);
+
+/** Reports whether USB devices are checked as part of display detection
+ *
+ *  @retval true  USB devices are checked
+ *  @retval valse UBB devices are not checked
+ */
+bool
+ddca_ddca_is_usb_display_detection_enabled();
 
 /** Gets a list of the detected displays.
  *
@@ -437,7 +459,9 @@ ddca_get_display_info_list2(
  *
  *  \param[in] dlist pointer to #DDCA_Display_Info_List
  */
-void ddca_free_display_info_list(DDCA_Display_Info_List * dlist);
+void
+ddca_free_display_info_list(
+      DDCA_Display_Info_List * dlist);
 
 /** Presents a report on a single display.
  *  The report is written to the current FOUT device for the current thread.
@@ -465,19 +489,6 @@ ddca_report_display_info_list(
       DDCA_Display_Info_List * dlist,
       int                      depth);
 
-
-/** \deprecated use #DDCA_Report_Displays()
- * Reports on all active displays.
- *  This function hooks into the code used by command "ddcutil detect"
- *
- *  @param[in] depth  logical indentation depth
- *  @return    number of MCCS capable displays
- */
-int
-ddca_report_active_displays(
-      int depth);
-
-
 /** Reports on all active displays.
  *  This function hooks into the code used by command "ddcutil detect"
  *
@@ -535,12 +546,12 @@ ddca_create_adlno_display_identifier(
 
 /** Creates a display identifier using some combination of the manufacturer id,
  * model name string and serial number string.  At least 1 of the 3 must be specified.
- * @param[in]  mfg_id  3 letter manufacturer id
- * @param[in]  model   model name string
- * @param[in]  sn     serial number string
+ * @param[in]  mfg_id    3 letter manufacturer id
+ * @param[in]  model     model name string
+ * @param[in]  sn        serial number string
  * @param[out] did_loc   where to return display identifier handle
- * @retval 0       success
- * @retval -EINVAL no argument specified, or argument too long
+ * @retval     0         success
+ * @retval     DDCRC_ARG all arguments NULL, or at least 1 too long
  *
  * \ingroup api_display_spec
  */
@@ -552,10 +563,10 @@ ddca_create_mfg_model_sn_display_identifier(
       DDCA_Display_Identifier* did_loc);
 
 /** Creates a display identifier using a 128 byte EDID
- * @param[in]   edid  pointer to 128 byte EDID
- * @param[out]  did_loc  where to return display identifier handle
- * @retval  0       success
- * @retval  -EINVAL edid==NULL
+ * @param[in]   edid       pointer to 128 byte EDID
+ * @param[out]  did_loc    where to return display identifier handle
+ * @retval      0          success
+ * @retval      DDCRC_ARG  edid==NULL
  *
  * \ingroup api_display_spec
  */
@@ -612,7 +623,7 @@ ddca_free_display_identifier(
  *
  *  \ingroup api_display_spec
  */
-char *
+const char *
 ddca_did_repr(
       DDCA_Display_Identifier did);
 
@@ -621,33 +632,59 @@ ddca_did_repr(
 // Display Reference
 //
 
-/** Gets a display reference for a display identifier.
+/**  \deprecated use #ddca_get_display_ref()
+ *  Gets a display reference for a display identifier.
  *  Normally, this is a permanently allocated #DDCA_Display_Ref
  *  created by monitor detection and does not need to be freed.
  *  Use #ddca_free_display_ref() to safely free.
  * @param[in]  did      display identifier
  * @param[out] dref_loc where to return display reference
- * @retval     0 success
- * @retval     -EINVAL  did is not a valid display identifier handle
+ * @retval     0                     success
+ * @retval     DDCRC_ARG             did is not a valid display identifier handle
  * @retval     DDCRC_INVALID_DISPLAY display not found
  *
  * \ingroup api_display_spec
  */
+// __attribute__ ((deprecated ("use ddca_get_display_ref()")))
 DDCA_Status
 ddca_create_display_ref(
       DDCA_Display_Identifier did,
       DDCA_Display_Ref*       dref_loc);
 
-/** Frees a display reference.
+
+/** Gets a display reference for a display identifier.
+ *  This is a permanently allocated #DDCA_Display_Ref
+ *  created by monitor detection and does not need to be freed.
+ * @param[in]  did      display identifier
+ * @param[out] dref_loc where to return display reference
+ * @retval     0                     success
+ * @retval     DDCRC_ARG             did is not a valid display identifier handle
+ * @retval     DDCRC_INVALID_DISPLAY display not found
+ *
+ * \since 0.9.5
+ * \ingroup api_display_spec
+ */
+DDCA_Status
+ddca_get_display_ref(
+      DDCA_Display_Identifier did,
+      DDCA_Display_Ref*       dref_loc);
+
+/**  \deprecated All display references are persistent
+ *
+ * Frees a display reference.
  *
  * Use this function to safely release a #DDCA_Display_Ref.
  * If the display reference was dynamically created, it is freed.
  * If the display references was permanently allocated (normal case), does nothing.
  * @param[in] dref  display reference to free
- * @return status code
+ * @retval DDCRC_OK     success, or dref == NULL
+ * @retval DDCRC_ARG    dref does not point to a valid display reference
+ * @retval DDCRC_LOCKED dref is to a transient instance, and it is referenced
+ *                      by an open display handle
  *
  * \ingroup api_display_spec
  */
+// __attribute__ ((deprecated ("DDCA_Display_Refs are always persistent")))
 DDCA_Status
 ddca_free_display_ref(
       DDCA_Display_Ref dref);
@@ -660,7 +697,7 @@ ddca_free_display_ref(
  *  @param[in]   dref display reference
  *  @return      string representation of display reference, NULL if invalid
  */
-char *
+const char *
 ddca_dref_repr(
       DDCA_Display_Ref dref);
 
@@ -680,20 +717,6 @@ ddca_dbgrpt_display_ref(
 // Display Handle
 //
 
-/** \deprecated Use #ddca_open_display2()
- * Open a display
- * @param[in]  ddca_dref    display reference for display to open
- * @param[out] ddca_dh_loc  where to return display handle
- * @return     status code
- *
- * Fails if display is already opened by another thread.
- * \ingroup api_display_spec
- */
-DDCA_Status
-ddca_open_display(
-      DDCA_Display_Ref      ddca_dref,
-      DDCA_Display_Handle * ddca_dh_loc);
-
 /** Open a display
  * @param[in]  ddca_dref    display reference for display to open
  * @param[in]  wait         if true, wait if display locked by another thread
@@ -710,8 +733,10 @@ ddca_open_display2(
       DDCA_Display_Handle * ddca_dh_loc);
 
 /** Close an open display
- * @param[in]  ddca_dh   display handle
- * @return     DDCA status code
+ * @param[in]  ddca_dh   display handle, if NULL do nothing
+ * @retval     DDCRC_OK  close succeeded, or ddca_dh == NULL
+ * @retval     DDCRC_ARG invalid handle
+ * @return     -errno    from underlying OS close()
  *
  * \ingroup api_display_spec
  */
@@ -728,7 +753,7 @@ ddca_close_display(
  *
  *  \ingroup api_display_spec
  */
-char *
+const char *
 ddca_dh_repr(
       DDCA_Display_Handle   ddca_dh);
 
@@ -798,17 +823,61 @@ ddca_free_parsed_capabilities(
  *  information is written, including command codes.
  *
  *  @param[in]  parsed_capabilities  pointer to #DDCA_Capabilities struct
+ *  @param[in]  ddca_dref            display reference, may be NULL
  *  @param[in]  depth  logical       indentation depth
  *
  *  @remark
- *  Future:  Feature value names will reflect any loaded monitor definition files
+ *  If ddca_dref is not NULL, feature value names will reflect any loaded monitor definition files
+ *  @since 0.9.3
+ */
+DDCA_Status
+ddca_report_parsed_capabilities_by_dref(
+      DDCA_Capabilities *      parsed_capabilities,
+      DDCA_Display_Ref         ddca_dref,
+      int                      depth);
+
+/** Reports the contents of a DDCA_Capabilities struct.
+ *
+ *  The report is written to the current FOUT location.
+ *
+ *  If the current output level is #DDCA_OL_VERBOSE, additional
+ *  information is written, including command codes.
+ *
+ *  @param[in]  parsed_capabilities  pointer to #DDCA_Capabilities struct
+ *  @param[in]  ddca_dh              display handle, may be NULL
+ *  @param[in]  depth                logical indentation depth
+ *  @retval     0                    success
+ *  @retval     DDCRC_ARG            invalid display handle
+ *
+ *  @remark
+ *  If ddca_dh is not NULL, feature value names will reflect any loaded monitor definition files
+ *  @since 0.9.3
+ */
+DDCA_Status
+ddca_report_parsed_capabilities_by_dh(
+      DDCA_Capabilities *      p_caps,
+      DDCA_Display_Handle      ddca_dh,
+      int                      depth);
+
+/** Reports the contents of a DDCA_Capabilities struct.
+ *
+ *  The report is written to the current FOUT location.
+ *
+ *  If the current output level is #DDCA_OL_VERBOSE, additional
+ *  information is written, including command codes.
+ *
+ *  @param[in]  parsed_capabilities  pointer to #DDCA_Capabilities struct
+ *  @param[in]  ddca_dref            display reference
+ *  @param[in]  depth  logical       indentation depth
+ *
+ *  @remark
+ *  Any user supplied feature definitions for the monitor are ignored.
  */
 void
 ddca_report_parsed_capabilities(
       DDCA_Capabilities *      parsed_capabilities,
       int                      depth);
 
-
 /** Returns the VCP feature codes defined in a
  *  parsed capabilities record as a #DDCA_Feature_LIst
  *
@@ -821,7 +890,6 @@ ddca_feature_list_from_capabilities(
       DDCA_Capabilities * parsed_caps);
 
 
-
 //
 //  MCCS Version Specification
 //
@@ -840,229 +908,206 @@ ddca_get_mccs_version_by_dh(
       DDCA_MCCS_Version_Spec* p_vspec);
 
 
-/** \deprecated */
-__attribute__ ((deprecated))
-DDCA_Status
-ddca_get_mccs_version_id(
-      DDCA_Display_Handle     ddca_dh,
-      DDCA_MCCS_Version_Id*   p_version_id);
-
-
 //
 // VCP Feature Metadata
 //
 
-// Deprecated functions using DDCA_Version_Feature_Info
+/** Controls whether user defined features (aka dynamic features) are supported.
+ *
+ *  @param[in] onoff true/false
+ *  @return    prior value
+ *
+ *  @since 0.9.3
+ */
+bool
+ddca_enable_udf(bool onoff);
 
-/** \deprecated Use #ddca_get_feature_flags_by_vspec() */
-__attribute__ ((deprecated))
-DDCA_Status
-ddca_get_feature_info_by_vcp_version(
-      DDCA_Vcp_Feature_Code         feature_code,
-      DDCA_MCCS_Version_Id          mccs_version_id,
-      DDCA_Version_Feature_Info**   info_loc) ;
-
-/** \deprecated Use #ddca_get_feature_flags_by_vspec() */
-__attribute__ ((deprecated))
-DDCA_Status
-ddca_get_feature_info_by_display(
-      DDCA_Display_Handle           ddca_dh,
-      DDCA_Vcp_Feature_Code         feature_code,
-      DDCA_Version_Feature_Info **  info_loc);
-
-/** \deprecated */
-__attribute__ ((deprecated))
-DDCA_Status
-ddca_free_feature_info(
-      DDCA_Version_Feature_Info * info);
-
-// Deprecated functions using DDCA_MCCS_Version_Id
-
-/** \deprecated Use #ddca_get_simple_sl_value_table_by_vspec() */
-__attribute__ ((deprecated))
-DDCA_Status
-ddca_get_simple_sl_value_table(
-      DDCA_Vcp_Feature_Code       feature_code,
-      DDCA_MCCS_Version_Id        mccs_version_id,
-      DDCA_Feature_Value_Table *  value_table_loc);   // DDCA_Feature_Value_Entry **
-
-
-// New master functions for feature metadata
+/** Query whether user defined features (aka dynamic features) are supported.
+ *
+ *  @retval true  UDF enabled
+ *  @retval false UDF disabled
+ *
+ *  @since 0.9.3
+ */
+bool
+ddca_is_udf_enabled(void);
 
 /**
- * Gets information for a VCP feature.
+ * Loads any user supplied feature definition files for the specified
+ * display.  Does nothing if they have already been loaded.
+ *
+ * @param[in] ddca_dref display reference
+ *
+ * @remark
+ * User supplied feature definition files are not yet publicly supported.
+ *
+ * @since 0.9.3
+ */
+DDCA_Status
+ddca_dfr_check_by_dref(DDCA_Display_Ref ddca_dref);
+
+/**
+ * Loads any user supplied feature definition files for the specified
+ * display.  Does nothing if they have already been loaded.
+ *
+ * @param[in] ddca_dh display handle
+ *
+ * @remark
+ * User supplied feature definition files are not yet publicly supported.
+ * @since 0.9.3
+ */
+DDCA_Status
+ddca_dfr_check_by_dh(DDCA_Display_Handle ddca_dh);
+
+/**
+ * Gets metadata for a VCP feature.
  *
  * Note that VCP characteristics (C vs NC, RW vs RO, etc) can vary by MCCS version.
  *
  * @param[in]  vspec            VCP version
  * @param[in]  feature_code     VCP feature code
  * @param[in]  create_default_if_not_found
- * @param[out] info             caller buffer to fill in
+ * @param[out] meta_loc         return pointer to metadata here
  * @return     status code
  * @retval     DDCRC_ARG        invalid display handle
  * @retval     DDCRC_UNKNOWN_FEATURE unrecognized feature code and
  *                              !create_default_if_not_found
  *
+ *  It is the responsibility of the caller to free the returned DDCA_Feature_Metadata instance.
+ *
  * @remark
  * Only takes into account VCP version.  Useful for reporting display agnostic
  * feature information.  For display sensitive feature information, i.e. taking
  * into account the specific monitor model, use #ddca_get_feature_metdata_by_dref().
  *
- * @since 0.9.0
+ * @since 0.9.3
  */
 DDCA_Status
 ddca_get_feature_metadata_by_vspec(
       DDCA_Vcp_Feature_Code       feature_code,
       DDCA_MCCS_Version_Spec      vspec,
       bool                        create_default_if_not_found,
-      DDCA_Feature_Metadata *     info); //    change to **?
-
+      DDCA_Feature_Metadata **    meta_loc);
 
 /**
- * Gets information for a VCP feature.
+ * Gets metadata for a VCP feature.
  *
  * Note that VCP characteristics (C vs NC, RW vs RO, etc) can vary by MCCS version.
  *
  * @param[in]  ddca_dref        display reference
  * @param[in]  feature_code     VCP feature code
  * @param[in]  create_default_if_not_found
- * @param[out] info             caller buffer to fill in
+ * @param[out] meta_loc         return pointer to metadata here
  * @return     status code
  * @retval     DDCRC_ARG        invalid display reference
  * @retval     DDCRC_UNKNOWN_FEATURE unrecognized feature code and
  *                              !create_default_if_not_found
  *
- * @since 0.9.0
+ * It is the responsibility of the caller to free the returned DDCA_Feature_Metadata instance.
+ * @since 0.9.3
  */
 DDCA_Status
 ddca_get_feature_metadata_by_dref(
       DDCA_Vcp_Feature_Code       feature_code,
       DDCA_Display_Ref            ddca_dref,
       bool                        create_default_if_not_found,
-      DDCA_Feature_Metadata *     info);
-
+      DDCA_Feature_Metadata **    meta_loc);
 
 /**
- * Gets information for a VCP feature.
+ * Gets metadata for a VCP feature.
  *
  * Note that VCP characteristics (C vs NC, RW vs RO, etc) can vary by MCCS version.
  *
  * @param[in]  ddca_dh          display handle
  * @param[in]  feature_code     VCP feature code
  * @param[in]  create_default_if_not_found
- * @param[out] info             caller buffer to fill in
+ * @param[out] meta_loc         return pointer to metadata here
  * @return     status code
  * @retval     DDCRC_ARG        invalid display handle
  * @retval     DDCRC_UNKNOWN_FEATURE unrecognized feature code and
  *                              !create_default_if_not_found
  *
- * @since 0.9.0
+ * It is the responsibility of the caller to free the returned DDCA_Feature_Metadata instance.
+ * @since 0.9.3
  */
 DDCA_Status
 ddca_get_feature_metadata_by_dh(
       DDCA_Vcp_Feature_Code       feature_code,
       DDCA_Display_Handle         ddca_dh,
       bool                        create_default_if_not_found,
-      DDCA_Feature_Metadata *     info);
-
+      DDCA_Feature_Metadata **    meta_loc);
 
 /**
- *  Frees the contents a #DDCA_Feature_Metadata instance.
+ *  Frees a #DDCA_Feature_Metadata instance.
  *
- *  Note that #DDCA_Feature_Metadata instances are typically
- *  allocated on the stack,  This function frees any data
- *  pointed to by the metadata instance, not the instance itself.
+ *  @param[in] metadata pointer to instance
+ *  @retval   0  normal
+ *  @since 0.9.3
  *
- *  @param[in] info  metadata instance, passed by value
- *  @return    status code
- *
- *  @since 0.9.0
+ *  @remark
+ *  It is not an error if the ***metadata*** pointer argument is NULL
  */
-DDCA_Status
-ddca_free_feature_metadata_contents(DDCA_Feature_Metadata info);
-
-
-// Granular functions for metadata
-
-
-// Current functions - Feature name
+void
+ddca_free_feature_metadata(DDCA_Feature_Metadata * metadata);
 
 /** Gets the VCP feature name.  If different MCCS versions use different names
  *  for the feature, this function makes a best guess.
  *
  * @param[in]  feature_code feature code
  * @return     pointer to feature name (do not free), NULL if unknown feature code
+ *
+ * @remark
+ * Since no specific display is indicated, this function ignores user defined
+ * monitor feature information.
  */
-char *
+const char *
 ddca_get_feature_name(DDCA_Vcp_Feature_Code feature_code);
 
 /** Gets the VCP feature name, which may vary by MCCS version and monitor model.
  *
  * @param[in]  feature_code  feature code
  * @param[in]  dref          display reference
- * @return     pointer to feature name (do not free), NULL if unknown feature code
+ * @param[out] name_loc      where to return pointer to feature name (do not free)
+ * @return     status code
  *
- * @since 0.9.0
+ * @since 0.9.2
  */
+__attribute__ ((deprecated ("use ddca_get_feature_metadata_by_dref()")))
 DDCA_Status
-ddca_feature_name_by_dref(
+ddca_get_feature_name_by_dref(
       DDCA_Vcp_Feature_Code  feature_code,
       DDCA_Display_Ref       dref,
       char **                name_loc);
 
-
-// Current functions - Feature characteristics
-
-
-// Current functions - NC lookup tables
-
-/** Convenience function that searches a #DDCA_Feature_Value_Table for a
+/** Convenience function that searches a Feature Value Table for a
  *  value and returns the corresponding name.
- *  @param[in]   feature_value_table pointer to first entry of table
- *  @param[in]   feature_value value to search for
- *  @param[out]  where to return pointer to name
+ *  @param[in]   feature_value_table  pointer to first entry of table
+ *  @param[in]   feature_value        value to search for
+ *  @param[out]  value_name_loc       where to return pointer to name
  *  @retval      DDCRC_OK  value found
  *  @retval      DDCRC_NOT_FOUND  value not found
  *
  * @remark
- * The value returned in **value_name_Loc** is a pointer into internal
- * ddcutil data structures.  Do not free.
+ * The value returned in **value_name_loc** is a pointer into the table
+ * data structure.  Do not free.
  */
 DDCA_Status
 ddca_get_simple_nc_feature_value_name_by_table(
-      DDCA_Feature_Value_Table    feature_value_table,
+      DDCA_Feature_Value_Entry *  feature_value_table,
       uint8_t                     feature_value,
       char**                      value_name_loc);
 
 
-
-// /** \deprecated */
-DDCA_Status
-ddca_get_simple_nc_feature_value_name_by_display(
-      DDCA_Display_Handle    ddca_dh,    // needed because value lookup mccs version dependent
-      DDCA_Vcp_Feature_Code  feature_code,
-      uint8_t                feature_value,
-      char**                 feature_name_loc);
-
-//
 //
 //  Miscellaneous Monitor Specific Functions
 //
 
-// /** \deprecated */
-__attribute__ ((deprecated))
-DDCA_Status
-ddca_get_edid_by_dref(
-      DDCA_Display_Ref ddca_dref,
-      uint8_t **       pbytes_loc);   // pointer into ddcutil data structures, do not free
-
-
 /** Shows information about a display, specified by a #Display_Ref
  *
  * Output is written using report functions
  *
- * \param dref   pointer to display reference
- * \param depth  logical indentation depth
+ * \param dref       pointer to display reference
+ * \param depth      logical indentation depth
  * \retval DDCRC_ARG invalid display ref
  * \retval 0         success
  *
@@ -1075,7 +1120,6 @@ ddca_get_edid_by_dref(
 DDCA_Status
 ddca_report_display_by_dref(DDCA_Display_Ref dref, int depth);
 
-
 //
 // Feature Lists
 //
@@ -1096,7 +1140,6 @@ const char *
 ddca_feature_list_id_name(
       DDCA_Feature_Subset_Id  feature_set_id);
 
-
 /** Given a feature set id, returns a #DDCA_Feature_List specifying all the
  *  feature codes in the set.
  *
@@ -1104,6 +1147,8 @@ ddca_feature_list_id_name(
  *  @param[in]  dref                   display reference
  *  @param[in]  include_table_features if true, Table type features are included
  *  @param[out] points to feature list to be filled in
+ *  @retval     DDCRC_ARG  invalid display reference
+ *  @retval     DDCRC_OK   success
  *
  *  @since 0.9.0
  */
@@ -1114,15 +1159,14 @@ ddca_get_feature_list_by_dref(
       bool                    include_table_features,
       DDCA_Feature_List*      p_feature_list);
 
-
-
 /** Empties a #DDCA_Feature_List
  *
  *  @param[in]  vcplist pointer to feature list
  *
  *  @since 0.9.0
  */
-void ddca_feature_list_clear(
+void
+ddca_feature_list_clear(
       DDCA_Feature_List* vcplist);
 
 /** Adds a feature code to a #DDCA_Feature_List
@@ -1166,7 +1210,6 @@ ddca_feature_list_or(
       DDCA_Feature_List* vcplist1,
       DDCA_Feature_List * vcplist2);
 
-
 /** Creates the intersection of 2 feature lists.
  *
  *  @param[in] vcplist1   pointer to first feature list
@@ -1183,7 +1226,6 @@ ddca_feature_list_and(
       DDCA_Feature_List* vcplist1,
       DDCA_Feature_List * vcplist2);
 
-
 /** Returns a feature list consisting of all the features in the
  *  first list that are not in the second.
  *
@@ -1201,7 +1243,6 @@ ddca_feature_list_and_not(
       DDCA_Feature_List* vcplist1,
       DDCA_Feature_List* vcplist2);
 
-
 /** Returns the number of features in a feature list
  *
  *  @param[in] feature_list   pointer to feature list
@@ -1228,13 +1269,16 @@ ddca_feature_list_count(
  *  Returns NULL if feature_list == NULL
  *  @since 0.9.0
  */
-char *
+const char *
 ddca_feature_list_string(
       DDCA_Feature_List * feature_list,
       char * value_prefix,
       char * sepstr);
 
 
+//
+// GET AND SET VCP VALUES
+//
 
 /*
  * The API for getting and setting VCP values is doubly specified,
@@ -1277,7 +1321,6 @@ void
 ddca_free_any_vcp_value(
       DDCA_Any_Vcp_Value * valrec);
 
-
 /** Produces a debugging report of a #DDCA_Any_Vcp_Value instance.
  *  The report is written to the current FOUT device.
  *  @param  valrec  instance to report
@@ -1302,7 +1345,11 @@ dbgrpt_any_vcp_value(
  *                           which will be filled in
  * @return status code
  *
- * @remark renamed from **ddca_get_nontable_vcp_value()**
+ * @remark
+ * If the returned status code is other than **DDCRC_OK**, a detailed
+ * error report can be obtained using #ddca_get_error_detail()
+ * @remark
+ * Renamed from **ddca_get_nontable_vcp_value()**
  * @since 0.9.0
  */
 DDCA_Status
@@ -1311,7 +1358,6 @@ ddca_get_non_table_vcp_value(
        DDCA_Vcp_Feature_Code      feature_code,
        DDCA_Non_Table_Vcp_Value*  valrec);
 
-
 /** Gets the value of a table VCP feature.
  *
  * @param[in]  ddca_dh         display handle
@@ -1319,6 +1365,9 @@ ddca_get_non_table_vcp_value(
  * @param[out] table_value_loc address at which to return the value
  * @return status code
  *
+ * @remark
+ * If the returned status code is other than **DDCRC_OK**, a detailed
+ * error report can be obtained using #ddca_get_error_detail()
  * @note
  * Implemented, but untested
  */
@@ -1328,7 +1377,6 @@ ddca_get_table_vcp_value(
        DDCA_Vcp_Feature_Code   feature_code,
        DDCA_Table_Vcp_Value ** table_value_loc);
 
-
 /** Gets the value of a VCP feature of any type.
  *
  * @param[in]  ddca_dh       display handle
@@ -1339,6 +1387,9 @@ ddca_get_table_vcp_value(
  * @return status code
  *
  * @remark
+ * If the returned status code is other than **DDCRC_OK**, a detailed
+ * error report can be obtained using #ddca_get_error_detail()
+ * @remark
  * Replaces **ddca_get_any_vcp_value()
  *
  * @since 0.9.0
@@ -1350,7 +1401,6 @@ ddca_get_any_vcp_value_using_explicit_type(
        DDCA_Vcp_Value_Type         value_type,
        DDCA_Any_Vcp_Value **       valrec_loc);
 
-
 /** Gets the value of a VCP feature of any type.
  *  The type is determined by using ddcutil's internal
  *  feature description table.
@@ -1369,6 +1419,9 @@ ddca_get_any_vcp_value_using_explicit_type(
  * @remark
  * It an error to call this function for a manufacturer-specific feature or
  * an unrecognized feature.
+ * @remark
+ * If the returned status code is other than **DDCRC_OK**, a detailed
+ * error report can be obtained using #ddca_get_error_detail()
  */
 DDCA_Status
 ddca_get_any_vcp_value_using_implicit_type(
@@ -1376,7 +1429,6 @@ ddca_get_any_vcp_value_using_implicit_type(
        DDCA_Vcp_Feature_Code       feature_code,
        DDCA_Any_Vcp_Value **       valrec_loc);
 
-
 /** Returns a string containing a formatted representation of the VCP value
  *  of a feature.  It is the responsibility of the caller to free this value.
  *
@@ -1385,15 +1437,15 @@ ddca_get_any_vcp_value_using_implicit_type(
  *  @param[out] formatted_value_loc Address at which to return the formatted value
  *  @return     status code, 0 if success
  *  @since 0.9.0
+ *  @deprecated Does not support user-supplied feature definitions
  */
+__attribute__ ((deprecated ("Does not support user-supplied feature definitions")))
 DDCA_Status
 ddca_get_formatted_vcp_value(
        DDCA_Display_Handle     ddca_dh,
        DDCA_Vcp_Feature_Code   feature_code,
        char**                  formatted_value_loc);
 
-
-
 /** Returns a formatted representation of a table VCP value.
  *  It is the responsibility of the caller to free the returned string.
  *
@@ -1411,8 +1463,6 @@ ddca_format_table_vcp_value_by_dref(
       DDCA_Table_Vcp_Value *  table_value,
       char **                 formatted_value_loc);
 
-
-
 /** Returns a formatted representation of a non-table VCP value.
  *  It is the responsibility of the caller to free the returned string.
  *
@@ -1430,7 +1480,6 @@ ddca_format_non_table_vcp_value_by_dref(
       DDCA_Non_Table_Vcp_Value *  valrec,
       char **                     formatted_value_loc);
 
-
 /** Returns a formatted representation of a VCP value of any type
  *  It is the responsibility of the caller to free the returned string.
  *
@@ -1469,21 +1518,6 @@ ddca_set_non_table_vcp_value(
       uint8_t                  lo_byte
      );
 
-/** @deprecated Use #ddca_set_non_table_vcp_value() */
-__attribute__ ((deprecated))
-DDCA_Status
-ddca_set_continuous_vcp_value(
-      DDCA_Display_Handle      ddca_dh,
-      DDCA_Vcp_Feature_Code    feature_code,
-      uint16_t                 new_value);
-
-__attribute__ ((deprecated))
-DDCA_Status
-ddca_set_simple_nc_vcp_value(
-      DDCA_Display_Handle      ddca_dh,
-      DDCA_Vcp_Feature_Code    feature_code,
-      uint8_t                  new_value);
-
 /** Sets a Table VCP value.
  *
  *  \param[in]   ddca_dh             display handle
@@ -1504,7 +1538,7 @@ ddca_set_table_vcp_value(
  *  \param[in]   feature_code   feature code
  *  \param[in]   new_value      value to set
  *  \return      status code
- *  \since 0,9.0
+ *  \since 0.9.0
  */
 DDCA_Status
 ddca_set_any_vcp_value(
