@@ -1,26 +1,3 @@
-/* ddcutil_types.h
- *
- * <copyright>
- * Copyright (C) 2014-2017 Sanford Rockowitz <rockowitz@minsoft.com>
- *
- * Licensed under the GNU General Public License Version 2
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- * </endcopyright>
- */
-
 /** @file ddcutil_types.h
  *  @brief File ddcutil_types.h contains type declarations for the C API.
  *
@@ -30,12 +7,14 @@
  *  within the code that implements the API.
  */
 
+// Copyright (C) 2014-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// SPDX-License-Identifier: GPL-2.0-or-later
+
 #ifndef DDCUTIL_TYPES_H_
 #define DDCUTIL_TYPES_H_
 
 /** \cond */
-// #include <linux/limits.h>
-// #include <stdbool.h>            // used only in obsolete DDCA_Global_Failure_Information
+#include <stdbool.h>
 #include <stdint.h>                // for uint8_t, unit16_t
 /** \endcond */
 
@@ -75,41 +54,49 @@ typedef int DDCA_Status;
 //! ddcutil version
 //!
 typedef struct {
-   uint8_t    major;
-   uint8_t    minor;
-   uint8_t    micro;
+   uint8_t    major;          ///< Major release number
+   uint8_t    minor;          ///< Minor release number
+   uint8_t    micro;          ///< Micro release number
 } DDCA_Ddcutil_Version_Spec;
 
+//! Build option flags, as returned by #ddca_build_options()
+//! The enum values are defined as 1,2,4 etc so that they can be or'd.
+typedef enum {
+   /** @brief ddcutil was built with support for AMD Display Library connected monitors */
+   DDCA_BUILT_WITH_ADL     = 0x01,
+   /** @brief ddcutil was built with support for USB connected monitors */
+   DDCA_BUILT_WITH_USB     = 0x02,
+  /** @brief ddcutil was built with support for failure simulation */
+   DDCA_BUILT_WITH_FAILSIM = 0x04
+} DDCA_Build_Option_Flags;
+
 
 //
-// Global Settings
+// Error Reporting
 //
 
-#ifdef OBSOLETE
-//! Failure information filled in at the time of a program abort,
-//! before longjmp() is called.
-typedef struct {
-   bool       info_set_fg;
-   char       funcname[64];
-   int        lineno;
-   char       fn[PATH_MAX];
-   int        status;
-} DDCA_Global_Failure_Information;
-#endif
+#define DDCA_ERROR_DETAIL_MARKER "EDTL"
+//! Detailed error report
+typedef struct ddca_error_detail {
+   char                       marker[4];         ///< Always "EDTL"
+   DDCA_Status                status_code;       ///< Error code
+   char *                     detail;            ///< Optional explanation string
+   uint16_t                   cause_ct;          ///< Number of sub-errors
+   struct ddca_error_detail * causes[];          ///< Variable length array of contributing errors
+} DDCA_Error_Detail;
+
 
 //
 // I2C Protocol Control
 //
 
 //! I2C timeout types
-//!
 typedef enum{
    DDCA_TIMEOUT_STANDARD,      /**< Normal retry interval */
    DDCA_TIMEOUT_TABLE_RETRY    /**< Special timeout for Table reads and writes */
 } DDCA_Timeout_Type;
 
 //! I2C retry limit types
-//
 typedef enum{
    DDCA_WRITE_ONLY_TRIES,     /**< Maximum write-only operation tries */
    DDCA_WRITE_READ_TRIES,     /**< Maximum read-write operation tries */
@@ -125,8 +112,7 @@ typedef enum{
 //!
 //! Values assigned to constants allow them to be or'd in bit flags.
 //!
-//!  Values are ascending in order of verbosity
-//!
+//! Values are ascending in order of verbosity
 typedef enum {
    DDCA_OL_TERSE  =0x04,         /**< Brief   output  */
    DDCA_OL_NORMAL =0x08,         /**< Normal  output */
@@ -134,6 +120,31 @@ typedef enum {
 } DDCA_Output_Level;
 
 
+//
+// Tracing
+//
+
+//! Trace Control
+//!
+//! Used as bitflags to specify multiple trace types
+typedef enum {
+ DDCA_TRC_BASE = 0x0080,        /**< base functions          */
+ DDCA_TRC_I2C  = 0x0040,        /**< I2C layer               */
+ DDCA_TRC_ADL  = 0x0020,        /**< ADL layer               */
+ DDCA_TRC_DDC  = 0x0010,        /**< DDC layer               */
+ DDCA_TRC_USB  = 0x0008,        /**< USB connected display functions */
+ DDCA_TRC_TOP  = 0x0004,        /**< ddcutil mainline        */
+ DDCA_TRC_ENV  = 0x0002,        /**< environment command     */
+ DDCA_TRC_API  = 0x0001,        /**< top level API functions */
+ DDCA_TRC_UDF  = 0x0100,        /**< user-defined, aka dynamic, features */
+ DDCA_TRC_VCP  = 0x0200,        /**< VCP layer, feature definitions */
+ DDCA_TRC_DDCIO = 0x0400,       /**< DDC IO functions */
+
+ DDCA_TRC_NONE = 0x0000,        /**< all tracing disabled    */
+ DDCA_TRC_ALL  = 0xffff         /**< all tracing enabled     */
+} DDCA_Trace_Group;
+
+
 //
 // Performance statistics
 //
@@ -145,11 +156,25 @@ typedef enum {
    DDCA_STATS_TRIES    = 0x01,    ///< retry statistics
    DDCA_STATS_ERRORS   = 0x02,    ///< error statistics
    DDCA_STATS_CALLS    = 0x04,    ///< system calls
-   DDCA_STATS_ELAPSED  = 0x08,     ///< total elapsed time
+   DDCA_STATS_ELAPSED  = 0x08,    ///< total elapsed time
    DDCA_STATS_ALL      = 0xFF     ///< indicates all statistics types
 } DDCA_Stats_Type;
 
 
+//
+// Output capture
+//
+
+//! Capture option flags, used by #ddca_start_capture()
+//!
+//! The enum values are defined as 1,2,4 etc so that they can be or'd.
+//!
+//!  @since 0.9.0
+typedef enum {
+   DDCA_CAPTURE_NOOPTS     = 0,     ///< @brief no options specified
+   DDCA_CAPTURE_STDERR     = 1      ///< @brief capture **stderr** as well as **stdout**
+} DDCA_Capture_Option_Flags;
+
 
 //
 //  Display Specification
@@ -157,51 +182,16 @@ typedef enum {
 
 /** \defgroup api_display_spec API Display Specification */
 
-
-/** @name Display Specification
-
-Monitors are referenced in 3 different ways depending on contexts:
-
-1) A #DDCA_Display_Identifier contains criteria for selecting a monitor,
-typically as entered by a user.
-These may directly identify a monitor (e.g. by I2C bus number), or entail a
-search (e.g. EDID).
-
-Resolving a #DDCA_Display_Identifier resolves to a #DDCA_Display_Ref.
-
-2) #DDCA_Display_Ref indicates the operating system path to a display.
-It can be an I2C identifier,an ADL identifier, or a USB identifier.
-
-For Display_Identifiers containing an I2C bus number or ADL adapter.display numbers,
-the translation from DDCA_Display_Identifier to DDCA_Display_Ref is direct.  
-Otherwise, a search of detected monitors must be performed.
-
-Opening a #DDCA_Display_Ref results in a #DDCA_Display_Handle.
-
-3) A #DDCA_Display_Handle references a display that has been "opened".  This is used
-for most function calls performing an operation on a display.
-
-For I2C and USB connected displays, an operating system open is performed when
-creating DDCA_Display_Handle from a DDCA_Display_Ref.
-DDCA_Display_Handle then contains the open file handle.
-
-For ADL displays, no actual open is performed when creating a DDCA_Display_Handle from
-a DDCA_Display_Ref.  The adapter number.device number pair are simply copied.
-
-\ingroup api_display_spec
-*/
-///@{
-
 /** Opaque display identifier
  *
- * A **DDCA_Display_Identifier** holds the criteria for selecting a display,
+ * A **DDCA_Display_Identifier** holds the criteria for selecting a monitor,
  * typically as specified by the user.
  *
  * It can take several forms:
  * - the display number assigned by **dccutil**
  * - an I2C bus number
  * - an ADL (adapter index, display index) pair
- * - a  USB (bus number, device number) pair
+ * - a  USB (bus number, device number) pair or USB device number
  * - an EDID
  * - manufacturer, model, and serial number strings
  *
@@ -211,11 +201,19 @@ typedef void * DDCA_Display_Identifier;
 
 /** Opaque display reference.
  *
- * A **DDCA_Display_Ref** references a display using the identifiers by which it is accessed
- * in Linux.  It takes one of three forms:
- * - an I2C bus number
- * - an ADL (adapter index, display index) pair
- * - a  USB (bus number, device number pair)
+ * A #DDCA_Display_Ref describes a monitor.  It contains 3 kinds of information:
+ * - Assigned ddcutil display number
+ * - The operating system path to the monitor, which is an I2C bus number, an
+ *   ADL identifier, or a USB device number.
+ * - Accumulated information about the monitor, such as the EDID or capabilities string.
+ *
+ * @remark
+ * When libddcutil is started, it detects all connected monitors and creates
+ * a persistent #DDCA_DisplayRef for each.
+ * @remark
+ * A #DDCA_Display_Ref can be obtained in 2 ways:
+ * - From the DDCA_Display_List returned by #ddca_get_display_info_list2()
+ * - Searching based on #DDCA_Display_Identifier using #ddca_get_display_ref()
  *
  * \ingroup api_display_spec
  */
@@ -224,7 +222,16 @@ typedef void * DDCA_Display_Ref;
 
 /** Opaque display handle
  *
- * A **DDCA_Display_Handle** represents an open display on which actions can be performed.
+ * A **DDCA_Display_Handle** represents an "open" display on which actions can be
+ * performed. This is required for communicating with a display. It is obtained by
+ * calling #ddca_open_display2().
+ *
+ * For I2C and USB connected displays, an operating system open is performed by
+ * # ddca_open_display2().  #DDCA_Display_Handle then contains the file handle
+ * returned by the operating system.
+ * For ADL displays, no actual operating system open is performed when creating
+ * a DDCA_Display_Handle.  The adapter number.device number pair are simply copied
+ * from the #DDCA_Display_Ref.
  *
  * \ingroup api_display_spec
  */
@@ -241,15 +248,65 @@ typedef struct {
 // uses -1,-1 for unset
 
 
+//
+// VCP Feature Information
+//
+
+/** MCCS Version in binary form */
+typedef struct {
+   uint8_t    major;           /**< major version number */
+   uint8_t    minor;           /*** minor version number */
+} DDCA_MCCS_Version_Spec;
+
+extern const DDCA_MCCS_Version_Spec DDCA_VSPEC_V10;        ///< MCCS version 1.0
+extern const DDCA_MCCS_Version_Spec DDCA_VSPEC_V20;        ///< MCCS version 2.0
+extern const DDCA_MCCS_Version_Spec DDCA_VSPEC_V21;        ///< MCCS version 2.1
+extern const DDCA_MCCS_Version_Spec DDCA_VSPEC_V30;        ///< MCCS version 3.0
+extern const DDCA_MCCS_Version_Spec DDCA_VSPEC_V22;        ///< MCCS version 2.2
+extern const DDCA_MCCS_Version_Spec DDCA_VSPEC_ANY;        ///< used as query specifier
+extern const DDCA_MCCS_Version_Spec DDCA_VSPEC_UNKNOWN;    ///< value for monitor has been queried unsuccessfully
+extern const DDCA_MCCS_Version_Spec DDCA_VSPEC_UNQUERIED;  ///< indicates version not queried
+
+
+/** MCCS VCP Feature Id */
+typedef uint8_t DDCA_Vcp_Feature_Code;
+
+
+/** Bitfield specifying a collection of VCP feature codes
+ *
+ *  @remark
+ *  This struct might be more appropriately named DDCA_Feature_Set, but
+ *  that results in confusing function names such as ddca_feature_set_set()
+ */
+typedef struct {
+   uint8_t bytes[32];
+} DDCA_Feature_List;
+
+
+/** Identifiers for publicly useful VCP feature subsets
+ *
+ * @remark
+ * These subset identifiers represent a subset of the much
+ * larger collection of subset ids used internally.
+ */
+typedef enum {
+   DDCA_SUBSET_UNSET = 0,      ///< No subset selected
+   DDCA_SUBSET_KNOWN,          ///< All features defined in a MCCS spec
+   DDCA_SUBSET_COLOR,          ///< Color related features
+   DDCA_SUBSET_PROFILE,        ///< Features saved and restored by loadvcp/setvcp
+   DDCA_SUBSET_MFG             ///< Feature codes reserved for manufacturer use (0x0e..0xff)
+} DDCA_Feature_Subset_Id;
+
+
 //
 // Display Information
 //
 
-/** Indicates how a display is accessed */
+/** Indicates how MCCS communication is performed */
 typedef enum {
-   DDCA_IO_DEVI2C,     /**< Use DDC to communicate with a /dev/i2c-n device */
-   DDCA_IO_ADL,        /**< Use ADL API */
-   DDCA_IO_USB         /**< Use USB reports for a USB connected monitor */
+   DDCA_IO_I2C,     /**< Use DDC to communicate with a /dev/i2c-n device */
+   DDCA_IO_ADL,     /**< Use ADL API */
+   DDCA_IO_USB      /**< Use USB reports for a USB connected monitor */
 } DDCA_IO_Mode;
 
 
@@ -264,20 +321,30 @@ typedef struct {
 } DDCA_IO_Path;
 
 
+// Maximum length of strings extracted from EDID, plus 1 for trailing NULL
+#define DDCA_EDID_MFG_ID_FIELD_SIZE 4
+#define DDCA_EDID_MODEL_NAME_FIELD_SIZE 14
+#define DDCA_EDID_SN_ASCII_FIELD_SIZE 14
+
+
 #define DDCA_DISPLAY_INFO_MARKER "DDIN"
 /** Describes one monitor detected by ddcutil. */
 typedef struct {
-   char              marker[4];        ///< always "DDIN"
-   int               dispno;           ///< ddcutil assigned display number
-   DDCA_IO_Path      path;             ///< physical access path to display
-   int               usb_bus;          ///< USB bus number, if USB connection
-   int               usb_device;       ///< USB device number, if USB connection
-   // or should these be actual character/byte arrays instead of pointers?
-   const char *      mfg_id;          ///< 3 character manufacturer id, from EDID
-   const char *      model_name;      ///< model name, from EDID
-   const char *      sn;              ///< ASCII serial number string from EDID
-   const uint8_t *   edid_bytes;      ///< raw bytes (128) of first EDID block
-   DDCA_Display_Ref  dref;            ///< opaque display reference
+   char                   marker[4];        ///< always "DDIN"
+   int                    dispno;           ///< ddcutil assigned display number
+   DDCA_IO_Path           path;             ///< physical access path to display
+   int                    usb_bus;          ///< USB bus number, if USB connection
+   int                    usb_device;       ///< USB device number, if USB connection
+   char                   mfg_id[    DDCA_EDID_MFG_ID_FIELD_SIZE    ]; ///< 3 character mfg id from EDID
+   char                   model_name[DDCA_EDID_MODEL_NAME_FIELD_SIZE]; ///< model name from EDID, 13 char max
+   char                   sn[        DDCA_EDID_SN_ASCII_FIELD_SIZE  ]; ///< "serial number" from EDID, 13 char max
+   uint16_t               product_code;     ///< model product number
+   uint8_t                edid_bytes[128];  ///< first 128 bytes of EDID
+   DDCA_MCCS_Version_Spec vcp_version;      ///< VCP version as pair of numbers
+#ifdef OLD
+   DDCA_MCCS_Version_Id   vcp_version_id;   ///< VCP version identifier (deprecated)
+#endif
+   DDCA_Display_Ref       dref;             ///< opaque display reference
 } DDCA_Display_Info;
 
 
@@ -288,49 +355,6 @@ typedef struct {
 } DDCA_Display_Info_List;
 
 
-
-//
-// VCP Feature Information
-//
-
-// Both DDCA_MCCS_Version_Spec and DDCA_MCCS_Version_Id exist for historical reasons.
-// DDCA_MCCS_Version_Spec is the form in which the version number is returned from a
-// GETVCP of feature xDF.  This form is used throughout much of ddcutil.
-// DDCA_MCCS_Version_Id reflects the fact that there are a small number of versions
-// and simplifies program logic that varies among versions.
-
-/** MCCS Version in binary form */
-typedef struct {
-   uint8_t    major;           /**< major version number */
-   uint8_t    minor;           /*** minor version number */
-} DDCA_MCCS_Version_Spec;
-
-
-/** @name version_id
- *  Ids for MCCS/VCP versions, reflecting the fact that
- *  there is a smaill set of valid version values.
- */
-///@{
-
-// in sync w constants MCCS_V.. in vcp_feature_codes.c
-/** MCCS (VCP) Feature Version IDs */
-typedef enum {
-   DDCA_VNONE =  0,     /**< As query, match any MCCS version, as response, version unknown */
-   DDCA_V10   =  1,     /**< MCCS v1.0 */
-   DDCA_V20   =  2,     /**< MCCS v2.0 */
-   DDCA_V21   =  4,     /**< MCCS v2.1 */
-   DDCA_V30   =  8,     /**< MCCS v3.0 */
-   DDCA_V22   = 16      /**< MCCS v2.2 */
-} DDCA_MCCS_Version_Id;
-
-#define DDCA_VANY  DDCA_VNONE    /**< For use on queries,   indicates match any version */
-#define DDCA_VUNK  DDCA_VNONE    /**< For use on responses, indicates version unknown   */
-
-///@}
-
-/** MCCS VCP Feature Id */
-typedef uint8_t DDCA_Vcp_Feature_Code;
-
 /** @name Version Feature Flags
  *
  * #DDCA_Version_Feature_Flags is a byte of flags describing attributes of a
@@ -358,11 +382,12 @@ typedef uint16_t DDCA_Version_Feature_Flags;
 #define DDCA_WRITABLE     (DDCA_WO | DDCA_RW)  /**< Feature is either RW or WO */
 
 // Further refine the C/NC/TABLE categorization of the MCCS spec
-// Exactly 1 of the following 7 bits is set
+// Exactly 1 of the following 8 bits is set
 #define DDCA_STD_CONT     0x0080       /**< Normal continuous feature */
 #define DDCA_COMPLEX_CONT 0x0040       /**< Continuous feature with special interpretation */
 #define DDCA_SIMPLE_NC    0x0020       /**< Non-continuous feature, having a defined list of values in byte SL */
 #define DDCA_COMPLEX_NC   0x0010       /**< Non-continuous feature, having a complex interpretation using one or more of SL, SH, ML, MH */
+#define DDCA_NC_CONT      0x0800       /**< NC feature combining reserved values with continuous range */
 // For WO NC features.  There's no interpretation function or lookup table
 // Used to mark that the feature is defined for a version
 #define DDCA_WO_NC        0x0008       /**< Used internally for write-only non-continuous features */
@@ -370,7 +395,7 @@ typedef uint16_t DDCA_Version_Feature_Flags;
 #define DDCA_WO_TABLE     0x0002       /**< Write only table feature */
 
 #define DDCA_CONT         (DDCA_STD_CONT|DDCA_COMPLEX_CONT)            /**< Continuous feature, of any subtype */
-#define DDCA_NC           (DDCA_SIMPLE_NC|DDCA_COMPLEX_NC|DDCA_WO_NC)  /**< Non-continuous feature of any subtype */
+#define DDCA_NC           (DDCA_SIMPLE_NC|DDCA_COMPLEX_NC|DDCA_WO_NC|DDCA_NC_CONT)  /**< Non-continuous feature of any subtype */
 #define DDCA_NON_TABLE    (DDCA_CONT | DDCA_NC)                        /**< Non-table feature of any type */
 
 #define DDCA_TABLE        (DDCA_NORMAL_TABLE | DDCA_WO_TABLE)        /**< Table type feature, of any subtype */
@@ -384,7 +409,11 @@ typedef uint16_t DDCA_Version_Feature_Flags;
 typedef uint16_t DDCA_Global_Feature_Flags;
 
 // Bits in DDCA_Global_Feature_Flags:
-#define DDCA_SYNTHETIC    0x8000
+#define DDCA_SYNTHETIC_VCP_FEATURE_TABLE_ENTRY  0x8000 /**< Used internally to indicate a temporary VCP_Feature_Table_Entry */
+#define DDCA_USER_DEFINED 0x4000      /**< User provided feature definition */
+// #define DDCA_SYNTHETIC_DDCA_FEATURE_METADATA 0x2000
+#define DDCA_PERSISTENT_METADATA 0x1000  /**< Part of internal feature tables, do not free */
+#define DDCA_SYNTHETIC     0x2000        /**< Generated feature definition  */
 
 typedef uint16_t DDCA_Feature_Flags;    // union (DDCA_Version_Feature_Flags, DDCA_Global_Feature_Flags)
 
@@ -400,25 +429,23 @@ struct {
 } DDCA_Feature_Value_Entry;
 
 // Makes reference to feature value table less implementation specific
-typedef DDCA_Feature_Value_Entry * DDCA_Feature_Value_Table;
+// deprecated
+// typedef DDCA_Feature_Value_Entry * DDCA_Feature_Value_Table;
 
-
-#define VCP_VERSION_SPECIFIC_FEATURE_INFO_MARKER "VSFI"
-/** Describes a VCP feature code, tailored for a specific VCP version */
+#define DDCA_FEATURE_METADATA_MARKER  "FMET"
+/** Describes a VCP feature code, tailored for a specific monitor.
+ *  Feature metadata can vary by VCP version and user defined features */
 typedef
 struct {
-   char                                  marker[4];      /**< equals VCP_VERSION_SPECIFIC_FEATURE_INFO_MARKER */
+   char                                  marker[4];      /**< always "FMET" */
    DDCA_Vcp_Feature_Code                 feature_code;   /**< VCP feature code */
-   DDCA_MCCS_Version_Spec                vspec;            // ???
-   DDCA_MCCS_Version_Id                  version_id;       // which ?
-   char *                                desc;           /**< feature description */
-   // Format_Normal_Feature_Detail_Function nontable_formatter;
-   // Format_Table_Feature_Detail_Function  table_formatter;
-    DDCA_Feature_Value_Table             sl_values;     /**< valid when DDCA_SIMPLE_NC set */
-   // VCP_Feature_Subset                 vcp_subsets;   // Need it?
-   char *                                feature_name;  /**< feature name */
-   DDCA_Feature_Flags                    feature_flags;
-} DDCA_Version_Feature_Info;
+// DDCA_MCCS_Version_Spec                vspec;          /**< MCCS version    */
+   DDCA_Feature_Flags                    feature_flags;  /**< feature type description */
+   DDCA_Feature_Value_Entry *            sl_values;      /**< valid when DDCA_SIMPLE_NC set */
+   char *                                feature_name;   /**< feature name */
+   char *                                feature_desc;   /**< feature description */
+   // possibly add pointers to formatting functions
+} DDCA_Feature_Metadata;
 
 
 //
@@ -442,6 +469,8 @@ struct {
    char                                 marker[4];       /**< always DDCA_CAPABILITIES_MARKER */
    char *                               unparsed_string; /**< unparsed capabilities string */
    DDCA_MCCS_Version_Spec               version_spec;    /**< parsed mccs_ver() field */
+   int                                  cmd_ct;          /**< number of command codes */
+   uint8_t *                            cmd_codes;       /**< array of command codes */
    int                                  vcp_code_ct;     /**< number of features in vcp() field */
    DDCA_Cap_Vcp *                       vcp_codes;       /**< array of pointers to structs describing each vcp code */
 } DDCA_Capabilities;
@@ -463,32 +492,22 @@ typedef enum {
 } DDCA_Vcp_Value_Type;
 
 
-/** #DDCA_Vcp_Value_Type_Parm extends #DDCA_Vcp_Value_Type to allow for its use as a
-    function call parameter where the type is unknown */
-typedef enum {
-   DDCA_UNSET_VCP_VALUE_TYPE_PARM = 0,   /**< Unspecified */
-   DDCA_NON_TABLE_VCP_VALUE_PARM  = 1,   /**< Continuous (C) or Non-Continuous (NC) value */
-   DDCA_TABLE_VCP_VALUE_PARM      = 2,   /**< Table (T) value */
-} DDCA_Vcp_Value_Type_Parm;
-
-
 typedef struct {
    uint8_t    mh;
    uint8_t    ml;
    uint8_t    sh;
    uint8_t    sl;
-} DDCA_Non_Table_Value;
+} DDCA_Non_Table_Vcp_Value;
 
 
-
-/** Represents a single table VCP value.   Consists of a count, followed by the bytes */
+/** Represents a single table VCP value.   Consists of a count, and a pointer to the bytes */
 typedef struct {
    uint16_t bytect;        /**< Number of bytes in value */
-   uint8_t  bytes[];       /**< Bytes of the value */
-} DDCA_Table_Value;
-
+   uint8_t*  bytes;        /**< Bytes of the value */
+} DDCA_Table_Vcp_Value;
 
 
+/** Stores a VCP feature value of any type */
 typedef struct {
    DDCA_Vcp_Feature_Code  opcode;         /**< VCP feature code */
    DDCA_Vcp_Value_Type    value_type;      // probably a different type would be better
@@ -509,31 +528,4 @@ typedef struct {
 #define VALREC_CUR_VAL(valrec) ( valrec->val.c_nc.sh << 8 | valrec->val.c_nc.sl )
 #define VALREC_MAX_VAL(valrec) ( valrec->val.c_nc.mh << 8 | valrec->val.c_nc.ml )
 
-
-// values are in sync with CMD_ constants defined in ddc_command_codes.h, unify?
-typedef enum {
-    DDCA_Q_VCP_GET         = 0x01,    // CMD_VCP_REQUEST
-    DDCA_Q_VCP_SET         = 0x03,    // CMD_VCP_SET
-    DDCA_Q_VCP_RESET       = 0x09,    // CMD_VCP_RESET
-    DDCA_Q_SAVE_SETTINGS  =  0x0c,    // CMD_SAVE_SETTINGS
-    DDCA_Q_TABLE_READ     =  0xe2,    // CMD_TABLE_READ_REQUST
-    DDCA_Q_TABLE_WRITE    = -0xe7,    // CMD_TABLE_WRITE
-    DDCA_Q_CAPABILITIES   =  0xf3,    // CMD_CAPABILITIES_REQUEST
-} DDCA_Queued_Request_Type;
-
-
-typedef struct {
-   DDCA_Queued_Request_Type   request_type;
-   DDCA_Vcp_Feature_Code      vcp_code;
-   // for DDCA_Q_SET
-   DDCA_Non_Table_Value       non_table_value;
-} DDCA_Queued_Request;
-
-
-
-/** Callback function to report VCP value change */
-typedef void (*DDCA_Notification_Func)(DDCA_Status psc, DDCA_Any_Vcp_Value* valrec);
-
-typedef int (*Simple_Callback_Func)(int val);
-
 #endif /* DDCUTIL_TYPES_H_ */
