@@ -1,7 +1,7 @@
 /* ddc_vcp_tests.h
  *
  * <copyright>
- * Copyright (C) 2014-2016 Sanford Rockowitz <rockowitz@minsoft.com>
+ * Copyright (C) 2014-2019 Sanford Rockowitz <rockowitz@minsoft.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -48,6 +48,7 @@
 #include "ddc/ddc_packet_io.h"
 #include "ddc/ddc_vcp.h"
 
+#include "test/i2c/i2c_testutil.h"
 #include "test/i2c/i2c_io_old.h"
 
 #include "test/ddc/ddc_vcp_tests.h"
@@ -256,6 +257,7 @@ void probe_get_luminosity(int busno, char * write_mode, char * read_mode) {
 
    file = i2c_open_bus(busno, CALLOPT_ERR_MSG);
    if (file < 0) {
+      free_ddc_packet(request_packet_ptr);
       return;
    }
    rc = i2c_set_addr(file, 0x37, CALLOPT_ERR_MSG);
@@ -293,7 +295,7 @@ void probe_get_luminosity(int busno, char * write_mode, char * read_mode) {
             Parsed_Nontable_Vcp_Response * interpretation_ptr = NULL;
             rc2 = get_interpreted_vcp_code(response_packet_ptr, false, &interpretation_ptr);
             if (rc2 == 0)
-               report_interpreted_nontable_vcp_response(interpretation_ptr, 0);
+               dbgrpt_interpreted_nontable_vcp_response(interpretation_ptr, 0);
             free_ddc_packet(response_packet_ptr);
          }
       } // read_ok
@@ -400,7 +402,7 @@ void get_luminosity_sample_code(int busno) {
               rc2 = get_interpreted_vcp_code(response_packet_ptr, false, &interpretation_ptr);
               if (rc2 == 0) {
                  printf("(%s) interpretation_ptr=%p\n", __func__, interpretation_ptr);
-                 report_interpreted_nontable_vcp_response(interpretation_ptr);
+                 dbgrpt_interpreted_nontable_vcp_response(interpretation_ptr);
               }
               // read_ok = true;
            }
