@@ -1,29 +1,8 @@
-/* usb_displays.c
- *
- * <copyright>
- * Copyright (C) 2014-2018 Sanford Rockowitz <rockowitz@minsoft.com>
- *
- * Licensed under the GNU General Public License Version 2
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- * </endcopyright>
+/** @file usb_displays.c
  */
 
-/** \file
- *
- */
+// Copyright (C) 2014-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// SPDX-License-Identifier: GPL-2.0-or-later
 
 /** \cond */
 #include <assert.h>
@@ -41,6 +20,7 @@
 #include "util/udev_util.h"
 #include "util/udev_usb_util.h"
 
+#include "usb_util/usb_hid_common.h"
 #include "usb_util/hiddev_reports.h"
 #include "usb_util/hiddev_util.h"
 
@@ -56,12 +36,12 @@
 
 
 // Trace class for this file
-static Trace_Group TRACE_GROUP = TRC_USB;  // may be unused if all diagnostics turned off
+static DDCA_Trace_Group TRACE_GROUP = DDCA_TRC_USB;  // may be unused if all diagnostics turned off
 
-// n. #pragma GCC diagnostic ignored "-Wunused-variable" not working
-void usb_core_unused_function_to_avoid_unused_variable_warning() {
-   printf("0x%02x\n",TRACE_GROUP);
-}
+// // n. #pragma GCC diagnostic ignored "-Wunused-variable" not working
+// void usb_core_unused_function_to_avoid_unused_variable_warning() {
+//    printf("0x%02x\n",TRACE_GROUP);
+// }
 
 // // Forward declarations
 // static GPtrArray * get_usb_monitor_list();  // returns array of Usb_Monitor_Info
@@ -308,7 +288,7 @@ static char * usb_synthesize_capabilities_string(Usb_Monitor_Info * moninfo) {
  */
 GPtrArray * get_usb_monitor_list() {
    bool debug = false;
-   DBGMSF0(debug, "Starting...");
+   DBGTRC(debug, TRACE_GROUP, "Starting...");
    DDCA_Output_Level ol = get_output_level();
 
    if (usb_monitors)      // already initialized?
@@ -326,10 +306,10 @@ GPtrArray * get_usb_monitor_list() {
          calloptions |= CALLOPT_ERR_MSG;
       int fd = usb_open_hiddev_device(hiddev_fn, calloptions);
       if (fd < 0 && ol >= DDCA_OL_VERBOSE) {
-         Usb_Detailed_Device_Summary * devsum = lookup_udev_usb_device_by_devname(hiddev_fn);
+         Usb_Detailed_Device_Summary * devsum = lookup_udev_usb_device_by_devname(hiddev_fn, false);
          if (devsum) {
             // report_usb_detailed_device_summary(devsum, 2);
-            f0printf(FERR, "  USB bus %s, device %s, vid:pid: %s:%s - %s:%s\n",
+            f0printf(fout(), "  USB bus %s, device %s, vid:pid: %s:%s - %s:%s\n",
                            devsum->busnum_s,
                            devsum->devnum_s,
                            devsum->vendor_id,
@@ -351,12 +331,16 @@ GPtrArray * get_usb_monitor_list() {
          devinfo = calloc(1,sizeof(struct hiddev_devinfo));
          if ( hiddev_get_device_info(fd, devinfo, CALLOPT_ERR_MSG) != 0 )
             goto close;
+
+         if (deny_hid_monitor_by_vid_pid(devinfo->vendor, devinfo->product) )
+            goto close;
+
          if (!is_hiddev_monitor(fd))
             goto close;
 
          parsed_edid = get_hiddev_edid_with_fallback(fd, devinfo);
          if (!parsed_edid) {
-            f0printf(FERR,
+            f0printf(ferr(),
                     "Monitor on device %s reports no EDID or has invalid EDID. Ignoring.\n",
                     hiddev_fn);
             goto close;
@@ -401,7 +385,7 @@ GPtrArray * get_usb_monitor_list() {
    g_ptr_array_set_free_func(hiddev_names, free);
    g_ptr_array_free(hiddev_names, true);
 
-   if (debug) {
+   if ( debug || IS_TRACING() ) {
       DBGMSG("Returning  %d monitors ", usb_monitors->len);
       // report_usb_monitors(usb_monitors,1);
    }
