@@ -1,31 +1,10 @@
-/* usb_vcp.c
+/* \file usb_vcp.c
  *
  * Get and set VCP feature codes for USB connected monitors.
- *
- * <copyright>
- * Copyright (C) 2017-2018 Sanford Rockowitz <rockowitz@minsoft.com>
- *
- * Licensed under the GNU General Public License Version 2
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- * </endcopyright>
  */
 
-/** \file
- *
- */
+// Copyright (C) 2016-2018 Sanford Rockowitz <rockowitz@minsoft.com>
+// SPDX-License-Identifier: GPL-2.0-or-later
 
 /** \cond */
 #include <assert.h>
@@ -55,7 +34,7 @@
 
 
 // Trace class for this file
-static Trace_Group TRACE_GROUP = TRC_USB;
+static DDCA_Trace_Group TRACE_GROUP = DDCA_TRC_USB;
 
 
 //
@@ -556,7 +535,7 @@ Public_Status_Code usb_get_vcp_value(
        Display_Handle *          dh,
        Byte                      feature_code,
        DDCA_Vcp_Value_Type            call_type,
-       Single_Vcp_Value **       pvalrec)
+       DDCA_Any_Vcp_Value **    pvalrec)
 {
    bool debug = false;
    DBGTRC(debug, TRACE_GROUP, "Starting. Reading feature 0x%02x", feature_code);
@@ -567,7 +546,7 @@ Public_Status_Code usb_get_vcp_value(
    Buffer * buffer = NULL;
 #endif
    Parsed_Nontable_Vcp_Response * parsed_nontable_response = NULL;
-   Single_Vcp_Value * valrec = NULL;
+   DDCA_Any_Vcp_Value * valrec = NULL;
    switch (call_type) {
 
    case (DDCA_NON_TABLE_VCP_VALUE):
@@ -605,7 +584,7 @@ Public_Status_Code usb_get_vcp_value(
 
    DBGTRC(debug, TRACE_GROUP, "Done.  Returning: %s", psc_desc(psc) );
    if (psc == 0 && debug)
-      report_single_vcp_value(valrec,1);
+      dbgrpt_single_vcp_value(valrec,1);
    assert( (psc == 0 && *pvalrec) || (psc != 0 && !*pvalrec) );
    return psc;
 }
@@ -697,11 +676,11 @@ usb_set_nontable_vcp_value(
 Public_Status_Code
 usb_set_vcp_value(                               // changed from set_vcp_value()
       Display_Handle *   dh,
-      Single_Vcp_Value * vrec)
+      DDCA_Any_Vcp_Value * vrec)
 {
    Public_Status_Code psc = 0;
    if (vrec->value_type == DDCA_NON_TABLE_VCP_VALUE) {
-      psc = usb_set_nontable_vcp_value(dh, vrec->opcode, vrec->val.c.cur_val);  // function name changed
+      psc = usb_set_nontable_vcp_value(dh, vrec->opcode, VALREC_CUR_VAL(vrec));  // function name changed
    }
    else {
       assert(vrec->value_type == DDCA_TABLE_VCP_VALUE);
