@@ -1,29 +1,9 @@
-/* hiddev_util.c
- *
- * <copyright>
- * Copyright (C) 2016-2018 Sanford Rockowitz <rockowitz@minsoft.com>
- *
- * Licensed under the GNU General Public License Version 2
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- * </endcopyright>
+/** @file hiddev_util.c
  */
 
-/** \f
- *
- */
+// Copyright (C) 2016-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// SPDX-License-Identifier: GPL-2.0-or-later
+
 
 /** \cond */
 #include <config.h>
@@ -99,7 +79,7 @@ static int is_hiddev(const struct dirent *ent) {
  * Returns:   GPtrArray of device device names.
  */
 GPtrArray * get_hiddev_device_names_using_filesys() {
-   const char *hidraw_paths[] = { "/dev/", "/dev/usb", NULL };
+   const char *hidraw_paths[] = { "/dev/", "/dev/usb/", NULL };
    return get_filenames_by_filter(hidraw_paths, is_hiddev);
 }
 
@@ -170,7 +150,7 @@ bye:
  * Arguments:     none
  * Returns:       array of hiddev path names in /dev
  *
- * Allows for easily switching between alternative implementation.
+ * Allows for easily switching between alternative implementations.
  */
 GPtrArray * get_hiddev_device_names() {
 
@@ -741,7 +721,12 @@ get_multibyte_value_by_uref_multi(
 
    rc = ioctl(fd, HIDIOCGUSAGES, uref_multi);  // Fills in usage value
    if (rc != 0) {
-      REPORT_USB_IOCTL_ERROR("HIDIOCGUSAGES", errno);
+      if (errno == EINVAL) {
+         if (debug)
+            printf("(%s) ioctl(HIDIOCGUSAGES) returned EINVAL\n", __func__);
+      }
+      else
+         REPORT_USB_IOCTL_ERROR("HIDIOCGUSAGES", errno);
       goto bye;
    }
    result = buffer_new(uref_multi->num_values, __func__);
@@ -750,7 +735,7 @@ get_multibyte_value_by_uref_multi(
 
 bye:
    if (debug) {
-      DBGMSG("Returning: %p", __func__, result);
+      DBGMSG("Returning: %p", result);
       if (result) {
          dbgrpt_buffer(result, 1);
       }
@@ -790,7 +775,12 @@ Buffer * hiddev_get_multibyte_report_value_by_hid_field_locator(
 
    int rc = ioctl(fd, HIDIOCGREPORT, &rinfo);
    if (rc != 0) {
-      REPORT_USB_IOCTL_ERROR("HIDIOCGREPORT", errno);
+      if (errno == EINVAL) {
+         if (debug)
+            printf("(%s) ioctl(HIDIOCGUSAGES) returned EINVAL\n", __func__);
+      }
+      else
+         REPORT_USB_IOCTL_ERROR("HIDIOCGREPORT", errno);
       goto bye;
    }
 
