@@ -5,7 +5,7 @@
  * libusb is not currently used by ddcutil.  This code is retained for reference.
  *
  * <copyright>
- * Copyright (C) 2016-2018 Sanford Rockowitz <rockowitz@minsoft.com>
+ * Copyright (C) 2016-2019 Sanford Rockowitz <rockowitz@minsoft.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -211,9 +211,10 @@ char * lookup_libusb_string(struct libusb_device_handle * dh, int string_id) {
    else {
       // 7/2017: assert was triggered by user, why?
       // replace assert by diagnostic message
-      printf("(%s) rc=%d, LIBUSB_STRING_BUFFER_SIZE=%d, strlen=%zu\n",
-            __func__, rc, LIBUSB_STRING_BUFFER_SIZE,    strlen(libusb_string_buffer) );
-      // assert(rc == strlen(libusb_string_buffer));
+      if (rc != strlen(libusb_string_buffer)) {
+         printf("(%s) rc=%d, LIBUSB_STRING_BUFFER_SIZE=%d, strlen=%zu, value=|%s|\n",
+                __func__, rc, LIBUSB_STRING_BUFFER_SIZE,    strlen(libusb_string_buffer), libusb_string_buffer );
+      }
    }
    return libusb_string_buffer;
 }
@@ -709,7 +710,7 @@ void report_libusb_config_descriptor(
    rpt_structure_loc("libusb_config_descriptor", config, depth);
 
    // Size of this descriptor (in bytes): uint8_t  bLength;
-   rpt_vstring(d1, "%-20s  %d", "bLength:", config->bLength, d1);
+   rpt_vstring(d1, "%-20s  %d", "bLength:", config->bLength);
 
 
    // Descriptor type. Will have value LIBUSB_DT_CONFIG in this context.
