@@ -1,37 +1,17 @@
-/* edid.c
+/** edid.c
  *
- * Functions for processing the EDID data structure, irrespective of how
- * the bytes of the EDID are obtained.
+ *  Functions to interpret the EDID data structure, irrespective of how
+ *  the bytes of the EDID are obtained.
  *
- * This should be the only source module that understands the internal
- * structure of the EDID.
+ *  This should be the only source module that understands the internal
+ *  structure of the EDID.
  *
- * <copyright>
- * Copyright (C) 2014-2017 Sanford Rockowitz <rockowitz@minsoft.com>
- *
- * Licensed under the GNU General Public License Version 2
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- * </endcopyright>
+ *  While the code here is generic to all EDIDs, only fields of use
+ *  to **ddcutil** are interpreted.
  */
 
-/** @file edid.c
- * Functions to interpret EDID.
- * While the code here is generic to all EDIDs, only fields of use
- * to **ddcutil** are interpreted.
- */
+// Copyright (C) 2014-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// SPDX-License-Identifier: GPL-2.0-or-later
 
 /** \cond */
 #include <assert.h>
@@ -74,6 +54,26 @@ Byte edid_checksum(Byte * edid) {
    return checksum;
 }
 
+bool is_valid_edid_checksum(Byte * edidbytes) {
+   return (edid_checksum(edidbytes) == 0);
+}
+
+bool is_valid_edid_header(Byte * edidbytes) {
+   bool debug = false;
+   bool result = true;
+
+   const Byte edid_header_tag[] = {0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00};
+   if (memcmp(edidbytes, edid_header_tag, 8) != 0) {
+      if (debug) {
+         char * hs = hexstring(edidbytes,8);
+         printf("(%s) Invalid initial EDID bytes: %s\n", __func__, hs);
+         free(hs);
+      }
+      result = false;
+   }
+   return result;
+}
+
 
 /** Unpacks the 2 byte manufacturer id field from the EDID into a 3 character
  * string.
@@ -213,24 +213,12 @@ static void get_edid_descriptor_strings(
  */
 Parsed_Edid * create_parsed_edid(Byte* edidbytes) {
    assert(edidbytes);
-   bool debug = false;
+   // bool debug = false;
    bool ok = true;
    Parsed_Edid* parsed_edid = NULL;
 
-   const Byte edid_header_tag[] = {0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00};
-   if (memcmp(edidbytes, edid_header_tag, 8) != 0) {
-      char * hs = hexstring(edidbytes,8);
-      if (debug)
-         printf("(%s) Invalid initial EDID bytes: %s\n", __func__, hs);
-      free(hs);
+   if ( !is_valid_edid_header(edidbytes) || !is_valid_edid_checksum(edidbytes) )
       goto bye;
-   }
-
-   if (edid_checksum(edidbytes) != 0x00) {
-      if (debug)
-         printf("(%s) Invalid EDID checksum: 0x%02x\n", __func__, edid_checksum(edidbytes));
-      goto bye;
-   }
 
    parsed_edid = calloc(1,sizeof(Parsed_Edid));
    assert(sizeof(parsed_edid->bytes) == 128);
@@ -242,7 +230,7 @@ Parsed_Edid * create_parsed_edid(Byte* edidbytes) {
            parsed_edid->mfg_id,
            sizeof(parsed_edid->mfg_id) );
 
-   parsed_edid->model_hex = edidbytes[0x0b] << 8 | edidbytes[0x0a];
+   parsed_edid->product_code = edidbytes[0x0b] << 8 | edidbytes[0x0a];
 
    parsed_edid->serial_binary = edidbytes[0x0c]       |
                                 edidbytes[0x0d] <<  8 |
@@ -339,7 +327,8 @@ void report_parsed_edid_base(Parsed_Edid * edid, bool verbose, bool show_raw, in
       rpt_vstring(d1,"EDID version:     %d.%d", edid->edid_version_major, edid->edid_version_minor);
 
       if (verbose) {
-         rpt_vstring(d1,"Product code:     0x%04x (%u)",      edid->model_hex, edid->model_hex);
+      // rpt_vstring(d1,"Product code:     0x%04x (%u)",      edid->product_code, edid->product_code);
+         rpt_vstring(d1,"Product code:     %u",          edid->product_code);
          // useless, binary serial number is typically 0x00000000 or 0x01010101
          // rpt_vstring(d1,"Binary sn:        %u (0x%08x)", edid->serial_binary, edid->serial_binary);
          rpt_vstring(d1,"Extra descriptor: %s",          edid->extra_descriptor_string);
@@ -404,7 +393,7 @@ void report_parsed_edid_base(Parsed_Edid * edid, bool verbose, bool show_raw, in
                break;
             default:
                // should be PROGRAM_LOGIC_ERROR, but that would violdate layering
-               rpt_vstring(d2, "Invalid digital display type: 0x02", display_type);
+               rpt_vstring(d2, "Invalid digital display type: 0x%02x", display_type);
             }
          }
          else {   // analog input
@@ -423,7 +412,7 @@ void report_parsed_edid_base(Parsed_Edid * edid, bool verbose, bool show_raw, in
                break;
             default:
                // should be PROGRAM_LOGIC_ERROR, but that would violdate layering
-               rpt_vstring(d2, "Invalid analog display type: 0x02", display_type);
+               rpt_vstring(d2, "Invalid analog display type: 0x%02x", display_type);
             }
          }
          rpt_vstring(d2, "Standard sRGB color space: %s", (display_type & 0x02) ? "True" : "False");
@@ -466,3 +455,16 @@ void report_parsed_edid_base(Parsed_Edid * edid, bool verbose, bool show_raw, in
 void report_parsed_edid(Parsed_Edid * edid, bool verbose, int depth) {
    report_parsed_edid_base(edid, verbose, verbose, depth);
 }
+
+
+/** Heuristic test for a laptop display.  Observed laptop displays
+ *  never have the model name and serial numbers set.
+ */
+bool is_embedded_parsed_edid(Parsed_Edid * parsed_edid) {
+   assert(parsed_edid);
+   bool result = streq(parsed_edid->model_name,  "Unspecified") &&
+                 streq(parsed_edid->serial_ascii,"Unspecified");
+   return result;
+}
+
+
